using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using GameEngine;

namespace Editor.Data
{
    public enum PolygonType { Physics };

    public class PhysicsPolygon
    {
        private Vector2[] points;
        public Vector2[] Points { get { return points; } }

        public Matrix translation = Matrix.Identity;
        private Effect primitivesEffect;
        private VertexPositionColor[] vertices;
        private VertexDeclaration decl;
        private bool dirtied;

        public Matrix parentMatrix = Matrix.Identity;

        private PolygonType type;
        public PolygonType Type
        {
            get { return type; }
            set { type = value; }
        }

        private bool selected;
        public bool Selected
        {
            get { return selected; }
            set { selected = value; }
        }

        private bool display;

        public bool Display
        {
            get { return display; }
            set { display = value; }
        }

        private string name = "Poly";

        public string Name
        {
            get { return name; }
            set { name = value; }
        }



        public PhysicsPolygon(Vector2[] points)
        {
            this.points = points;
            dirtied = true;
            this.type = PolygonType.Physics;
            this.selected = false;
            this.display = true;
        }

        public void LoadContent(ContentManager content)
        {
            primitivesEffect = content.Load<Effect>("primitives");
            
        }

        public void SetEffect(Effect effect)
        {
            this.primitivesEffect = effect;
        }

        private void UpdateVertices()
        {
            if (dirtied)
            {
                vertices = new VertexPositionColor[points.Length + 1];
                for (int i = 0; i < points.Length; i++)
                {
                    vertices[i].Color = Color.Blue;
                    vertices[i].Position = new Vector3(points[i], 0);
                }

                vertices[points.Length] = vertices[0];
            }
        }

        public void Draw(DrawInfo drawInfo)
        {
            UpdateVertices();
            if (decl == null)
            {
                decl = new VertexDeclaration(drawInfo.GraphicsDevice, VertexPositionColor.VertexElements);
            }

            Matrix testMatrix = parentMatrix;
            testMatrix.M31 = testMatrix.M32 = testMatrix.M34 = 0;
            testMatrix.M33 = 1;
            testMatrix.M43 = 0;

            drawInfo.GraphicsDevice.VertexDeclaration = decl;
            primitivesEffect.Parameters["WorldViewProj"].SetValue(testMatrix * drawInfo.View * drawInfo.Projection);
            primitivesEffect.Begin();

            foreach (EffectPass pass in primitivesEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                drawInfo.GraphicsDevice.DrawUserPrimitives<VertexPositionColor>(PrimitiveType.LineStrip, vertices, 0, points.Length);
                pass.End();
            }
            primitivesEffect.End();

        }

    }
}
