using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Content;
using System.IO;
using System.Diagnostics;
using Microsoft.Build.BuildEngine;
using System.Runtime.InteropServices;
using Microsoft.Build.Framework;

namespace Editor.Editor
{
    public class ContentHandler
    {

        private string contentDir;

        public string ContentDir
        {
            get { return contentDir; }
        }

        ContentManager localContentManager;

        string buildDirectory, binDir;
        FileSystemWatcher watcher;
        
        List<string> files = new List<string>();
        List<string> changes = new List<string>();
        List<string> removed = new List<string>();
        List<string> created = new List<string>();
        Dictionary<string, string> renames = new Dictionary<string, string>();

        public event EventHandler Changed;

        public List<string> Files
        {
            get { return files; }
        }


        public ContentHandler(string contentDir, string objDir, IServiceProvider services)
        {
            this.contentDir = contentDir;
            buildDirectory = objDir;
            binDir = Path.Combine(buildDirectory, "bin");
            Directory.CreateDirectory(binDir);

            localContentManager = new ContentManager(services, objDir);
            watcher = new FileSystemWatcher(contentDir);
            watcher.IncludeSubdirectories = true;
            watcher.Changed += new FileSystemEventHandler(watcher_Changed);
            watcher.Renamed += new RenamedEventHandler(watcher_Renamed);
            watcher.Created += new FileSystemEventHandler(watcher_Created);
        }

        void watcher_Created(object sender, FileSystemEventArgs e)
        {
            Debug.WriteLine("Created: " + e.Name);
            created.Add(e.Name);
            if (Changed != null)
                Changed(this, null);
        }

        void watcher_Renamed(object sender, RenamedEventArgs e)
        {
            Debug.WriteLine("Renamed: " + e.OldName + " -> " + e.Name);
            renames.Add(e.OldName, e.Name);
            if (Changed != null)
                Changed(this, null);
        }

        void watcher_Changed(object sender, FileSystemEventArgs e)
        {
            Debug.WriteLine("Changed("+e.ChangeType+"): " + e.Name);
            changes.Add(e.Name);
            if (Changed != null)
                Changed(this, null);
        }

        public void UpdateFileList(params string[] extensions)
        {
            watcher.EnableRaisingEvents = false;
            files.Clear();
            foreach (string file in Directory.GetFiles(contentDir, "*", SearchOption.AllDirectories))
            {
                if (!StringInArray(Path.GetExtension(file), extensions))
                    continue;

                files.Add(GetRelative(contentDir, file));
            }
            //watcher.EnableRaisingEvents = true;

        }

        public bool HasChanges { get { return changes.Count != 0 || renames.Count != 0 || removed.Count != 0 || created.Count != 0; } }

        /*public bool BuildContent()
        {
            string[] errors;
            return BuildContent(out errors);
        }*/

        /*public T Load<T>(string asset)
        {
            T retVal = localContentManager.Load<T>(asset);
            if (retVal == null)
            {
                if (BuildContent())
                {
                    retVal = localContentManager.Load<T>(asset);
                }
            }
            return retVal;
        }*/

        /*public bool BuildContent(out string[] error)
        {
            string contentProject = Path.Combine(contentDir, "Content.contentproj");
            ErrorLogger logger = new ErrorLogger();
            Engine engine = new Engine(RuntimeEnvironment.GetRuntimeDirectory());
            Project p = new Project(engine);
            p.Load(contentProject);
            engine.RegisterLogger(logger);
            
            bool result = p.Build();
            error = logger.Errors.ToArray();
            return result;
        }*/


        public static string GetRelative(string directory, string file)
        {
            string d = Path.GetFullPath(directory);
            string f = Path.GetFullPath(file);
            if (f.StartsWith(d, StringComparison.CurrentCultureIgnoreCase))
            {
                f = f.Substring(d.Length);
                while (f.StartsWith("/") || f.StartsWith(Path.DirectorySeparatorChar.ToString()))
                    f = f.Substring(1);
            }
            return f;
        }


        static bool StringInArray(string input, string[] strings)
        {
            for (int i = 0; i < strings.Length; i++)
            {
                if (strings[i].Equals(input, StringComparison.CurrentCultureIgnoreCase))
                    return true;
            }
            return false;
        }

        /*
        /// <summary>
        /// Creates a temporary MSBuild content project in memory.
        /// </summary>
        void CreateBuildProject()
        {
            string projectPath = Path.Combine(buildDirectory, "content.contentproj");
            string outputPath = Path.Combine(buildDirectory, "bin");

            // Create the build engine.
            msBuildEngine = new Engine(RuntimeEnvironment.GetRuntimeDirectory());

            // Hook up our custom error logger.
            errorLogger = new ErrorLogger();

            msBuildEngine.RegisterLogger(errorLogger);

            // Create the build project.
            msBuildProject = new Project(msBuildEngine);

            msBuildProject.FullFileName = projectPath;

            msBuildProject.SetProperty("XnaPlatform", "Windows");
            msBuildProject.SetProperty("XnaFrameworkVersion", "v2.0");
            msBuildProject.SetProperty("Configuration", "Release");
            msBuildProject.SetProperty("OutputPath", outputPath);

            // Register any custom importers or processors.
            foreach (string pipelineAssembly in pipelineAssemblies)
            {
                msBuildProject.AddNewItem("Reference", pipelineAssembly);
            }

            // Include the standard targets file that defines
            // how to build XNA Framework content.
            msBuildProject.AddNewImport("$(MSBuildExtensionsPath)\\Microsoft\\XNA " +
                                        "Game Studio\\v2.0\\Microsoft.Xna.GameStudio" +
                                        ".ContentPipeline.targets", null);
        }*/



    }
}
