using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Editor.Tools;
using System.Collections;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Editor.Data;
using System.Runtime.Serialization.Formatters.Binary;
using GameEngine.Xml;
using GameEngine;
using Editor.Editor;
using MultiSelectTreeview;
using System.Linq;

namespace Editor
{
    public partial class EditorPage : UserControl
    {
        ContentManager contentManager;
        XNAProjectWrapper xnaProject;
        GameEngine.ForestHandler forest = new GameEngine.ForestHandler();

        public Dictionary<string, object> LoadProperties = new Dictionary<string, object>();
        public ContentManager Content { get { return contentManager; } }
        public ForestHandler Forest { get { return forest; } }
        public EditorControl EditorControl { get { return editorControl1; } }
        public bool Dirty { get { return editorControl1.Dirty; } set { editorControl1.Dirty = value; } }

        ContextMenu menu = new ContextMenu();
        MenuItem addMenu = new MenuItem("Add Node");

        public EditorPage()
        {
            InitializeComponent();
        }

        public EditorPage(string contentDirectory, XNAProjectWrapper xnaProject )
            : this()
        {
            this.xnaProject = xnaProject;
            contentManager = new ContentManager(editorControl1.Services, contentDirectory);

            forest.NodeAdded += new GameEngine.ForestMapChange(forest_NodeAdded);
            forest.NodeAdded += new GameEngine.ForestMapChange(editorControl1.forest_NodeAdded);
            forest.NodeRemoved += new GameEngine.ForestMapChange(forest_NodeRemoved);
            forest.NodeRemoved += new GameEngine.ForestMapChange(editorControl1.forest_NodeRemoved);
            editorControl1.forest = forest;
            editorControl1.SelectionChanged += new SelectionChangedHandler(editorControl1_SelectionChanged);
            editorControl1.GameContent = contentManager;
            menu.MenuItems.Add("Convert to physics prop", new EventHandler(convertToPhysicsProp));
            menu.MenuItems.Add("Remove item...", new EventHandler(removeTreeItem));
            menu.MenuItems.Add("-");
            menu.MenuItems.Add(addMenu);


            foreach (var t in NodeReflector.GetNodeTypes())
            {
                ToolStripMenuItem itam = new ToolStripMenuItem(t.Name);
                itam.Click += new EventHandler(addNode);
                itam.Tag = t;
                addToolStripMenuItem.DropDownItems.Add(itam);
                //addMenu.MenuItems.Add(itam);
            }
        }


        void editorControl1_Enter(object sender, EventArgs e)
        {
            splitContainer1.Panel1.BackColor = editorControl1.Focused ? System.Drawing.Color.Green : System.Drawing.Color.Red;
        }

        public SceneNode GetFirstSelectedNode()
        {
            if (treeView1.SelectedNode == null)
                return null;
            return treeView1.SelectedNode.Tag as SceneNode;
        }



        void editorControl1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
 

            List<TreeNode> selectedNodes = new List<TreeNode>();
            foreach (SceneNode selectedNode in e.SelectedNodes)
            {
                foreach (TreeNode treeNode in treeView1.Nodes)
                    editorControl1_SelectionChanged_TraverseNodeDFS(treeNode, selectedNode, selectedNodes);

            }
            treeView1.SelectedNodes = selectedNodes;
            
        }

        private void editorControl1_SelectionChanged_TraverseNodeDFS(TreeNode treeNode, SceneNode sceneNode, List<TreeNode> selectedNodes)
        {
            foreach (TreeNode childNode in treeNode.Nodes)
                editorControl1_SelectionChanged_TraverseNodeDFS(childNode, sceneNode, selectedNodes);
            if (((SceneNode)treeNode.Tag).Path == sceneNode.Path)
            {
                selectedNodes.Add(treeNode);
                treeNode.EnsureVisible();
            }
        }

        void forest_NodeRemoved(GameEngine.ForestHandler forest, string path, GameEngine.Xml.SceneNode node)
        {
            TreeNode[] results = treeView1.Nodes.Find(path, true);
            if (results.Length == 1)
            {
                results[0].Remove();
            }
        }

        void forest_NodeAdded(GameEngine.ForestHandler forest, string path, GameEngine.Xml.SceneNode node)
        {

            TreeNode treeNode = new TreeNode(node.Name);
            treeNode.Name = node.Path;
            treeNode.Tag = node;
            if (node.Parent == null)
            {
                treeView1.Nodes.Add(treeNode);
            }
            else
            {
                if (string.IsNullOrEmpty(node.Parent.Include))
                {
                    TreeNode[] parentNode = treeView1.Nodes.Find(node.Parent.Path, true);
                    if (parentNode.Length == 1)
                    {
                        parentNode[0].Nodes.Add(treeNode);
                    }
                }
            }
        }


        #region treeView
        private void btnContentMenu_Click(object sender, EventArgs e)
        {
            menu.Tag = null;
            menu.MenuItems[0].Visible = false;
            menu.MenuItems[1].Visible = false;
            menu.MenuItems[2].Visible = false;
            addMenu.Text = "Add Root Node";

            menu.Show(btnContentMenu, new System.Drawing.Point(0, btnContentMenu.Height));

            addMenu.Text = "Add Node";
            menu.MenuItems[0].Visible = true;
            menu.MenuItems[1].Visible = true;
            menu.MenuItems[2].Visible = true;
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node != null)
            {
                propertyGrid1.SelectedObject = e.Node.Tag;
                if (e.Node.Tag is ObjectNode)
                {
                    // We should only be clearing the editor's selection if there's exactly one node selected when we reach here
                    // as that means we have cleared the selection in the treeview...
                    if (((MultiSelectTreeview.MultiSelectTreeview)e.Node.TreeView).SelectedNodes.Count == 1)
                        editorControl1.ToolHandler.Selection.ClearSelection();
                    editorControl1.ToolHandler.Selection.AddToSelection((ObjectNode)e.Node.Tag);
                }

                if (e.Node.Tag is PolygonWallNode)
                {
                    lastWallNode = e.Node.Tag as PolygonWallNode;
                }
            }
        }

  
        private void propertyGrid1_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
        {
            GameEngine.Xml.SceneNode node = propertyGrid1.SelectedObject as GameEngine.Xml.SceneNode; 
            if (node != null && e.ChangedItem.Label == "Name")
            {
                if (forest.Map.ContainsKey(node.Path))
                    node.Name = (string)e.OldValue;
                else
                    forest.UpdateMap();
    
                TreeNode n = treeView1.Nodes.Find(node.Path, true)[0];
                treeView1.SelectedNode = n;
            }
        }

        void RefreshPropGrid(object sender, MouseEventArgs e)
        {
            propertyGrid1.Refresh();
        }
        private void editorControl1_MouseLeave(object sender, EventArgs e)
        {
            propertyGrid1.Refresh();
            treeView1.Focus();
        }
        #endregion

        public void AddNode(SceneNode node, NodeList list)
        {
            if (list == null)
                list = forest.Roots;

            node.Name = findUniqueName(list, node.Name);
            list.Add(node);

            try
            {
                INodeLoadable loadable = node as INodeLoadable;
                if (loadable != null)
                {
                    loadable.LoadContent(contentManager, editorControl1.GraphicsDevice);
                }
            }
            catch
            {
                forest.Roots.UnloadRecursive();
                //forest.TraverseBreadthFirst(UnloadForest);
                xnaProject.Content.Save();
                BuildContentDialog bcd = new BuildContentDialog(xnaProject);
                bcd.ShowDialog();
                forest.Roots.LoadRecursive(contentManager, editorControl1.GraphicsDevice);
                //forest.TraverseBreadthFirst(LoadForest);
            }
            
            if (list == forest.Roots)
                forest.UpdateMap();
            else
                forest.UpdateMapNode(list.Owner);
        }

        public void AddNodes(List<SceneNode> newNodes, NodeList list)
        {
            if (list == null)
                list = forest.Roots;


            foreach (var node in newNodes)
            {
                node.Name = findUniqueName(list, node.Name);
                list.Add(node);
            }

            try
            {
                forest.Roots.LoadRecursive(contentManager, editorControl1.GraphicsDevice);
                //forest.TraverseBreadthFirst(LoadForest, newNodes);
            }
            catch
            {
                forest.Roots.UnloadRecursive();
                //forest.TraverseBreadthFirst(UnloadForest);
                xnaProject.Content.Save();
                BuildContentDialog bcd = new BuildContentDialog(xnaProject);
                bcd.ShowDialog();
                forest.Roots.LoadRecursive(contentManager, editorControl1.GraphicsDevice);
            }
            if (list == forest.Roots)
                forest.UpdateMap();
            else
                forest.UpdateMapNode(list.Owner);
        }
        
        private void editorControl1_Initialized(object sender, EventArgs e)
        {
            editorControl1.ToolHandler.GetTool<PolygonTool>().PolygonCreated += new EventHandler<PolygonTool.PolygonEventArgs>(PolygonCreated);

            // Load assets
            LoadProperties.Add("IsEditor", true);
            forest.Roots.LoadRecursive(contentManager, editorControl1.GraphicsDevice);
        }


        private string findUniqueName(NodeList nodeList, string oldName)
        {

            // Make sure that the sceneNode doesn't already contain a child with this name before adding
            bool checkName = true;
            string newName = oldName;
            int nameFix = 1;
            while (checkName)
            {
                if (nodeList.Count == 0)
                    checkName = false;
                foreach (SceneNode childNode in nodeList)
                {
                    if (childNode.Name == newName)
                    {
                        // Let's try that again, with a new name...
                        newName = oldName + " (" + nameFix.ToString() + ")";
                        nameFix++;
                        break;
                    }
                    // If we've reach here and have checked all the children, then we have a unique name, yay!
                    if (nodeList.IndexOf(childNode) == nodeList.Count - 1)
                        checkName = false;
                }
            }
            return newName;
        }

        private void treeView1_ItemDrag(object sender, ItemDragEventArgs e)
        {
            TreeNode theNode = (TreeNode)e.Item;

            DoDragDrop(theNode, DragDropEffects.Move);
        }

        private void treeView1_DragEnter(object sender, DragEventArgs e)
        {
            // This means we are dragging a scenenode inside the scene itself (that always yields a single scenenode)
            if (e.Data.GetDataPresent(typeof(TreeNode).FullName))
                e.Effect = DragDropEffects.Move;
            // This would mean we're dragging from the main treeview (that always yields a list of scenenodes)
            else if (e.Data.GetDataPresent(typeof(List<SceneNode>).FullName))
                e.Effect = DragDropEffects.Copy;
        }

        private void treeView1_DragDrop(object sender, DragEventArgs e)
        {
            // This means we are dragging a scenenode inside the scene itself (that always yields a single scenenode)
            if (e.Data.GetDataPresent(typeof(TreeNode).FullName))
            {
                // Find the item we dropped on...
                System.Drawing.Point pt = treeView1.PointToClient(new System.Drawing.Point(e.X, e.Y));
                TreeNode theNode = treeView1.GetNodeAt(pt);

                // Get the node corresponding to the tree node we dragged
                TreeNode droppedTreeNode = ((TreeNode)e.Data.GetData(typeof(TreeNode).FullName));
                SceneNode droppedNode = (SceneNode)droppedTreeNode.Tag;

                // if we drop something on itself, then plz do nothing
                if (theNode == droppedTreeNode)
                    return;

                if (theNode != null && ((SceneNode)theNode.Tag).Include != null)
                {
                    MessageBox.Show("Sorry, cannot drop nodes into included items...");
                    return;
                }


                // Rip the node out of the forest...
                forest.RemoveNode(droppedNode.Path);

                if (theNode == null)
                {
                    // Insert as new root node
                    AddNode(droppedNode, forest.Roots);
                }
                else
                {
                    // Insert as child in the node it was dropped on...
                    AddNode(droppedNode, (theNode.Tag as SceneNode).ChildNodes);
                }
                
            }
            // This would mean we're dragging from the main treeview (that always yields a list of scenenodes)
            else if (e.Data.GetDataPresent(typeof(List<SceneNode>).FullName))
            {
                List<SceneNode> newNodes = (List<SceneNode>)e.Data.GetData(typeof(List<SceneNode>).FullName);

                // Find the item we dropped on...
                System.Drawing.Point pt = treeView1.PointToClient(new System.Drawing.Point(e.X, e.Y));
                TreeNode theNode = treeView1.GetNodeAt(pt);

                // If we did not drop the item on a node, add it to roots
                if (theNode == null)
                {
                    AddNodes(newNodes, forest.Roots);
                }
                else
                {
                    AddNodes(newNodes, (theNode.Tag as SceneNode).ChildNodes);
                }

            }
        }

        void UnloadForest(ForestHandler sender, string path, GameEngine.Xml.SceneNode node)
        {
            INodeLoadable loadable = node as INodeLoadable;
            if (loadable != null)
            {
                loadable.UnloadContent();
            }

        }

        private void treeView1_KeyUp(object sender, KeyEventArgs e)
        {
            if (treeView1.SelectedNode != null && e.KeyCode == Keys.Delete)
                if (MessageBox.Show("Are you sure you wish to to delete the item from the level?", "Delete?", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
                    forest.RemoveNode(((SceneNode)treeView1.SelectedNode.Tag).Path);
        }

        private void treeView1_MouseClick(object sender, MouseEventArgs e)
        {


            if (e.Button == MouseButtons.Right)
            {
                TreeNode theNode = treeView1.GetNodeAt(e.X, e.Y);
                treeView1.SelectedNode = theNode;
                /*if (theNode == null)
                {
                    //menu.Tag = theNode;
                    //menu.Show(treeView1, new System.Drawing.Point(e.X, e.Y));
                    treeView1.SelectedNode = null;
                } */
                    
            }
        }

        void addNode(object sender, EventArgs args)
        {
            Type t = ((ToolStripItem)sender).Tag as Type;

            // Create new node
            SceneNode newNode = NodeReflector.InstanciateNode(t, t.Name) as SceneNode;

            if (treeView1.SelectedNode == null)
            {
                // Get all the forest roots
                NodeList roots = new NodeList(new SceneNode("tempNode"));
                foreach (SceneNode child in forest.Roots)
                    roots.Add(child);

                // ensure it has a unique name
                newNode.Name = findUniqueName(roots, newNode.Name);

                // Add to parent
                forest.AddRoot(newNode);

                // Make sure the map knows
                forest.UpdateMap();
            }
            else
            {
                // Get parent node 
                SceneNode parent = treeView1.SelectedNode.Tag as SceneNode;

                // ensure it has a unique name
                newNode.Name = findUniqueName(parent.ChildNodes, newNode.Name);

                // Add to parent
                parent.ChildNodes.Add(newNode);

                // Make sure the map knows
                forest.UpdateMapNode(parent);
            }
        }

        private void convertToPhysicsProp(object sender, EventArgs e)
        {
            /*ToolStripItem c = sender as ToolStripItem;
            SceneNode sceneNode;
            forest.ContainsNode(treeView1.SelectedNode.Name, out sceneNode);

            // Create new physics prop with the name of the old node, plus Phys, to identify it as that node's physics representation...
            PhysObjectNode physicsProp = new PhysObjectNode(sceneNode.Name + " Phys");

            // Find out of the node we're converting is a root node
            bool isRoot = sceneNode.Parent == null;

            if (isRoot)
            {
                // Fix name of new physics prop...
                NodeList nodeList = new NodeList(new SceneNode("temp"));
                foreach (SceneNode tempNode in forest.Roots)
                    nodeList.Add(tempNode);
                physicsProp.Name = findUniqueName(nodeList, physicsProp.Name);

                // Ass to the forest as a new root node
                forest.AddRoot(physicsProp);
                // Remove the converted node from the list of roots
                forest.Roots.Remove(sceneNode);
            }
            else
            {
                // Fix name of new physics prop...
                physicsProp.Name = findUniqueName(sceneNode.Parent.ChildNodes, physicsProp.Name);
                // Add to forest as child to menu.Tag's parent
                sceneNode.Parent.ChildNodes.Add(physicsProp);
                // Remove the node from the list of the parent node's children
                sceneNode.Parent.ChildNodes.Remove(sceneNode);
            }

            // (Re)assign the converted node to be a child of the physics prop
            physicsProp.ChildNodes.Add(sceneNode);

            // This is only possible if the SceneNode is at least an ObjectNode...
            if (sceneNode is ObjectNode)
            {
                // Set physics prop's position etc. to menu.Tag's position etc.
                physicsProp.InitialPosition = ((ObjectNode)sceneNode).InitialPosition;
                physicsProp.InitialRotation = ((ObjectNode)sceneNode).InitialRotation;
                physicsProp.InitialScale = ((ObjectNode)sceneNode).InitialScale;

                // Set the menu.Tag's position to zero (relative to its new position inside the physics prop)
                ((ObjectNode)sceneNode).InitialPosition = Vector3.Zero;
                ((ObjectNode)sceneNode).InitialRotation = Quaternion.Identity;
                ((ObjectNode)sceneNode).InitialScale = Vector3.One;
            }

            // Update the forest
            forest.TraverseBreadthFirst(LoadForest);
            forest.UpdateMap();*/
        }

        private void removeTreeItem(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you wish to to delete the item from the level?", "Delete?", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
                forest.RemoveNode(((SceneNode)treeView1.SelectedNode.Tag).Path);
        }

        static PolygonWallNode lastWallNode;

        private void PolygonCreated(object sender, PolygonTool.PolygonEventArgs args)
        {
            // Generate a new name for the polygon and put it into the Polygons node

            PolygonTool tool = editorControl1.ToolHandler.GetTool<PolygonTool>();
            PolygonNode ppn = PolygonTool.SelectedType.GetConstructor(new Type[] { typeof(string) }).Invoke(new object[] { PolygonTool.SelectedType.Name }) as PolygonNode;
            ppn.SetPoints(args.Polygon.Points);

            TrySetWallPolySettings(ppn);


            SceneNode node = GetFirstSelectedNode();
            
            if (node != null && !(node is PolygonNode))
            {
                AddNode(ppn, node.ChildNodes);
            }
            else
            {
                AddNode(ppn, null);
            }

            editorControl1.ToolHandler.Selection.ClearSelection();
            editorControl1.ToolHandler.Selection.AddToSelection(ppn);
            editorControl1.ToolHandler.Selection.EndSelection();
            if(treeView1.SelectedNode != null)
                propertyGrid1.SelectedObject = treeView1.SelectedNode.Tag;
            /*string name = "Poly";
            int i = 1;
            while (polygonsNode.Nodes.Find(name, true).Length != 0)
                name = "Poly_" + i++;
            AddPolygon(new PhysicsPolygonNode(args.Polygon, name));*/
        }

        private void TrySetWallPolySettings(PolygonNode ppn)
        {
            PolygonWallNode pwn = ppn as PolygonWallNode;
            if (pwn != null)
            {

                if (lastWallNode != null)
                {
                    pwn.Height = lastWallNode.Height;

                    pwn.WallColor = lastWallNode.WallColor;
                    pwn.WallTexture = lastWallNode.WallTexture;
                    pwn.WallTextureOffset = lastWallNode.WallTextureOffset;
                    pwn.WallTextureSize = lastWallNode.WallTextureSize;

                    pwn.TopColor = lastWallNode.TopColor;
                    pwn.TopTexture = lastWallNode.TopTexture;
                    pwn.TopTextureOffset = lastWallNode.TopTextureOffset;
                    pwn.TopTextureSize = lastWallNode.TopTextureSize;
                }
                else 
                {
                    var tileInfoNode = forest.NodesDepthFirst.FirstOrDefault(x => x is TileInfoNode) as TileInfoNode;
                    if (tileInfoNode != null)
                    {
                        //pwn.Height = tileInfoNode.Height;
                    }
                }
                lastWallNode = pwn;
            }
        }


        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            bool selection = treeView1.SelectedNodes.Count > 0;

            convertToolStripMenuItem.Enabled = selection;
            rToolStripMenuItem.Enabled = selection;

            convertToolStripMenuItem.DropDownItems.Clear();

            if(selection) {

                ToolStripMenuItem physPropConvert = new ToolStripMenuItem("Physics Object");
                physPropConvert.Click += new EventHandler(convertToPhysicsProp);
                convertToolStripMenuItem.DropDownItems.Add(physPropConvert);

                if (treeView1.SelectedNode.Tag is PolygonNode)
                {
                    var types = NodeReflector.GetNodeTypes(typeof(PolygonNode));
                    if (types.Count() > 1)
                    {
                        convertToolStripMenuItem.DropDownItems.Add("-");
                        foreach (var item in types)
                        {
                            if (item == typeof(PolygonNode) || item == treeView1.SelectedNode.Tag.GetType())
                                continue;

                            ToolStripMenuItem convertButton = new ToolStripMenuItem(item.Name);
                            convertButton.Click += new EventHandler(ConvertPolygon);
                            convertButton.Tag = item;
                            convertToolStripMenuItem.DropDownItems.Add(convertButton);

                        }
                    }

                }
            }

            e.Cancel = false;
        }

        private void ConvertPolygon(object sender, EventArgs args)
        {
            Type targetType = ((ToolStripMenuItem)sender).Tag as Type;

            foreach (var selectedNode in treeView1.SelectedNodes)
            {
                PolygonNode node = selectedNode.Tag as PolygonNode;
                if (node == null)
                    continue;
                NodeList targetList = node.Parent == null ? forest.Roots : node.Parent.ChildNodes;

                PolygonNode newNode = NodeReflector.InstanciateNode(targetType, node.Name + "(C)") as PolygonNode;
                newNode.InitialPosition = node.InitialPosition;
                newNode.InitialRotation = node.InitialRotation;
                newNode.InitialScale = node.InitialScale;

                while (node.ChildNodes.Count > 0)
                {
                    SceneNode n = node.ChildNodes[0];
                    node.ChildNodes.RemoveAt(0);
                    newNode.ChildNodes.Add(n);
                }
                newNode.Points.AddRange(node.Points);
                newNode.Tag = node.Tag;

                targetList.Remove(node);
                targetList.Add(newNode);

                forest.UpdateMap();
                TrySetWallPolySettings(newNode);


                newNode.Dirty();
                if (newNode is INodeLoadable)
                    ((INodeLoadable)newNode).LoadContent(contentManager, editorControl1.GraphicsDevice);
            }


        }

    }
}
