using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework.Content.Pipeline.Serialization.Compiler;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using BinarySerialization;
using Microsoft.Xna.Framework;
using Editor.Editor;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;
using GameEngine.Level;
using Editor;
using System.Security;
using System.Runtime.InteropServices;
using SharpMedia.XNA;
using System.Threading;
using System.Diagnostics;
using System.Collections;
using System.Reflection;
using Editor.Tools;
using Editor.Data;
using Microsoft.Xna.Framework.Content;
using GameEngine.Xml;
using System.Xml;
using GameEngine.Designer;
using GameEngine;
using System.Linq;

namespace PhysicsEditor
{
    public partial class Form1 : Form
    {

        XNAProjectWrapper xnaProject;



        public static Settings settings;
        
        public Form1()
        {
            LoadSettings();
            InitializeComponent();
            EditorControl.Form = this;
            Application.Idle += new EventHandler(EditorControl.UpdateLoop);

            setupToolstrip();

            xnaProject = new XNAProjectWrapper(settings.ProjectFile);
//            BuildContentDialog dialog = new BuildContentDialog(xnaProject);
//            dialog.ShowDialog();

            UpdateFileList();

        }

        private void LoadSettings()
        {
            settings = Settings.Load("settings.bin");
            if (settings == null || !File.Exists(settings.ProjectFile))
            {
                settings = new Settings();
                OpenFileDialog fbd = new OpenFileDialog();
                fbd.Multiselect = false;

                fbd.Filter = "Projects (*.csproj)|*.csproj|All Files(*.*)|*.*";
                if (fbd.ShowDialog() == DialogResult.Cancel || !File.Exists(fbd.FileName))
                    Environment.Exit(1);

                settings.ProjectFile = fbd.FileName;
                settings.Save("settings.bin");
            }
        }

        void UpdateTextureList()
        {
            TextureChooser.Textures.Clear();

            foreach (ContentItemWrapper item in xnaProject.Content.Items)
            {
                switch (Path.GetExtension(item.Include.ToLower()))
                {
                    case ".jpg":
                    case ".png":
                    case ".dds":
                        TextureChooser.Textures.Add(item.AssetPath);
                        break;
                }
            }
        }

        void UpdateFileList()
        {
            
            fileTree.BeginUpdate();
            listView1.BeginUpdate();
            listView1.Items.Clear();
            fileTree.Nodes["Content"].Nodes.Clear();
            UpdateTextureList();
            Dictionary<string, ContentItemWrapper> items = new Dictionary<string, ContentItemWrapper>();
            foreach (ContentItemWrapper item in xnaProject.Content.Items)
            {
                if(!items.Keys.Contains(item.Include.ToLower()))
                    items.Add(item.Include.ToLower(), item);
            }

            string[] files = Directory.GetFiles(xnaProject.Content.Directory, "*", SearchOption.AllDirectories);

            Array.Sort(files, FileUtil.CompareFiles);

            foreach (string f in files)
            {
                TreeNode node = fileTree.Nodes["Content"];
                string file = FileUtil.GetRelative(Path.GetDirectoryName(xnaProject.Content.File), f);
                string fileTitle = FileUtil.SplitPath(file).Last();
                string path = "";
                string ext = Path.GetExtension(file).ToLower();

                if ((!Directory.Exists(file) && ext != ".x" && ext != ".fbx" && ext != ".xml" && ext != ".dds" && ext != ".jpg" && ext != ".png") || file.StartsWith("bin", StringComparison.InvariantCultureIgnoreCase) || file.StartsWith("obj", StringComparison.InvariantCultureIgnoreCase))
                    continue;
                
                if(items.ContainsKey(file.ToLower()))
                {
                    ListViewItem newItem = new ListViewItem(fileTitle);
                    switch (ext)
                    {
                        case ".x":
                        case ".fbx":
                            newItem.ImageIndex = 1;
                            newItem.Group = listView1.Groups["lvgModels"];
                            break;
                        case ".xml":
                            newItem.ImageIndex = 2;
                            newItem.Group = listView1.Groups["lvgScenes"];
                            break;
                        default:
                            newItem.ImageIndex = 2;
                            newItem.Group = listView1.Groups["lgvOthers"];
                            break;
                    }
                    newItem.Tag = items[file.ToLower()];
                    listView1.Items.Add(newItem);
                }

                foreach (string part in FileUtil.SplitPath(file))
                {
                    path = Path.Combine(path, part);
                    if (node.Nodes.ContainsKey(part))
                    {
                        node = node.Nodes[part];
                    }
                    else
                    {
                        TreeNode newNode = new TreeNode(part);
                        newNode.Name = part;
                        switch (Path.GetExtension(part).ToLower())
                        {
                            case ".x":
                            case ".fbx":
                                newNode.SelectedImageIndex = newNode.ImageIndex = 1;
                                break;
                            case "":
                                newNode.SelectedImageIndex = newNode.ImageIndex = 0;
                                break;

                            default:
                                newNode.SelectedImageIndex = newNode.ImageIndex = 2;
                                break;
                        }


                        node.Nodes.Add(newNode);
                        if (Path.HasExtension(part))
                        {
                            if (items.ContainsKey(file.ToLower()))
                            {
                                newNode.Tag = items[file.ToLower()];
                                newNode.Checked = true;
                                newNode.EnsureVisible();

                            }
                            else
                            {
                                newNode.Tag = file.ToLower();
                            }
                        }
                        node = newNode;
                    }
                }


                fileTree.Nodes["Content"].Expand();
            }

            // find all tilesets
            TilesetManager.TileSets.Clear();
            foreach (var file in Directory.GetFiles(Path.Combine(xnaProject.Content.Directory, "Tilesets"), "*.xml", SearchOption.TopDirectoryOnly))
            {
                try
                {
                    foreach (var item in XmlTest.LoadTest(file).Where(x => x is TilesetNode).Cast<TilesetNode>())
                    {
                        item.Tag = file;
                        TilesetManager.TileSets.Add(item);
                    }
                }
                catch
                {
                }
            }
            TilesetManager.TileSets.Sort((x, y) => x.Name.CompareTo(y.Name));

            fileTree.EndUpdate();
            listView1.EndUpdate();

        }

        public void SavePage(TabPage tab, EditorPage page, bool saveAs)
        {
            string file = tab.Name;

            if (file == "Untitled" || !File.Exists(file) || saveAs)
            {
                // Do file selection
                SaveFileDialog sfd = new SaveFileDialog();
                sfd.InitialDirectory = Path.GetDirectoryName(xnaProject.Content.File);
                sfd.Filter = "Xml File (*.xml)|*.xml|All Files (*.*)|*.*";
                sfd.AddExtension = true;
                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    file = sfd.FileName;
                }
            }

            if (file != "Untitled")
            {
                if (page.Forest.Roots.Count > 0)
                {
                    XmlDocument doc = XmlHelper.CreateDocument(page.Forest.Roots);
                    doc.Save(file);
                    UpdateFileList();
                    tab.Name = file;
                    tab.Text = Path.GetFileName(file);
                }
            }
        }

        private void addNewTab(TabPage page, EditorPage editPage)
        {
            editPage.Dock = DockStyle.Fill;
            hookUpToolbar(editPage);
            page.Tag = editPage;
            page.Controls.Add(editPage);
            tabControl1.TabPages.Add(page);
            hookUpToolsToolstrip(editPage);
            tabControl1.SelectTab(page);
            // Adding new pages to a tabcontrol and selecting that tab apparently does not actually select it... according to the events, at any rate
            tabControl1_Selecting(this, new TabControlCancelEventArgs(page, tabControl1.TabPages.IndexOf(page), false, TabControlAction.Selecting));
            toolStrip1.Enabled = true;
        }


        #region toolStrip

        private Dictionary<string, List<ToolStripItem>> toolStrips = new Dictionary<string, List<ToolStripItem>>();

        void setupToolstrip()
        {
            setupToolstripSelectionTool();
            setupToolstripPolygonTool();
            setupToolstripScaleTool();
            setupToolstripRotateTool();
            setupToolstripMoveTool();
            setupToolstripPolygonModifierTool();
        }

        void setActiveToolstrip(string newTool)
        {
            while (toolStrip1.Items.Count > toolStrip1.Items.IndexOf(toolSep) + 1)
                toolStrip1.Items.Remove(toolStrip1.Items[toolStrip1.Items.IndexOf(toolSep) + 1]);

            foreach (ToolStripItem theItem in toolStrips[newTool])
                toolStrip1.Items.Add(theItem);
        }

        void editor_ToolChanged(object sender, ToolChangeEventArgs e)
        {
            foreach (ToolStripItem theItem in toolStrip1.Items)
            {
                if (theItem.Tag == null)
                    continue;
                if (!((String)theItem.Tag).EndsWith("Tool"))
                    continue;

                if ((string)theItem.Tag == e.NewTool)
                    ((ToolStripButton)theItem).Checked = true;
                else
                    ((ToolStripButton)theItem).Checked = false;
            }
            setActiveToolstrip(e.NewTool);
        }

        void editor_GridVisibilityChanged(object sender, GridVisibilityChangedEventArgs e)
        {
            switch (e.WhichGrid)
            {
                case "xy":
                    tsbShowXYGrid.Checked = e.NewValue;
                    tsbGridIncreaseXY.Enabled = e.NewValue;
                    tsbGridDecreaseXY.Enabled = e.NewValue;
                    break;
                case "xz":
                    tsbShowXZGrid.Checked = e.NewValue;
                    tsbGridIncreaseXZ.Enabled = e.NewValue;
                    tsbGridDecreaseXZ.Enabled = e.NewValue;
                    break;
                default:
                    break;
            }
        }

        private void setupToolstripSelectionTool()
        {
            List<ToolStripItem> selectionTools = new List<ToolStripItem>();
            ToolStripLabel seLabel = new ToolStripLabel("Click to ray select or click/drag to box select");
            selectionTools.Add(seLabel);
            /*ToolStripButton seRaySelection = new ToolStripButton("Ray selection");
            seRaySelection.Tag = "seRay";
            ToolStripButton seBoxSelection = new ToolStripButton("Box selection");
            seBoxSelection.Tag = "seBox";
            selectionTools.Add(seRaySelection);
            selectionTools.Add(seBoxSelection);*/
            toolStrips.Add("SelectionTool", selectionTools);
        }

        private void setupToolstripPolygonTool()
        {
            List<ToolStripItem> polygonTools = new List<ToolStripItem>();
            //ToolStripLabel poLabel = new ToolStripLabel("Click to create vertices, click near start node or right click to end polygon");
            //polygonTools.Add(poLabel);

            ToolStripComboBox combo = new ToolStripComboBox("polygonTypes") { DropDownStyle = ComboBoxStyle.DropDownList };
            combo.SelectedIndexChanged += new EventHandler(delegate(object sender, EventArgs a) 
                {
                    PolygonTool.SelectedTypeIndex = ((ToolStripComboBox)sender).SelectedIndex;
                });
            
            

            ToolStripSplitButton button = new ToolStripSplitButton("PolygonNode");
            
            
            foreach (var item in PolygonTool.PolygonTypes)
            {
                combo.Items.Add(item.Name);
            }
            combo.SelectedIndex = PolygonTool.SelectedTypeIndex;
            polygonTools.Add(combo);
            toolStrips.Add("PolygonTool", polygonTools);
        }

        private void setupToolstripPolygonModifierTool()
        {
            List<ToolStripItem> polygonTools = new List<ToolStripItem>();
            ToolStripLabel poLabel = new ToolStripLabel("PolygonModifier tool - needs a proper toolstrip");
            polygonTools.Add(poLabel);
            toolStrips.Add("PolygonModifierTool", polygonTools);
        }

        private void setupToolstripScaleTool()
        {
            List<ToolStripItem> scaleTools = new List<ToolStripItem>();
            ToolStripLabel scAxisLabel = new ToolStripLabel("Hold the x, y and z keys to lock scaling along those axes");
            scaleTools.Add(scAxisLabel);
            /*ToolStripButton scAxisX = new ToolStripButton("X");
            scAxisX.Tag = "scAxisX";
            ToolStripButton scAxisY = new ToolStripButton("Y");
            scAxisY.Tag = "scAxisY";
            ToolStripButton scAxisZ = new ToolStripButton("Z");
            scAxisZ.Tag = "scAxisZ";
            ToolStripLabel scAxisLabel2 = new ToolStripLabel("axes");
            scaleTools.Add(scAxisX);
            scaleTools.Add(scAxisY);
            scaleTools.Add(scAxisZ);
            scaleTools.Add(scAxisLabel2);*/
            toolStrips.Add("ScaleTool", scaleTools);
        }

        #region RotateTool

        private void setupToolstripRotateTool()
        {
            List<ToolStripItem> rotateTools = new List<ToolStripItem>();
            ToolStripButton roHelperGrid = new ToolStripButton("Show helper grid");
            roHelperGrid.Tag = "roHelperGrid";
            ToolStripSeparator roSep = new ToolStripSeparator();
            ToolStripLabel roAxisLabel = new ToolStripLabel("Rotate around the");
            ToolStripButton roAxisX = new ToolStripButton("X");
            roAxisX.Tag = "roAxisX";
            ToolStripButton roAxisY = new ToolStripButton("Y");
            roAxisY.Tag = "roAxisY";
            ToolStripButton roAxisZ = new ToolStripButton("Z");
            roAxisZ.Tag = "roAxisZ";
            ToolStripLabel roAxisLabel2 = new ToolStripLabel("axis");
            rotateTools.Add(roHelperGrid);
            rotateTools.Add(roSep);
            rotateTools.Add(roAxisLabel);
            rotateTools.Add(roAxisX);
            rotateTools.Add(roAxisY);
            rotateTools.Add(roAxisZ);
            rotateTools.Add(roAxisLabel2);
            toolStrips.Add("RotateTool", rotateTools);
        }

        void rotateTool_RotationAxisChanged(object sender, RotateToolRotationAxisChangedEventArgs e)
        {
            ((ToolStripButton)toolStrips["RotateTool"][3]).Checked = (e.NewAxis == Axis.X) ? true : false;
            ((ToolStripButton)toolStrips["RotateTool"][4]).Checked = (e.NewAxis == Axis.Y) ? true : false;
            ((ToolStripButton)toolStrips["RotateTool"][5]).Checked = (e.NewAxis == Axis.Z) ? true : false;
        }

        void rotateTool_HelperGridVisibilityChanged(object sender, Tool.HelperGridVisibilityChangedEventArgs e)
        {
            ((ToolStripButton)toolStrips["RotateTool"][0]).Checked = e.NewValue;
        }

        #endregion

        #region MoveTool

        private void setupToolstripMoveTool()
        {
            List<ToolStripItem> moveTools = new List<ToolStripItem>();
            ToolStripButton moHelperGrid = new ToolStripButton("Show helper grid");
            moHelperGrid.Tag = "moHelperGrid";
            ToolStripSeparator moSep = new ToolStripSeparator();
            ToolStripLabel moAxisLabel = new ToolStripLabel("Move in the");
            ToolStripButton moAxisX = new ToolStripButton("XY");
            moAxisX.Tag = "moAxisX";
            ToolStripButton moAxisY = new ToolStripButton("YZ");
            moAxisY.Tag = "moAxisY";
            ToolStripButton moAxisZ = new ToolStripButton("XZ");
            moAxisZ.Tag = "moAxisZ";
            ToolStripLabel moAxisLabel2 = new ToolStripLabel("plane");
            moveTools.Add(moHelperGrid);
            moveTools.Add(moSep);
            moveTools.Add(moAxisLabel);
            moveTools.Add(moAxisX);
            moveTools.Add(moAxisY);
            moveTools.Add(moAxisZ);
            moveTools.Add(moAxisLabel2);
            toolStrips.Add("MoveTool", moveTools);
        }

        void moveTool_WorkingPlaneChanged(object sender, Editor.Tools.MoveToolWorkingPlaneChangedEventArgs e)
        {
            ((ToolStripButton)toolStrips["MoveTool"][3]).Checked = (e.NewGrid == OrientationPlane.XY) ? true : false;
            ((ToolStripButton)toolStrips["MoveTool"][4]).Checked = (e.NewGrid == OrientationPlane.YZ) ? true : false;
            ((ToolStripButton)toolStrips["MoveTool"][5]).Checked = (e.NewGrid == OrientationPlane.XZ) ? true : false;
        }

        void moveTool_HelperGridVisibilityChanged(object sender, Editor.Tool.HelperGridVisibilityChangedEventArgs e)
        {
            ((ToolStripButton)toolStrips["MoveTool"][0]).Checked = e.NewValue;
        }

        #endregion

        void hookUpToolsToolstrip(EditorPage editPage)
        {
            editPage.EditorControl.ToolHandler.GetTool<MoveTool>().WorkingPlaneChanged += new MoveToolWorkingPlaneChangedHandler(moveTool_WorkingPlaneChanged);
            editPage.EditorControl.ToolHandler.GetTool<MoveTool>().HelperGridVisibilityChanged += new Tool.HelperGridVisibilityChangedHandler(moveTool_HelperGridVisibilityChanged);
            editPage.EditorControl.ToolHandler.GetTool<RotateTool>().RotationAxisChanged += new RotateToolRotationAxisChangedHandler(rotateTool_RotationAxisChanged);
            editPage.EditorControl.ToolHandler.GetTool<RotateTool>().HelperGridVisibilityChanged += new Tool.HelperGridVisibilityChangedHandler(rotateTool_HelperGridVisibilityChanged);
            PolygonTool.SelectedPolygonTypeChanged += new EventHandler(PolygonTool_SelectedPolygonTypeChanged);
        }

        void PolygonTool_SelectedPolygonTypeChanged(object sender, EventArgs e)
        {
            foreach (ToolStripItem theItem in toolStrip1.Items)
            {
                if (theItem.Name != "polygonTypes")
                    continue;

                ((ToolStripComboBox)theItem).SelectedIndex = PolygonTool.SelectedTypeIndex;
            }
        }

        void toolStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            ToolStrip theSender = (ToolStrip)sender;
            if (tabControl1.SelectedTab != null)
            {
                EditorPage control = tabControl1.SelectedTab.Tag as EditorPage;
                if (control != null)
                {
                    switch ((string)e.ClickedItem.Tag)
                    {
                        // Grid control
                        case "GridXY":
                            control.EditorControl.ToggleGridVisibility("xy");
                            break;
                        case "GridIncreaseXY":
                            control.EditorControl.GridResize("xy", EditorControl.GridChange.Larger);
                            break;
                        case "GridDecreaseXY":
                            control.EditorControl.GridResize("xy", EditorControl.GridChange.Smaller);
                            break;
                        case "GridXZ":
                            control.EditorControl.ToggleGridVisibility("xz");
                            break;
                        case "GridIncreaseXZ":
                            control.EditorControl.GridResize("xz", EditorControl.GridChange.Larger);
                            break;
                        case "GridDecreaseXZ":
                            control.EditorControl.GridResize("xz", EditorControl.GridChange.Smaller);
                            break;
                        // Tool selector
                        case "SelectionTool":
                        case "PolygonTool":
                        case "ScaleTool":
                        case "RotateTool":
                        case "MoveTool":
                        case "PolygonModifierTool":
                            control.EditorControl.SetTool((string)e.ClickedItem.Tag);
                            break;
                        // Selection tool controls
                        // Polygon tool controls
                        // Scale tool controls
                        // Rotate tool controls
                        case "roHelperGrid":
                            ((RotateTool)control.EditorControl.ToolHandler.ActiveTool).ToggleHelperGrid();
                            break;
                        case "roAxisX":
                            ((RotateTool)control.EditorControl.ToolHandler.ActiveTool).SetRotationAxis(Axis.X);
                            break;
                        case "roAxisY":
                            ((RotateTool)control.EditorControl.ToolHandler.ActiveTool).SetRotationAxis(Axis.Y);
                            break;
                        case "roAxisZ":
                            ((RotateTool)control.EditorControl.ToolHandler.ActiveTool).SetRotationAxis(Axis.Z);
                            break;
                        // Move tool controls
                        case "moHelperGrid":
                            ((MoveTool)control.EditorControl.ToolHandler.ActiveTool).ToggleHelperGrid();
                            break;
                        case "moAxisX":
                            ((MoveTool)control.EditorControl.ToolHandler.ActiveTool).SetWorkingPlane(OrientationPlane.XY);
                            break;
                        case "moAxisY":
                            ((MoveTool)control.EditorControl.ToolHandler.ActiveTool).SetWorkingPlane(OrientationPlane.YZ);
                            break;
                        case "moAxisZ":
                            ((MoveTool)control.EditorControl.ToolHandler.ActiveTool).SetWorkingPlane(OrientationPlane.XZ);
                            break;
                        default:
                            break;
                    }
                }
            }
        }


        private void hookUpToolbar(EditorPage editPage)
        {
            editPage.EditorControl.ToolChanged += new ToolChangedHandler(editor_ToolChanged);
            editPage.EditorControl.GridVisibilityChanged += new GridVisibilityChangedHandler(editor_GridVisibilityChanged);
        }

        #endregion

        #region menu
        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tabControl1.SelectedTab != null)
            {
                SavePage(tabControl1.SelectedTab, (EditorPage)tabControl1.SelectedTab.Tag, sender == saveAsToolStripMenuItem);
            }
        }
        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string contentDir = Path.GetDirectoryName(xnaProject.Content.File);
            TabPage page = new TabPage("Untitled");
            page.Name = "Untitled";

            EditorPage editPage = new EditorPage(Path.Combine(Path.GetDirectoryName(xnaProject.ProjectFile), Path.Combine(xnaProject.OutputPath, "Content")), xnaProject);
            addNewTab(page, editPage);
        }

        private void newTileMenuItem_Click(object sender, EventArgs e)
        {

            using (NewLevelTile ts = new NewLevelTile())
            {
                if (ts.ShowDialog() == DialogResult.OK)
                {
                    string contentDir = Path.GetDirectoryName(this.xnaProject.Content.File);
                    TabPage page = new TabPage(ts.txtTileName.Text);
                    page.Name = ts.txtTileName.Text;

                    string contentDirectory = Path.Combine(Path.GetDirectoryName(this.xnaProject.ProjectFile), Path.Combine(this.xnaProject.OutputPath, "Content"));
                    EditorPage editPage = new EditorPage(contentDirectory, this.xnaProject);

                    addNewTab(page, editPage);

                    TileInfoNode tin = new TileInfoNode(ts.TileName);
                    //tin.TileSize = ts.TileSize;
                    tin.TileType = ts.TileType;
                    tin.Tag = ts.Tileset;
                    editPage.AddNode(tin, editPage.Forest.Roots);
                    editPage.EditorControl.CreateGrids(ts.Tileset.TileSize / 10, true);

                }
            }
        }

        private void rebuildToolStripMenuItem_Click(object sender, EventArgs e)
        {
            xnaProject.Content.Save();
            xnaProject.Reload();
            UpdateFileList();
            BuildContentDialog dialog = new BuildContentDialog(xnaProject);
            dialog.ShowDialog();
        }
        private void newSceneToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string contentDir = Path.GetDirectoryName(xnaProject.Content.File);
            TabPage page = new TabPage("New Scene");
            page.Name = "New Scene";

            EditorPage editPage = new EditorPage(Path.Combine(Path.GetDirectoryName(xnaProject.ProjectFile), Path.Combine(xnaProject.OutputPath, "Content")), xnaProject);
            editPage.Dock = DockStyle.Fill;
            hookUpToolbar(editPage);

            page.Tag = editPage;
            page.Controls.Add(editPage);
            tabControl1.TabPages.Add(page);
            tabControl1.SelectedTab = page;
        }

        private void showAllContentButton_Click(object sender, EventArgs e)
        {
            showAllContentButton.Checked = !showAllContentButton.Checked;
            listView1.Visible = showAllContentButton.Checked;
            UpdateFileList();
        }
        #endregion


        #region filetree

        private void LoadXmlScene(string file, ContentItemWrapper item)
        {
            if (tabControl1.TabPages.ContainsKey(file))
            {
                tabControl1.SelectedTab = tabControl1.TabPages[file];
                return;
            }

            EditorPage editPage = new EditorPage(Path.Combine(Path.GetDirectoryName(xnaProject.ProjectFile), Path.Combine(xnaProject.OutputPath, "Content")), xnaProject);
            editPage.Tag = item;

            TabPage page = new TabPage(Path.GetFileName(file));
            page.Name = file;

            addNewTab(page, editPage);


            List<SceneNode> nodes = null;
            try
            {

                nodes = XmlTest.LoadTest(file);
            }
            catch
            {
                return;
            }

            // figure out if this is a Tile
            // Set the tileinfonode.tag to the tileset belonging to this one!
            TileInfoNode tileInfoNode = nodes.FirstOrDefault(x => x is TileInfoNode) as TileInfoNode;
            string absolutePath = Path.GetFullPath(file);
            if (tileInfoNode != null)
            {
                foreach (var tileset in TilesetManager.TileSets)
                {
                    string tilesetDir = Path.Combine(Path.GetDirectoryName(tileset.Tag as string), tileset.Name);
                    if (absolutePath.StartsWith(tilesetDir, StringComparison.CurrentCultureIgnoreCase))
                    {
                        tileInfoNode.Tag = tileset;
                        editPage.EditorControl.CreateGrids(tileset.TileSize / 10, true);
                        break;
                    }
                }
            }
            //editPage.Forest.AddRoots(nodes);
            editPage.AddNodes(nodes, editPage.Forest.Roots);

        }

        private void fileTree_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Node != null && e.Node.Tag != null)
            {
                if (e.Node.Tag is ContentItemWrapper)
                {
                    ContentItemWrapper item = e.Node.Tag as ContentItemWrapper;

                    string ext = Path.GetExtension(item.Include).ToLower();
                    switch (ext)
                    {
                        case ".xml":
                            string contentDir = Path.GetDirectoryName(xnaProject.Content.File);
                            LoadXmlScene(Path.Combine(contentDir, item.Include), item);
                            break;
                        case ".x":
                        case ".fbx":
                            TabPage page = tabControl1.SelectedTab;
                            if (page != null)
                            {
                                EditorPage editPage = page.Tag as EditorPage;
                                ModelNode modelNode = new ModelNode(item.Name);
                                modelNode.Asset = item.AssetPath;
                                SceneNode node = editPage.GetFirstSelectedNode();
                                if (node != null)
                                {
                                    editPage.AddNode(modelNode, node.ChildNodes);
                                }
                                else
                                {
                                    editPage.AddNode(modelNode, null);
                                }
                            }

                            break;
                    }
                }
                else if (e.Node.Tag is string)
                {
                    string file = e.Node.Tag as string;
                    string ext = Path.GetExtension(file).ToLower();
                    switch (ext)
                    {
                        case ".xml":
                            string contentDir = Path.GetDirectoryName(xnaProject.Content.File);
                            LoadXmlScene(Path.Combine(contentDir,file), null);
                            break;
                    }
                }
            }
        }

        private void fileLists_ItemDrag(object sender, ItemDragEventArgs e)
        {
            ContentItemWrapper item;
            if (sender is TreeView)
            {
                // Let's assume that people who start dragging something want to add that asset to the project...
                ((TreeNode)e.Item).Checked = true;
                item = ((TreeNode)e.Item).Tag as ContentItemWrapper;
            }
            else
                item = ((ListViewItem)e.Item).Tag as ContentItemWrapper;

            if (item == null)
                return;
            string ext = Path.GetExtension(item.Include).ToLower();
            string contentDir = Path.GetDirectoryName(xnaProject.Content.File);
            string file = Path.Combine(contentDir, item.Include);

            List<SceneNode> nodes = null;
            switch (ext)
            {
                case ".xml":
                    try
                    {
                        nodes = XmlTest.LoadTest(file);
                    }
                    catch { return; }

                    break;
                case ".x":
                case ".fbx":
                    nodes = new List<SceneNode>();
                    ModelNode modelNode = new ModelNode(item.Name);
                    modelNode.Asset = item.AssetPath;
                    nodes.Add(modelNode);

                    break;
            }
            if (nodes != null)
                DoDragDrop(nodes, DragDropEffects.All);
        }
        private void fileTree_BeforeCheck(object sender, TreeViewCancelEventArgs e)
        {
            e.Cancel = e.Node == null || e.Node.Tag == null;
        }

        private void fileTree_AfterCheck(object sender, TreeViewEventArgs e)
        {
            if (e.Node != null && e.Node.Tag != null)
            {
                if (e.Node.Tag is string && e.Node.Checked == true)
                {
                    string file = e.Node.Tag as string;
                    ContentItemWrapper item = null;

                    // Add Item
                    switch (Path.GetExtension(file).ToLower())
                    {
                        case ".x":
                            item = xnaProject.Content.AddItem(Path.GetFileName(file).Replace('.', '_'), file, "XImporter", "GameAnimationProcessor");
                            e.Node.Tag = item;
                            break;
                        case ".fbx":
                            item = xnaProject.Content.AddItem(Path.GetFileName(file).Replace('.', '_'), file, "FbxImporter", "BBoxProcessor");
                            e.Node.Tag = item;
                            break;
                        case ".xml":
                            item = xnaProject.Content.AddItem(Path.GetFileName(file).Replace('.', '_'), file, "XmlLevelImporter", "PassThroughProcessor");
                            e.Node.Tag = item;
                            break;
                        case ".dds":
                        case ".jpg":
                        case ".png":
                            item = xnaProject.Content.AddItem(Path.GetFileName(file).Replace('.', '_'), file, "TextureImporter", "TextureProcessor");
                            e.Node.Tag = item;
                            UpdateTextureList();
                            break;

                    }

                }
                else if (e.Node.Tag is ContentItemWrapper && e.Node.Checked == false)
                {
                    // remove item
                    ContentItemWrapper item = e.Node.Tag as ContentItemWrapper;
                    e.Node.Tag = item.Include;
                    xnaProject.Content.RemoveItem(item);
                    UpdateTextureList();

                }
            }
        }

        #endregion


        private void tabControl1_MouseClick(object sender, MouseEventArgs e)
        {
            TabPage tab = tabControl1.SelectedTab;
            if (tab != null && e.Button == MouseButtons.Middle)
            {
                EditorPage editPage = tab.Tag as EditorPage;
                if (editPage.Dirty)
                {
                    DialogResult res = MessageBox.Show("Changes have not been saved, save now?", "Unsaved Changes", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);
                    if (res == DialogResult.Cancel)
                        return;
                    
                    if (res == DialogResult.Yes)
                        SavePage(tab, editPage, false);
                }
                editPage.EditorControl.Dispose();
                tabControl1.TabPages.Remove(tab);

                if (tabControl1.TabCount == 0)
                    toolStrip1.Enabled = false;
            }
        }

        private void tabControl1_Selecting(object sender, TabControlCancelEventArgs e)
        {
            if (e.TabPage != null)
            {
                ((EditorPage)e.TabPage.Tag).EditorControl.Active = true;
                ((EditorPage)e.TabPage.Tag).EditorControl.SetTool(((EditorPage)e.TabPage.Tag).EditorControl.ToolName);
                ((EditorPage)e.TabPage.Tag).EditorControl.SetGridVisibility("xy", ((EditorPage)e.TabPage.Tag).EditorControl.XYGridVisible);
                ((EditorPage)e.TabPage.Tag).EditorControl.SetGridVisibility("xz", ((EditorPage)e.TabPage.Tag).EditorControl.XZGridVisible);
                ((EditorPage)e.TabPage.Tag).EditorControl.ToolHandler.GetTool<MoveTool>().SetHelperGridVisibility(((EditorPage)e.TabPage.Tag).EditorControl.ToolHandler.GetTool<MoveTool>().HelperGridVisible);
                ((EditorPage)e.TabPage.Tag).EditorControl.ToolHandler.GetTool<MoveTool>().SetWorkingPlane(((EditorPage)e.TabPage.Tag).EditorControl.ToolHandler.GetTool<MoveTool>().WorkingPlane);
                ((EditorPage)e.TabPage.Tag).EditorControl.ToolHandler.GetTool<RotateTool>().SetHelperGridVisibility(((EditorPage)e.TabPage.Tag).EditorControl.ToolHandler.GetTool<RotateTool>().HelperGridVisible);
                ((EditorPage)e.TabPage.Tag).EditorControl.ToolHandler.GetTool<RotateTool>().SetRotationAxis(((EditorPage)e.TabPage.Tag).EditorControl.ToolHandler.GetTool<RotateTool>().RotationAxis);
            }
        }

        private void tabControl1_Deselected(object sender, TabControlEventArgs e)
        {
            if (e.TabPage != null)
            {
                ((EditorPage)e.TabPage.Tag).EditorControl.Active = false;
            }
        }



        private void Form1_KeyUp(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case System.Windows.Forms.Keys.F5:
                    RunGame();
                    break;
                case System.Windows.Forms.Keys.F6:
                    rebuildToolStripMenuItem_Click(null, null);
                    break;
                case System.Windows.Forms.Keys.S:
                    if (!e.Control || tabControl1.SelectedTab == null)
                        break;

                    SavePage(tabControl1.SelectedTab, tabControl1.SelectedTab.Tag as EditorPage, false);
                    break;
                default:
                    break;
            }

        }

        public void RunGame()
        {
            try
            {
                rebuildToolStripMenuItem_Click(null, null);

                /*Thread t = new Thread(delegate()
                {
                    AppDomain domain = AppDomain.CreateDomain("hey");
                    domain.ExecuteAssembly(Path.Combine(Path.GetDirectoryName(xnaProject.ProjectFile), Path.Combine(xnaProject.OutputPath, xnaProject.TargetFileName)), null, new string[0]);
                    AppDomain.Unload(domain);
                });
                t.Start();*/
                Process.Start(Path.Combine(Path.GetDirectoryName(xnaProject.ProjectFile), Path.Combine(xnaProject.OutputPath, xnaProject.TargetFileName)), "");
            }
            catch
            {
            }

        }

        private void fileTree_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode != System.Windows.Forms.Keys.Delete)
                return;

            if (fileTree.SelectedNode == null)
                return;

            if (MessageBox.Show("Confirm Delete?", "Delete File", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1) != DialogResult.Yes)
                return;

            ContentItemWrapper item = fileTree.SelectedNode.Tag as ContentItemWrapper;
            string path;
            if (item != null)
            {
                path = item.FullPath;
                xnaProject.Content.RemoveItem(item);
                xnaProject.Content.Save();
            }
            else if (fileTree.SelectedNode.Tag is string)
            {
                path = fileTree.SelectedNode.Tag as string;
            }
            else
                return;
            
            fileTree.SelectedNode.Remove();
            
            // remove item
            try
            {
                
                File.Delete(path);
            }
            catch
            {
            }
            

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            xnaProject.Content.Save();
            foreach (TabPage tab in tabControl1.TabPages)
            {
                EditorPage editPage = tab.Tag as EditorPage;

                DialogResult res = MessageBox.Show("Changes have not been saved, save now?", tab.Text as string, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);
                if (res == DialogResult.Cancel)
                {
                    e.Cancel = true;
                    return;
                }

                if (res == DialogResult.Yes)
                    SavePage(tab, editPage, false);

            }
        }

        private void setContentDirToolStripMenuItem_Click(object sender, EventArgs e)
        {
                settings = new Settings();
                OpenFileDialog fbd = new OpenFileDialog();
                fbd.Multiselect = false;

                fbd.Filter = "Projects (*.csproj)|*.csproj|All Files(*.*)|*.*";
                if (fbd.ShowDialog() == DialogResult.Cancel || !File.Exists(fbd.FileName))
                {
                }
                else
                {
                    settings.ProjectFile = fbd.FileName;
                    settings.Save("settings.bin");
                }
        }

        private void listView1_ItemActivate(object sender, EventArgs e)
        {
            if (listView1.SelectedItems[0] == null)
                return;

            ListViewItem activatedItem = listView1.SelectedItems[0];
            if (activatedItem.Tag != null)
            {
                if (activatedItem.Tag is ContentItemWrapper)
                {
                    ContentItemWrapper item = activatedItem.Tag as ContentItemWrapper;

                    string ext = Path.GetExtension(item.Include).ToLower();
                    switch (ext)
                    {
                        case ".xml":
                            string contentDir = Path.GetDirectoryName(xnaProject.Content.File);
                            LoadXmlScene(Path.Combine(contentDir, item.Include), item);
                            break;
                        case ".x":
                        case ".fbx":
                            TabPage page = tabControl1.SelectedTab;
                            if (page != null)
                            {
                                EditorPage editPage = page.Tag as EditorPage;
                                ModelNode modelNode = new ModelNode(item.Name);
                                modelNode.Asset = item.AssetPath;
                                SceneNode node = editPage.GetFirstSelectedNode();
                                if (node != null)
                                {
                                    editPage.AddNode(modelNode, node.ChildNodes);
                                }
                                else
                                {
                                    editPage.AddNode(modelNode, null);
                                }
                            }

                            break;
                    }
                }
                else if (activatedItem.Tag is string)
                {
                    string file = activatedItem.Tag as string;
                    string ext = Path.GetExtension(file).ToLower();
                    switch (ext)
                    {
                        case ".xml":
                            string contentDir = Path.GetDirectoryName(xnaProject.Content.File);
                            LoadXmlScene(Path.Combine(contentDir, file), null);
                            break;
                    }
                }
            }
        }
    }
}