using System;
using System.Collections.Generic;
using System.Text;
using GameEngine.Data;
using GameEngine.Level;
using System.ComponentModel;
using Microsoft.Xna.Framework;
using System.Windows.Forms;
using Editor.Data;

namespace Editor
{
    public class PhysicsPolygonNode
    {
        private PhysicsPolygon polygon;

        [Browsable(false)]
        public PhysicsPolygon Polygon
        {
            get { return polygon; }
            set { polygon = value; }
        }

        public string Name
        {
            get { return polygon.Name; }
            set { polygon.Name = value; }
        }

        public PhysicsPolygonNode(PhysicsPolygon polygon)
        {
            this.polygon = polygon;
        }

        public PhysicsPolygonNode(PhysicsPolygon polygon, string name)
        {
            this.polygon = polygon;
            this.Name = name;
        }

        public PhysicsPolygonData CreateData()
        {
            PhysicsPolygonData propData = new PhysicsPolygonData(Name, polygon.Points);
            return propData;
        }
    }


    public class StaticPropNode : GameObjectNode
    {
        private string file;

        public string File
        {
            get { return file; }
            set { file = value; }
        }
        public StaticPropNode(StaticProp prop, string file)
            : base(prop)
        {
            this.file = file;
        }

        public StaticPropData CreateData()
        {
            StaticPropData propData = new StaticPropData(file, Name, Position, Scale, obj.Rotation);
            return propData;
        }
    }

    public class GameObjectNode
    {
        protected GameObject obj;


        public string Name
        {
            get { return obj.Name; }
            set { obj.Name = value; }
        }

        public Vector3 Position
        {
            get { return obj.Position; }
            set { obj.Position = value; }
        }

        public Vector3 Scale
        {
            get { return obj.Scale; }
            set { obj.Scale = value; }
        }

        [Browsable(false)]
        public Quaternion Rotation
        {
            get { return obj.Rotation; }
        }

        [Browsable(false)]
        public GameObject GameObject { get { return obj; } }

        public GameObjectNode(GameObject obj)
        {
            this.obj = obj;
        }

    }
}
