using System;
using System.Collections.Generic;
using System.Text;
using GameEngine.Xml;

namespace GameEngine
{
    public delegate void ForestMapChange(ForestHandler forest, string path, SceneNode node);
    public class ForestHandler
    {
        // access to dictionary of nodes
        private Dictionary<string, SceneNode> map = new Dictionary<string,SceneNode>();
        public Dictionary<string, SceneNode> Map { get { return map; } }

        // List to iterate when processing tree
        List<SceneNode> roots = new List<SceneNode>();
        public List<SceneNode> Roots { get { return roots; } }
        // Ensure the map corresponds to the tree
        public void UpdateMap()
        {
            foreach (SceneNode root in roots)
            {
                UpdateMapNode(root);
            }
            List<string> keys = new List<string>(map.Keys);
            foreach (string key in keys)
            {
                SceneNode node;
                if (!ContainsNode(key, out node))
                {
                    map.Remove(key);
                    if (NodeRemoved != null)
                    {
                        NodeRemoved(this, key, node);
                    }
                }
            }
        }

        public void UpdateMapNode(SceneNode node)
        {
            string nodePath = node.Path;
            if (!Map.ContainsKey(nodePath))
            {
                Map.Add(nodePath, node);
                if (NodeAdded != null)
                {
                    NodeAdded(this, nodePath, node);
                }
            }
            foreach (SceneNode childNode in node.ChildNodes)
            {
                UpdateMapNode(childNode);
            }
        }
        
        // Events fired from UpdateMap
        public event ForestMapChange NodeAdded;
        public event ForestMapChange NodeRemoved;

        // adds nodes
        public void AddRoot(SceneNode node)
        {
            roots.Add(node);
            UpdateMapNode(node);
        }

        public void AddRoots(IEnumerable<SceneNode> nodes)
        {
            roots.AddRange(nodes);
            foreach (SceneNode node in nodes)
            {
                UpdateMapNode(node);
            }
        }

        public bool ContainsNode(string path, out SceneNode node)
        {
            node = null;
            string[] parts = path.Split('.');

            IEnumerable<SceneNode> nodes = roots;

            for (int i = 0; i < parts.Length; i++)
            {
                foreach (SceneNode cNode in nodes)
                {
                    if (cNode.Name == parts[i])
                    {
                        if (i == parts.Length - 1)
                        {
                            node = cNode;
                            return true;
                        }
                        else
                        {
                            nodes = cNode.ChildNodes;
                            break;
                        }
                    }
                }

            }
            return false;
        }

        // removes nodes
        private void RemoveNodeFromTree(SceneNode node, SceneNode treeRootNode)
        {
            for (int i = treeRootNode.ChildNodes.Count - 1; i >= 0; i--)
            {
                if (treeRootNode.ChildNodes[i].ChildNodes.Count > 0)
                    RemoveNodeFromTree(node, treeRootNode.ChildNodes[i]);
                if (treeRootNode.ChildNodes[i].Path.StartsWith(node.Path))
                    treeRootNode.ChildNodes.Remove(treeRootNode.ChildNodes[i]);
            }
        }

        private void RemoveNodeFromForest(SceneNode node)
        {
            for (int i = roots.Count - 1; i >= 0; i--)
            {
                if (roots[i].ChildNodes.Count > 0)
                    RemoveNodeFromTree(node, roots[i]);
                if (roots[i].Path.StartsWith(node.Path))
                    roots.Remove(roots[i]);
            }
        }

        private void RemoveNode(SceneNode node)
        {
            /*string nodePath = node.Path;
            foreach (SceneNode child in node.ChildNodes)
            {
                RemoveNode(child);
            }
            map.Remove(nodePath);
            if (NodeRemoved != null)
            {
                NodeRemoved(this, nodePath, node);
            }*/
            RemoveNodeFromForest(node);
            UpdateMap();
        }

        public void RemoveNode(string path)
        {
            if (map.ContainsKey(path))
                RemoveNode(map[path]);
        }

        // Traverse Tree
        public void TraverseDepthFirst(ForestMapChange callback)
        {
            foreach (SceneNode node in roots)
            {
                TraverseDepthFirst(callback, node);
            }   
        }
        private void TraverseDepthFirst(ForestMapChange callback, SceneNode node)
        {
            callback(this, node.Path, node);
            foreach (SceneNode child in node.ChildNodes)
            {
                TraverseDepthFirst(callback, child);
            }
        }

        public void TraverseBreadthFirst(ForestMapChange callback, List<SceneNode> roots)
        {
            Queue<SceneNode> nodes = new Queue<SceneNode>();
            foreach (SceneNode node in roots)
            {
                nodes.Enqueue(node);
            }
            while (nodes.Count > 0)
            {
                SceneNode currentNode = nodes.Dequeue();

                callback(this, currentNode.Path, currentNode);
                foreach (SceneNode child in currentNode.ChildNodes)
                {
                    nodes.Enqueue(child);
                }
            }
        }

        public void TraverseBreadthFirst(ForestMapChange callback)
        {
            Queue<SceneNode> nodes = new Queue<SceneNode>();
            foreach (SceneNode node in roots)
            {
                nodes.Enqueue(node);
            }
            while (nodes.Count > 0)
            {
                SceneNode currentNode = nodes.Dequeue();

                callback(this, currentNode.Path, currentNode);
                foreach (SceneNode child in currentNode.ChildNodes)
                {
                    nodes.Enqueue(child);
                }
            }
        }
        

        // loads nodes

        
        
        
        // updates the nodes in the tree


        // draws nodes
        // fires events when nodes are added / deleted in the tree

        
        
        // Has a rename function and event to fire when a node is renamed ?
        // Editorflag ?


    }
}
