using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Editor.Editor;
using GameEngine.Level;
using GameEngine.ScreenSystem;
using GameEngine.Helpers;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using GameEngine;
using GameEngine.Xml;
using GameEngine.Cameras;

namespace Editor.Tools
{
    public class ScaleTool : Tool
    {
        public float ScaleSpeed = 0.001f;
        
        private bool working;
        private Vector2 startingPoint;
        private List<Vector3> originalScales = new List<Vector3>();
        private List<Vector3> originalOffsets = new List<Vector3>();
        private List<Vector3> originalPositions = new List<Vector3>();

        private Vector3 originalSelectionExtent;

        private Vector3 scaleAxes;

        private SpriteFont screenFont;

        private bool scaleX, scaleY, scaleZ;

        private ObjectNode[] selection;

        public ScaleTool(OrthoOrbitCamera camera, ToolHandler handler)
            : base(camera, handler)
        {
            this.Name = "ScaleTool";
            working = false;
        }


        public override void LoadContent(ContentManager content)
        {
            screenFont = content.Load<SpriteFont>("Fonts/Arial");            
        }

        public override void Update(GameTime gameTime)
        {
            
        }

        public override void Draw(DrawInfo drawInfo)
        {
            
        }

        public override void Draw2D(SpriteBatch spriteBatch, DrawInfo drawInfo)
        {
            string axesStr = "";
            if (scaleX)
                axesStr += "X";
            if (scaleY)
                axesStr += "Y";
            if (scaleZ)
                axesStr += "Z";

            spriteBatch.DrawString(screenFont, "Scaling along: " + axesStr, new Vector2(30f, 30f), Color.White);
        }

        public override void ToolStart()
        {
            
        }

        public override void ToolEnd()
        {
            
        }

        public override void HandleInput(GameTime gameTime)
        {
            EnableControls(enabledControls);

            scaleX = InputState.IsKeyDown(Keys.X);
            scaleY = (InputState.IsKeyDown(Keys.Y) || InputState.IsKeyDown(Keys.C));
            scaleZ = InputState.IsKeyDown(Keys.Z);

            // if nothing pressed, scale everything
            if (!(scaleX || scaleY || scaleZ))
            {
                scaleX = scaleY = scaleZ = true;
            }

            if (toolHandler.Selection.Count > 0)
            {
                if (!working)
                {
                    if (InputState.MousePressed(MouseButton.Left))
                    {
                        if (toolHandler.Selection.Intersects(camera.GetPickingRay(Cursor.Position)))
                        {
                            working = true;
                            startingPoint = Cursor.Position;

                            selection = toolHandler.Selection.Get3DSelection();

                            originalSelectionExtent = toolHandler.Selection.SelectionExtent;

                            originalScales.Clear();
                            originalOffsets.Clear();
                            originalPositions.Clear();
                            for (int i = 0; i < selection.Length; i++)
                            {
                                originalScales.Add(selection[i].Scale);
                                originalPositions.Add(selection[i].WorldPosition);
                                originalOffsets.Add(selection[i].WorldPosition - toolHandler.Selection.SelectionCenter);
                            }
                        }
                        else
                        {
                            toolHandler.SwitchTemporarilyToSelection(gameTime);
                        }
                    }
                }
                else
                {
                    if (InputState.MouseReleased(MouseButton.Left))
                    {
                        working = false;
                    }
                    else
                    {
                        float delta = startingPoint.Y - Cursor.Position.Y;

                        scaleAxes.X = scaleX ? 1 : 0;
                        scaleAxes.Y = scaleY ? 1 : 0;
                        scaleAxes.Z = scaleZ ? 1 : 0;

                        Vector3 scaleBy = scaleAxes * delta * ScaleSpeed + Vector3.One;

                        if (selection.Length == 1)
                        {
                            selection[0].Scale = originalScales[0] * scaleBy;
                        }
                        else
                        {
                            for (int i = 0; i < selection.Length; i++)
                            {
                                selection[i].Scale = originalScales[i] * scaleBy;
                                selection[i].Position = originalPositions[i] + originalOffsets[i] * scaleBy - originalOffsets[i];
                            }

                            toolHandler.Selection.SelectionExtent = originalSelectionExtent * scaleBy;
                        }
                    }
                }
            }
            else
            {
                if (InputState.MousePressed(MouseButton.Left))
                {
                    toolHandler.SwitchTemporarilyToSelection(gameTime);
                }
            }
        }
    }
}
