using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;
using System.Reflection;

namespace GameContentPipeline
{
    /// <summary>
    /// A simple binder that works in funny contexts.
    /// </summary>
    public class Binder : SerializationBinder
    {
        /// <summary>
        /// This must be overloaded to find the types for serialization.
        /// </summary>
        /// <param name="assemblyName">The name of the assembly a type is in.</param>
        /// <param name="typeName">The name of the type to find.</param>
        /// <returns>The type that was found.</returns>
        public override Type BindToType(string assemblyName, string typeName)
        {
            string shortAssemblyName = assemblyName.Split(',')[0];
            Assembly[] assemblies = AppDomain.CurrentDomain.GetAssemblies();

            // probe for a matching assembly name
            Assembly assembly =
                Array.Find(assemblies,
                            delegate(Assembly tester)
                            {
                                return shortAssemblyName == tester.FullName.Split(',')[0];
                            });

            // Get the type from the assembly - note, this assumes the assembly
            // has been loaded and is resident. 
            Type result = assembly.GetType(typeName);

            // We didn't find the type in the assembly, it's probably
            // a funky generic.
            if (result == null)
            {
                result = Type.GetType(typeName);
            }

            return result;
        }
    }
}
