using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Content.Pipeline;
using System.Xml;
using Microsoft.Xna.Framework.Content.Pipeline.Serialization.Compiler;
using GameEngine.Xml;
using GameEngine.Helpers;
using System.IO;
using System.Linq;
using GameEngine.LevelGeneration;

namespace GameContentPipeline
{
    [ContentImporter(".xml", DisplayName= "SP1 - XML")]
    public class XmlLevelImporter : ContentImporter<XmlDocument>
    {
        public override XmlDocument Import(string filename, ContentImporterContext context)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(filename);
            return doc;
        }
    }
    
    [ContentImporter(".xml", DisplayName = "SP1 - Tileset Importer")]
    public class XmlTilesetImporter : ContentImporter<TilesetWrapper>
    {
        public override TilesetWrapper Import(string filename, ContentImporterContext context)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(filename);
            List<SceneNode> nodes = XmlTest.LoadTest(doc);

            TilesetWrapper wrapper = new TilesetWrapper();
            wrapper.Tilesets = doc;

            foreach (var node in nodes)
            {
                TilesetNode n = node as TilesetNode;
                if (n == null)
                    continue;
                if (!wrapper.Tiles.ContainsKey(n.Name))
                    wrapper.Tiles.Add(n.Name, new Dictionary<TileType, List<XmlDocument>>());
                if (!wrapper.SpecialTiles.ContainsKey(n.Name))
                    wrapper.SpecialTiles.Add(n.Name, new Dictionary<SpecialTileNodeType, List<XmlDocument>>());
                foreach (var file in Directory.GetFiles(Path.Combine(Path.GetDirectoryName(filename), n.Name), "*.xml", SearchOption.AllDirectories))
                {
                    try {
                        XmlDocument t = new XmlDocument();
                        t.Load(file);
                        context.AddDependency(file);
                        List<SceneNode> tNodes = XmlTest.LoadTest(t);
                        TileInfoNode tn = tNodes.FirstOrDefault(x => x is TileInfoNode) as TileInfoNode;
                        if (tn == null)
                        {
                            context.Logger.LogWarning("", new ContentIdentity(Path.GetFullPath(file)), "Could not add tile: Xml does not contain a TileInfoNode");
                            continue;
                        }
                        if (tn.SpecialType == SpecialTileNodeType.None)
                        {
                            if (!wrapper.Tiles[n.Name].ContainsKey(tn.TileType))
                                wrapper.Tiles[n.Name].Add(tn.TileType, new List<XmlDocument>());
                            wrapper.Tiles[n.Name][tn.TileType].Add(t);
                        }
                        else
                        {
                            if (!wrapper.SpecialTiles[n.Name].ContainsKey(tn.SpecialType))
                                wrapper.SpecialTiles[n.Name].Add(tn.SpecialType, new List<XmlDocument>());
                            wrapper.SpecialTiles[n.Name][tn.SpecialType].Add(t);
                        }
                    } 
                    catch(Exception e) 
                    {
                        context.Logger.LogWarning("", new ContentIdentity(Path.GetFullPath(file)), "Could not add tile: {0}", e.Message);
                    }
                }

            }


            return wrapper;
        }
    }

    [ContentTypeWriter]
    public class XmlLevelWriter : ContentTypeWriter<XmlDocument>
    {
        protected override void Write(ContentWriter output, XmlDocument value)
        {
            output.Write(value.InnerXml);
        }

        public override string GetRuntimeReader(Microsoft.Xna.Framework.TargetPlatform targetPlatform)
        {
            return typeof(XmlLevelReader).AssemblyQualifiedName;
        }
    }
}
