using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;

namespace GameEngine.Animation
{
    public class AnimationData
    {
        public bool Skinned { get; set; }
        public Matrix[] BindPoses { get; set; }
        public Matrix[] InverseBindPoses { get; set; }
        public int[] SkeletonHierarchy { get; set; }
        public Dictionary<string, AnimationTrack> Tracks { get; set; }
        public Dictionary<string, int> BoneMap { get; set; }
        public int[] ModelBoneToSkinMap { get; set; }
        public int[] SkinBoneToModelMap;

        public AnimationData()
        {
            Tracks = new Dictionary<string, AnimationTrack>();
            BoneMap = new Dictionary<string, int>();
        }
    }

    public class AnimationTrack
    {
        public TimeSpan Duration { get; set; }
        public List<Keyframe> Keyframes { get; set; }

        public AnimationTrack(TimeSpan duration, List<Keyframe> keyframes)
        {
            this.Duration = duration;
            this.Keyframes = keyframes;
        }

    }

    public class Keyframe
    {
        public int Bone;
        public TimeSpan Time;
        public Matrix Transform;

        public Keyframe(int bone, TimeSpan time, Matrix transform)
        {
            this.Bone = bone;
            this.Time = time;
            this.Transform = transform;
        }
    }

    public class AnimationDataReader : ContentTypeReader<AnimationData>
    {
        protected override AnimationData Read(ContentReader input, AnimationData existingInstance)
        {
            AnimationData data = new AnimationData();

            data.Skinned = input.ReadBoolean();
            data.Tracks = input.ReadObject<Dictionary<string, AnimationTrack>>();

            if (data.Skinned)
            {
                data.BindPoses = input.ReadObject<Matrix[]>();
                data.InverseBindPoses = input.ReadObject<Matrix[]>();
                data.SkeletonHierarchy = input.ReadObject<int[]>();
                data.BoneMap = input.ReadObject<Dictionary<string, int>>();
                data.ModelBoneToSkinMap = input.ReadObject<int[]>();

            }
            return data;
        }
    }

    public class AnimationTrackReader : ContentTypeReader<AnimationTrack>
    {
        protected override AnimationTrack Read(ContentReader input, AnimationTrack existingInstance)
        {
            TimeSpan duration = input.ReadObject<TimeSpan>();
            List<Keyframe> keyframes = input.ReadObject<List<Keyframe>>();

            return new AnimationTrack(duration, keyframes);
        }
    }

    public class KeyframeReader : ContentTypeReader<Keyframe>
    {
        protected override Keyframe Read(ContentReader input, Keyframe existingInstance)
        {
            int bone = input.ReadObject<int>();
            TimeSpan time = input.ReadObject<TimeSpan>();
            Matrix transform = input.ReadObject<Matrix>();

            return new Keyframe(bone, time, transform);
        }
    }

}
