using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace GameEngine.Cameras
{
    public abstract class Camera
    {
        public Vector3 Position;
        
        protected Matrix view;
        protected Matrix projection;
        protected BoundingFrustum frustrum = new BoundingFrustum(Matrix.Identity);
        public Vector3 Target;
        public Vector3 Up;

        protected Viewport viewPort;

        protected float nearPlane;
        protected float farPlane;

        public Camera()
        {
            FieldOfView = MathHelper.PiOver4;
            Target = Vector3.Zero;
            Up = Vector3.Up;
            Position = Vector3.UnitZ;
            FarPlane = 1000f;
            NearPlane = 1f;
        }

        public Camera(float fieldOfView, Vector3 position, Vector3 target, Vector3 up, Viewport viewPort, float nearPlane, float farPlane)
        {
            this.FieldOfView = fieldOfView;
            this.Position = position;
            this.Target = target;
            this.Up = up;
            this.viewPort = viewPort;
            this.nearPlane = nearPlane;
            this.farPlane = farPlane;
        }

        public abstract void UpdateCamera(GameTime gameTime);

        public Ray GetPickingRay(Vector2 screenPosition)
        {
            Vector3 nearSource = new Vector3(screenPosition, 0f);
            Vector3 farSource = new Vector3(screenPosition, 1f);

            Vector3 nearPoint = viewPort.Unproject(nearSource, projection, view, Matrix.Identity);

            Vector3 farPoint = viewPort.Unproject(farSource, projection, view, Matrix.Identity);

            Vector3 direction = farPoint - nearPoint;
            direction.Normalize();

            return new Ray(nearPoint, direction);
        }
                

        public Matrix View { get { return view; } }
        public Matrix Projection { get { return projection; } }
        public Viewport Viewport { get { return viewPort; } set { viewPort = value; } }
        public float NearPlane { get { return nearPlane; } set { nearPlane = value; } }
        public float FarPlane { get { return farPlane; } set { farPlane = value; } }
        public float FieldOfView { get; set; }
        public BoundingFrustum Frustrum { get { return frustrum; } }

    }
}
