using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace GameEngine.Cameras
{
    public class PerspectiveCamera
    {
        private float fieldOfView;

        public float FieldOfView
        {
            get { return fieldOfView; }
            set { fieldOfView = value; }
        }

        public Vector3 Target;

        public Vector3 Position;

        public Vector3 Up = Vector3.Up;

        public Matrix View;

        public Matrix Projection;



        private Viewport viewPort;

        public Viewport Viewport
        {
            get { return viewPort; }
            set { viewPort = value; }
        }

        private float nearPlane;

        public float NearPlane
        {
            get { return nearPlane; }
            set { nearPlane = value; }
        }

        private float farPlane;

        public float FarPlane
        {
            get { return farPlane; }
            set { farPlane = value; }
        }


        public PerspectiveCamera()
        {
            
        }

        public PerspectiveCamera(float fieldOfView, Vector3 position, Vector3 target, Vector3 up, Viewport vp, float nearPlane, float farPlane )
        {
            this.fieldOfView = fieldOfView;
            this.Position = position;
            this.Target = target;
            this.Up = up;
            this.viewPort = vp;
            this.nearPlane = nearPlane;
            this.farPlane = farPlane;
        }

        public float r;
        public double theta;
        public double phi;

        public void Orbit(float horizontalOffSet, float verticalOffSet)
        {
            UpdateSphericalCoordinates();


            theta += horizontalOffSet;
            phi += verticalOffSet;


            newZ = r * Math.Cos(theta) * Math.Sin(phi);
            newX = r * Math.Sin(theta) * Math.Sin(phi);
            newY = r * Math.Cos(phi);

            Position = new Vector3((float)newX, (float)newY, (float)newZ);
        }

        public double newX;
        public double newY;
        public double newZ;



        public void UpdateSphericalCoordinates()
        {
            r = Position.Length();
            theta = Math.Atan2(Position.X, Position.Z);
            phi = Math.Acos(Position.Y / r);

            newZ = r * Math.Cos(theta) * Math.Sin(phi);
            newX = r * Math.Sin(theta) * Math.Sin(phi);
            newY = r * Math.Cos(phi);
        }

        public void UpdateMatrices()
        {
            Matrix.CreateLookAt(ref Position, ref Target, ref Up, out View);
            Matrix.CreatePerspectiveFieldOfView(fieldOfView, Viewport.AspectRatio, nearPlane, farPlane, out Projection);
        }

    }
}
