﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using GameEngine;
using GameEngine.Helpers;
using GameEngine.Xml;
using Microsoft.Xna.Framework;

namespace GamePrototype
{
    public class GeometryDrawer
    {
        Effect gbuffer;
        VertexDeclaration declaration;

        public GeometryDrawer()
        {
        }

        public void Load(ContentManager content, GraphicsDevice device)
        {
            declaration = new VertexDeclaration(device, VertexPositionNormalTexture.VertexElements);
            gbuffer = content.Load<Effect>(EngineFX.RenderGBuffer);
        }

        public void Draw(DrawInfo di)
        {
            di.Gdis.Sort(GeometryDrawInfo.TextureComparison);
            Texture2D texture = null;
            
            gbuffer.Parameters["Projection"].SetValue(di.Camera.Projection);
            gbuffer.Parameters["View"].SetValue(di.Camera.View);
            di.GraphicsDevice.VertexDeclaration = declaration;
            gbuffer.Begin(SaveStateMode.None);
            foreach (var geom in di.Gdis)
            {

                if (texture != geom.texture)
                {
                    texture = geom.texture;
                    gbuffer.Parameters["Texture"].SetValue(texture);
                }
                gbuffer.Parameters["World"].SetValue(geom.world);

                gbuffer.CurrentTechnique.Passes[0].Begin();


                di.GraphicsDevice.Vertices[0].SetSource(geom.geometry.buffer, 0, VertexPositionNormalTexture.SizeInBytes);
                di.GraphicsDevice.Indices = geom.geometry.indices;
                geom.geometry.Draw(di.GraphicsDevice);

                gbuffer.CurrentTechnique.Passes[0].End();

            }
            gbuffer.End();

            Model m = null;
            di.Models.Sort(ModelNode.SortByAsset);
            foreach (var model in di.Models)
            {
                if (m != model.Model)
                {
                    if (m != null)
                    {
                        DrawModel(m, di);
                    }
                    m = model.Model;
                    for (int i = 0; i < m.Meshes.Count; i++)
                    {
                        if (m.Meshes[i].Tag == null)
                            m.Meshes[i].Tag = new List<ModelNode>();
                    }
                }

                for (int i = 0; i < m.Meshes.Count; i++)
                {
                    ModelMesh mesh = m.Meshes[i];
                    List<ModelNode> n = mesh.Tag as List<ModelNode>;
                    n.Add(model);
                }
            }


            
            if (m != null)
                DrawModel(m, di);
            di.Gdis.Clear();




            di.Models.Clear();
        }
        private void DrawModel(Model m, DrawInfo di) 
        {
            for (int i = 0; i < m.Meshes.Count; i++)
			{
                ModelMesh mesh = m.Meshes[i];
                List<ModelNode> n = mesh.Tag as List<ModelNode>;
                
                if (n.Count > 0)
                {
                    di.GraphicsDevice.Indices = mesh.IndexBuffer;

                    //mesh.Draw(SaveStateMode.None);
                    for (int j = 0; j < mesh.MeshParts.Count; j++)
                    {
                        mesh.MeshParts[j].Effect.Begin(SaveStateMode.None);
                        di.GraphicsDevice.VertexDeclaration = mesh.MeshParts[j].VertexDeclaration;
                        di.GraphicsDevice.Vertices[0].SetSource(mesh.VertexBuffer, mesh.MeshParts[j].StreamOffset, mesh.MeshParts[j].VertexStride);


                        foreach (var node in n)
                        {
                            Matrix world;

                            if (!node.HasAnimation)
                            {
                                world = node.StaticManipulator.GetWorldTransformation(mesh.ParentBone.Index) * node.World;
                            }
                            else
                            {
                                if (node.AnimationPlayer.Data.Skinned && node.AnimationPlayer.Data.ModelBoneToSkinMap[mesh.ParentBone.Parent.Index] == -1)
                                {
                                    world = node.World;
                                }
                                else
                                {
                                    world = node.AnimationPlayer.WorldTransforms[mesh.ParentBone.Parent.Index] * node.World;
                                }
                            }
                            mesh.MeshParts[j].Effect.Parameters["Projection"].SetValue(di.Camera.Projection);
                            mesh.MeshParts[j].Effect.Parameters["View"].SetValue(di.Camera.View);
                            mesh.MeshParts[j].Effect.Parameters["World"].SetValue(world);

                            node.ConfigureEffect(mesh, j);
                            
                            if (node.HasAnimation && node.AnimationPlayer.Data.Skinned && node.AnimationPlayer.Data.ModelBoneToSkinMap[mesh.ParentBone.Parent.Index] == -1)
                            {
                                mesh.MeshParts[j].Effect.Parameters["Bones"].SetValue(node.AnimationPlayer.SkinTransforms);
                            }


                            mesh.MeshParts[j].Effect.CurrentTechnique.Passes[0].Begin();
                            di.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.TriangleList, mesh.MeshParts[j].BaseVertex, 0, mesh.MeshParts[j].NumVertices, mesh.MeshParts[j].StartIndex, mesh.MeshParts[j].PrimitiveCount);
                            mesh.MeshParts[j].Effect.CurrentTechnique.Passes[0].End();
                        }
                        mesh.MeshParts[j].Effect.End();
                    }

                
                }
                n.Clear();

			}
        }
    }
}
