﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using GameEngine.Xml;

namespace GameEngine.Helpers
{
    public struct BoundingCircle
    {
        private float radius;
        public float Radius { get { return radius; } set { radius = value; } }

        private Vector2 center;
        public Vector2 Center { get { return center; } set { center = value; } }

        public BoundingCircle(Vector2 center, float radius)
        {
            this.center = center;
            this.radius = radius;
        }

        public bool Intersects(BoundingCircle other, out Vector2 resolve)
        {
            Vector2 centerDiff = other.Center - center;
            float distSquared = centerDiff.LengthSquared();
            float radiSum = other.Radius + radius;
            resolve = Vector2.Zero;

            if (distSquared == 0)
                return true;

            if (distSquared < radiSum * radiSum)
            {
                float dist = (float)Math.Sqrt(distSquared);
                resolve = (radiSum - dist) * (centerDiff/dist);
                
                return true;
            }
            return false;
        }

        public bool Intersects(BoundingCircle other, Vector2 lastMove, out Vector2 resolve)
        {
            Vector2 centerDiff = other.Center - center;
            float distSquared = centerDiff.LengthSquared();
            float radiSum = other.Radius + radius;
            resolve = Vector2.Zero;

            if (distSquared == 0)
            {
                if(lastMove != Vector2.Zero)
                    resolve = -1 * Vector2.Normalize(lastMove) * radius;
                return true;
            }

            if (distSquared < radiSum * radiSum)
            {
                float dist = (float)Math.Sqrt(distSquared);
                resolve = (radiSum - dist) * (centerDiff / dist);

                return true;
            }
            return false;
        }

        public bool Intersects(CollisionRay ray, out Vector2 colNormal)
        {
            float colDist;
            if (ray.Intersects(this, out colDist))
            {
                colNormal = (ray.Position + ray.Direction * colDist - center) / radius;
                return true;
            }
            else
            {
                colNormal = Vector2.Zero;
                return false;
            }
        }
        
        public bool Intersects(PolygonNode poly, out Vector2 resolve) 
        {
            if (poly.Intersects(this, Vector2.Zero, out resolve))
            {
                resolve *= -1;
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
