using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using GameEngine.Cameras;

namespace GameEngine.Helpers
{
    static public class Cursor
    {
        private static float cursorSpeed = 250f;

        public static float CursorSpeed
        {
            get { return cursorSpeed; }
            set { cursorSpeed = value; }
        }

        private static Texture2D cursorTexture;

        private static Vector2 position;

        public static Vector2 Position
        {
            get { return position; }
            set { position = value; }
        }

        private static Grid xyGrid;

        public static Grid XyGrid
        {
            get { return xyGrid; }
        }

        private static Grid xzGrid;
        public static Grid XzGrid
        {
            get { return xzGrid; }
        }

        private static Plane xyPlane = new Plane(new Vector3(0, 0, 0), new Vector3(1, 0, 0), new Vector3(0, 1, 0));
        private static Plane xzPlane = new Plane(new Vector3(0, 0, 0), new Vector3(1, 0, 0), new Vector3(0, 0, 1));

        private static Camera camera;

        public static void Initialize(Texture2D texture, Grid xyGrid_in, Grid xzGrid_in, Camera camera_in)
        {
            cursorTexture = texture;
            position = Vector2.Zero;
            xyGrid = xyGrid_in;
            xzGrid = xzGrid_in;
            camera = camera_in;
        }

        public static void Update()
        {
            MouseState mouseState = Mouse.GetState();
            position.X = mouseState.X;
            position.Y = mouseState.Y;
        }

        public static void Draw2D(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(cursorTexture, Position, null, Color.White, 0.0f, Vector2.Zero, 1.0f, SpriteEffects.None, 0.0f);
        }

        // XY plane picking

        public static Vector2? GetXYPlanePosition() { return GetXYPlanePosition(position); }
        public static Vector2? GetXYPlanePosition(Vector2 fromPosition) { return GetXYPlanePosition(camera.GetPickingRay(fromPosition)); }
        public static Vector2? GetXYPlanePosition(Ray ray) 
        {
            Vector3? retVal = PlaneRayIntersectionPoint(ray, xyPlane);
            if (retVal == null)
                return null;
            else
                return new Vector2(retVal.Value.X, retVal.Value.Y); 
        }
        public static Vector2? GetXYPlanePosition(Vector2 fromPosition, bool snap)
        {
            Vector2? result = GetXYPlanePosition(position);
            if (snap && result != null)
            {
                return xyGrid.Snap(result.Value);
            }

            return result;
        }
        public static Vector2? GetXYPlanePosition(Ray ray, bool snap)
        {
            Vector2? result = GetXYPlanePosition(ray);
            if (snap && result != null)
            {
                return xyGrid.Snap(result.Value);
            }

            return result;
        }
        public static Vector2? GetXYPlanePosition(bool snap)
        {
            Vector2? result = GetXYPlanePosition();
            if (snap && result != null)
            {
                return xyGrid.Snap(result.Value);
            }

            return result;
        }


        public static Vector3? PlaneRayIntersectionPoint(Ray ray, Plane p)
        {
            Nullable<float> intersectDistance = ray.Intersects(p);
            if (intersectDistance != null)
            {
                return ray.Position + ray.Direction * intersectDistance.Value;
            }
            else
            {
                return null;
            }
        }

        // Z-plane picking
        public static Vector2? GetXZPlanePosition()
        {
            return GetXZPlanePosition(position);
        }

        public static Vector2? GetXZPlanePosition(Vector2 fromPosition)
        {
            return GetXZPlanePosition(camera.GetPickingRay(fromPosition));
        }

        public static Vector2? GetXZPlanePosition(Ray ray) 
        {
            Vector3? retVal = PlaneRayIntersectionPoint(ray, xzPlane);
            if (retVal == null)
                return null;
            else
                return new Vector2(retVal.Value.X, retVal.Value.Z);
        }
        public static Vector2? GetXZPlanePosition(Ray ray, bool snap)
        {
            Vector2? result = GetXZPlanePosition(ray);
            if (snap && result != null)
            {
                return xzGrid.Snap(result.Value);
            }

            return result;
        }
        public static Vector2? GetXZPlanePosition(Vector2 fromPosition, bool snap)
        {
            Vector2? result = GetXZPlanePosition(position);
            if (snap && result != null)
            {
                return xzGrid.Snap(result.Value);
            }

            return result;
        }

        public static Vector2? GetXZPlanePosition(bool snap)
        {
            Vector2? result = GetXZPlanePosition();
            if (snap && result != null)
            {
                return xzGrid.Snap(result.Value);
            }

            return result;
        }
    }
}
