using System;
using System.Collections.Generic;
using System.Text;
using GameEngine.Cameras;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace GameEngine.Helpers
{
    public class OrientationGizmo
    {
        private VertexBuffer buffer;
        private VertexDeclaration declaration;
        private Effect primitiveEffect;

        private Vector3 scale = Vector3.One;

        private Color xColor = Color.Red;
        private Color yColor = Color.Green;
        private Color zColor = Color.Blue;

        private Matrix world;        
        private VertexPositionColor[] vertices;
        private float size;

        public OrientationGizmo()
        {
            
        }

        public void Initialize(GraphicsDevice graphicsDevice, Camera camera, Effect primitiveEffect)
        {
            Unload();
            
            this.primitiveEffect = primitiveEffect;
                        
            buffer = new VertexBuffer(graphicsDevice, typeof(VertexPositionColor), 6, BufferUsage.WriteOnly);
            vertices = new VertexPositionColor[6];

            buffer.SetData(vertices);

            declaration = new VertexDeclaration(graphicsDevice, VertexPositionColor.VertexElements);
        }



        public void Update(Camera camera)
        {
            

            world = Matrix.CreateTranslation(Vector3.Zero);
            size = 10;

            Vector3 camDir = Vector3.Normalize(Vector3.Subtract(camera.Target, camera.Position));

            Vector3 offSet = camera.Position + camDir * 100;
            
            Vector3 side = Vector3.Cross(camDir, camera.Up);
            side.Normalize();

            offSet += side * 100;

            Vector3 camUp = Vector3.Cross(side, camDir);
            camUp.Normalize();

            offSet += camUp * (100 / camera.Viewport.AspectRatio);

            if (vertices != null)
            {
                vertices[0] = new VertexPositionColor(offSet, xColor);
                vertices[1] = new VertexPositionColor(offSet + new Vector3(size, 0, 0), xColor);

                vertices[2] = new VertexPositionColor(offSet, yColor);
                vertices[3] = new VertexPositionColor(offSet + new Vector3(0, size, 0), yColor);

                vertices[4] = new VertexPositionColor(offSet, zColor);
                vertices[5] = new VertexPositionColor(offSet + new Vector3(0, 0, size), zColor);

                buffer.SetData(vertices);                                
            }
        }

        public void Draw(GraphicsDevice graphics, Camera camera)
        {
            graphics.VertexDeclaration = declaration;
            graphics.Vertices[0].SetSource(buffer, 0, VertexPositionColor.SizeInBytes);

            primitiveEffect.Parameters["WorldViewProj"].SetValue((world * camera.View) * camera.Projection);
            primitiveEffect.Parameters["Color"].SetValue(Color.TransparentWhite.ToVector4());
            
            primitiveEffect.Begin();

            foreach (EffectPass pass in primitiveEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                graphics.DrawPrimitives(PrimitiveType.LineList, 0, 4);
                pass.End();
            }
            primitiveEffect.End();
        }

        private void Unload()
        {
            if (buffer != null)
            {
                buffer.Dispose();
                buffer = null;
            }
            if (declaration != null)
            {
                declaration.Dispose();
                declaration = null;
            }

        }


        
    }
}
