﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using GamePrototype;

namespace GameEngine.Helpers
{
    public class PolygonWall : IDisposable
 {
        VertexBuffer buffer;
        IndexBuffer indices;
        VertexDeclaration decl;
        int pointCountx2;

        private PolygonWall()
        {
        }

        ~PolygonWall()
        {
            Dispose();
        }

        public static PolygonWall Generate(GraphicsDevice device, List<Vector2> points, float height, Vector2 texOffset, Vector2 texSize, Vector4 color)
        {
            PolygonWall p = new PolygonWall();

            Color c = new Color(color);

            VertexPositionColorTexture[] walls = new VertexPositionColorTexture[points.Count * 2 + 2];
            int[] wallIndices = new int[points.Count * 2 + 2];

            float distance = texOffset.X;

            for (int i = 0; i < points.Count + 1; i++)
            {
                int ix = i % points.Count;
                
                if (i > 0)
                {
                    distance += Vector2.Distance(points[i - 1], points[ix]);
                }

                Vector3 top = new Vector3(points[ix].X, height, points[ix].Y);
                Vector3 bottom = new Vector3(points[ix].X, 0, points[ix].Y);

                int a = i * 2;
                int b = a + 1;

                Vector2 aUV = texSize != Vector2.Zero ? new Vector2(distance / texSize.X, (height + texOffset.Y) / texSize.Y) : new Vector2(i, 1);
                Vector2 bUV = texSize != Vector2.Zero ? new Vector2(distance / texSize.X, texOffset.Y / texSize.Y) : new Vector2(i, 0);

                walls[a] = new VertexPositionColorTexture(bottom, c, aUV);
                walls[b] = new VertexPositionColorTexture(top, c, bUV);

                wallIndices[a] = a;
                wallIndices[b] = b;
            }
            //wallIndices[walls.Length] = 0;
            //wallIndices[walls.Length + 1] = 1;

            p.buffer = new VertexBuffer(device, typeof(VertexPositionColorTexture), walls.Length, BufferUsage.WriteOnly);
            p.buffer.SetData(walls);

            p.indices = new IndexBuffer(device, typeof(int), wallIndices.Length, BufferUsage.WriteOnly);
            p.indices.SetData(wallIndices);

            p.decl = new VertexDeclaration(device, VertexPositionColorTexture.VertexElements);
            p.pointCountx2 = points.Count * 2;
            return p;
        }

        public void SetSource(GraphicsDevice device)
        {
            device.VertexDeclaration = decl;
            device.Vertices[0].SetSource(buffer, 0, VertexPositionColorTexture.SizeInBytes);
            device.Indices = indices;
        }

        public void Draw(GraphicsDevice device, Effect effect)
        {
            SetSource(device);
            effect.Begin();

            for (int i = 0; i < effect.CurrentTechnique.Passes.Count; i++)
            {
                effect.CurrentTechnique.Passes[i].Begin();

                Draw(device);

                effect.CurrentTechnique.Passes[i].End();
            }

            effect.End();
        }

        public void Draw(GraphicsDevice device)
        {
            device.DrawIndexedPrimitives(PrimitiveType.TriangleStrip, 0, 0, pointCountx2, 0, pointCountx2);
        }

        public void Dispose()
        {
            Util.NullDispose(ref buffer);
            Util.NullDispose(ref indices);
            Util.NullDispose(ref decl);
        }
        
    }
}
