﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using GamePrototype;

namespace GameEngine.Helpers
{
    public class PolygonWallNormals : Geometry
    {
        private PolygonWallNormals()
        {
            
        }

        ~PolygonWallNormals()
        {
            Dispose();
        }

        public static PolygonWallNormals Generate(GraphicsDevice device, List<Vector2> points, float height, Vector2 texOffset, Vector2 texSize, Vector4 color)
        {
            PolygonWallNormals p = new PolygonWallNormals();

            Color c = new Color(color);

            VertexPositionNormalTexture[] walls = new VertexPositionNormalTexture[points.Count * 4];
            int[] wallIndices = new int[points.Count * 6];

            float distance = texOffset.X;

            for (int i = 0; i < points.Count; i++)
            {
                int next = (i + 1) % points.Count;

                Vector3 top = new Vector3(points[i].X, height, points[i].Y);
                Vector3 bottom = new Vector3(points[i].X, 0, points[i].Y);

                Vector3 nTop = new Vector3(points[next].X, height, points[next].Y);
                Vector3 nBottom = new Vector3(points[next].X, 0, points[next].Y);

                Vector2 topUV = texSize != Vector2.Zero ? new Vector2(distance / texSize.X, texOffset.Y / texSize.Y) : new Vector2(0, 0);
                Vector2 bottomUV = texSize != Vector2.Zero ? new Vector2(distance / texSize.X, (height + texOffset.Y) / texSize.Y) : new Vector2(0, 1);

                distance += Vector2.Distance(points[i], points[next]);

                Vector2 nTopUV = texSize != Vector2.Zero ? new Vector2(distance / texSize.X, texOffset.Y / texSize.Y) : new Vector2(1, 0);
                Vector2 nBottomUV = texSize != Vector2.Zero ? new Vector2(distance / texSize.X, (height + texOffset.Y) / texSize.Y) : new Vector2(1, 1);

                int a = i * 4;

                Vector2 normal = (points[next] - points[i]).GetPerpendicular();
                Vector3 n = new Vector3(normal.X, 0, normal.Y);
                walls[a] = new VertexPositionNormalTexture(top, n, topUV);
                walls[a + 1] = new VertexPositionNormalTexture(nTop, n, nTopUV);
                walls[a + 2] = new VertexPositionNormalTexture(nBottom, n, nBottomUV);
                walls[a + 3] = new VertexPositionNormalTexture(bottom, n, bottomUV);

                int na = i * 6;
                wallIndices[na] = a;
                wallIndices[na + 1] = a + 1;
                wallIndices[na + 2] = a + 3;
                wallIndices[na + 3] = a + 1;
                wallIndices[na + 4] = a + 2;
                wallIndices[na + 5] = a + 3;
            }

            p.buffer = new VertexBuffer(device, typeof(VertexPositionNormalTexture), walls.Length, BufferUsage.WriteOnly);
            p.buffer.SetData(walls);

            p.indices = new IndexBuffer(device, typeof(int), wallIndices.Length, BufferUsage.WriteOnly);
            p.indices.SetData(wallIndices);

            p.decl = new VertexDeclaration(device, VertexPositionNormalTexture.VertexElements);
            p.vertexCount = p.primitiveCount = points.Count * 2;
            p.primitiveType = PrimitiveType.TriangleList;
            return p;
        }



        
    }
}
