using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

namespace GameEngine.Helpers
{
    public class SelectionFrustum
    {
        Plane near, far, top, bottom, left, right;

        public SelectionFrustum(Vector3 nearTopRight, Vector3 nearTopLeft, Vector3 nearBottomRight, Vector3 farBottomLeft, Vector3 farBottomRight, Vector3 farTopLeft)
        {
            near = new Plane(nearTopRight, nearBottomRight, nearTopLeft);
            far = new Plane(farBottomLeft, farBottomRight, farTopLeft);

            top = new Plane(nearTopRight, nearTopLeft, farTopLeft);
            bottom = new Plane(farBottomLeft, nearBottomRight, farBottomRight);

            left = new Plane(farTopLeft, nearTopLeft, farBottomLeft);
            right = new Plane(nearBottomRight, nearTopRight, farBottomRight);

        }


        public static float DistanceFromPointToPlane(ref Vector3 point, ref Plane plane)
        {
            return Vector3.Dot(Vector3.Normalize(plane.Normal), point) + plane.D;
        }

        public bool Contains(Vector3 point)
        {
            if (DistanceFromPointToPlane(ref point, ref near) < 0)
                return false;
            if (DistanceFromPointToPlane(ref point, ref far) < 0)
                return false;
            if (DistanceFromPointToPlane(ref point, ref top) < 0)
                return false;
            if (DistanceFromPointToPlane(ref point, ref bottom) < 0)
                return false;
            if (DistanceFromPointToPlane(ref point, ref left) < 0)
                return false;
            if (DistanceFromPointToPlane(ref point, ref right) < 0)
                return false;

            // If on the right side of all planes, must be inside the frustum
            return true;
        }

        public bool Intersects(OrientedBoundingBox obox)
        {
            if (obox.Intersects(near) < 0)
                return false;
            if (obox.Intersects(far) < 0)
                return false;
            if (obox.Intersects(top) < 0)
                return false;
            if (obox.Intersects(bottom) < 0)
                return false;
            if (obox.Intersects(left) < 0)
                return false;
            if (obox.Intersects(right) < 0)
                return false;

            // If none of the points are inside, return false
            return true;
        }

    }
}
