using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace GameEngine.Level
{
    public class Snapper
    {
        VertexBuffer buffer;
        VertexDeclaration decl;
        Effect primitiveEffect;
        IndexBuffer indices;
        float size;
        public Color Color = Color.Cyan;

        Matrix world;
        public Vector3 Position;
        public Vector3 Scale = Vector3.One;
        public Quaternion Rotation;

        public Snapper(float size)
        {
            this.size = size;
        }

        public void Draw(DrawInfo di)
        {

            di.GraphicsDevice.VertexDeclaration = decl;
            di.GraphicsDevice.Vertices[0].SetSource(buffer, 0, VertexPositionColor.SizeInBytes);
            di.GraphicsDevice.Indices = indices;
            primitiveEffect.Parameters["WorldViewProj"].SetValue((world * di.Camera.View) * di.Camera.Projection);
            primitiveEffect.Parameters["Color"].SetValue(Color.ToVector4());
            primitiveEffect.Begin();

            foreach (EffectPass pass in primitiveEffect.CurrentTechnique.Passes)
            {

                pass.Begin();
                di.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.LineList, 0, 0, 14, 0, 7 );
                di.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.LineStrip, 6, 0, 4, 14, 4);
                di.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.LineStrip, 10, 0, 4, 14, 4);
                pass.End();
            }
            primitiveEffect.End();
        }

        public void Initialize(GraphicsDevice GraphicsDevice, Effect primitivesEffect)
        {
            this.primitiveEffect = primitivesEffect;
            buffer = new VertexBuffer(GraphicsDevice, typeof(VertexPositionColor), 14, BufferUsage.WriteOnly);
            indices = new IndexBuffer(GraphicsDevice, typeof(short), 19, BufferUsage.WriteOnly);
            VertexPositionColor[] vertices = new VertexPositionColor[14];

            float d = size / 4f;

            vertices[0] = new VertexPositionColor(new Vector3(-size, 0, 0), Color.White);
            vertices[1] = new VertexPositionColor(new Vector3(size, 0, 0), Color.White);
            vertices[2] = new VertexPositionColor(new Vector3(0, 0, size), Color.White);
            vertices[3] = new VertexPositionColor(new Vector3(0, 0, -size), Color.White);
            vertices[4] = new VertexPositionColor(new Vector3(0, size, 0), Color.White);
            vertices[5] = new VertexPositionColor(new Vector3(0, -size, 0), Color.White);
            vertices[6] = new VertexPositionColor(new Vector3(-d, -d, -d), Color.White);
            vertices[7] = new VertexPositionColor(new Vector3(d, -d, -d), Color.White);
            vertices[8] = new VertexPositionColor(new Vector3(d, -d, d), Color.White);
            vertices[9] = new VertexPositionColor(new Vector3(-d, -d, d), Color.White);
            vertices[10] = new VertexPositionColor(new Vector3(-d, d, -d), Color.White);
            vertices[11] = new VertexPositionColor(new Vector3(d, d, -d), Color.White);
            vertices[12] = new VertexPositionColor(new Vector3(d, d, d), Color.White);
            vertices[13] = new VertexPositionColor(new Vector3(-d, d, d), Color.White);

            short[] indix = new short[] { 0, 1, 2, 3, 4, 5,    6, 10, 7, 11, 8, 12, 9, 13,             0, 1, 2, 3, 0, };

            indices.SetData(indix);
            buffer.SetData(vertices);

            decl = new VertexDeclaration(GraphicsDevice, VertexPositionColor.VertexElements);
        }

        public void Update()
        {
            world = Matrix.CreateFromQuaternion(Rotation) * Matrix.CreateScale(Scale) * Matrix.CreateTranslation(Position);//  ;

        }

        public void Unload()
        {
            if (buffer != null)
            {
                buffer.Dispose();
                buffer = null;
            }
            if (decl != null)
            {
                decl.Dispose();
                decl = null;
            }
            if (indices != null)
            {
                indices.Dispose();
                indices = null;
            }
        }
    }
}
