using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace GameEngine.Level
{
    public class StaticProp : GameObject
    {
        public Model Model;
        Matrix w;
        Matrix[] bones;

        public StaticProp(Model model)
        {
            this.Model = model;
            bones = new Matrix[model.Bones.Count];
            model.CopyAbsoluteBoneTransformsTo(bones);
        }

        public override void Update(GameTime time)
        {
            w = Matrix.CreateFromQuaternion(Rotation) * Matrix.CreateScale(Scale) * Matrix.CreateTranslation(Position);//  ;
            //w = Matrix.CreateTranslation(Position);
        }


        public override void Draw(GameTime time, DrawInfo di)
        {
            Matrix worldViewProj;
            
            foreach (ModelMesh mesh in Model.Meshes)
            {
                Matrix.Multiply(ref bones[mesh.ParentBone.Index], ref w, out worldViewProj);
                Matrix.Multiply(ref worldViewProj, ref di.View, out worldViewProj);
                Matrix.Multiply(ref worldViewProj, ref di.Projection, out worldViewProj);
                
                
                foreach (Effect e in mesh.Effects)
                {
                    if (e is BasicEffect)
                    {
                        BasicEffect be = e as BasicEffect;
                        be.World = bones[mesh.ParentBone.Index] * w;
                        be.View = di.View;
                        be.Projection = di.Projection;
                        be.EnableDefaultLighting();
                    }else{
                        e.Parameters["WorldViewProj"].SetValue(worldViewProj);
                    }
                }

                mesh.Draw();
            }


        }

        public static StaticProp FromData(GameEngine.Data.StaticPropData prop, Model m)
        {
            StaticProp newProp = new StaticProp(m);
            newProp.Name = prop.ID;
            newProp.Position = prop.Position;
            newProp.Rotation = prop.Rotation;
            newProp.Scale = prop.Scale;
            return newProp;
        }

    }
}
