﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace GameEngine.Physics
{
    public class VerletDebugger
    {
        private BasicEffect basicEffect;
        private VerletSystem system;
        private VertexBuffer vertexBuffer;
        private VertexDeclaration vertexDeclaration;
        private VertexPositionColor[] vertices;
        private List<VerletConstraint> visibleConstraints;

        public VerletDebugger(VerletSystem system)
        {
            this.system = system;
            visibleConstraints = new List<VerletConstraint>();

            foreach (VerletConstraint constraint in system.Constraints)
            {
                if (constraint.Visible)
                {
                    visibleConstraints.Add(constraint);
                }
            }
        }

        public void Initialize(GraphicsDevice graphics)
        {
            basicEffect = new BasicEffect(graphics, null);

            vertexDeclaration = new VertexDeclaration(graphics, VertexPositionColor.VertexElements);
            vertexBuffer = new VertexBuffer
                (graphics, typeof(VertexPositionColor), visibleConstraints.Count * 2, BufferUsage.WriteOnly);

            vertices = new VertexPositionColor[visibleConstraints.Count * 2];

            SetVertices();
        }

        private void SetVertices()
        {
            for (int i = 0; i < visibleConstraints.Count; i++)
            {
                vertices[i * 2] = new VertexPositionColor(system.Verlets[visibleConstraints[i].A].Position, Color.White);
                vertices[i * 2 + 1] = new VertexPositionColor(system.Verlets[visibleConstraints[i].B].Position, Color.White);

                vertexBuffer.SetData(vertices);
            }
        }

        public void Draw(GraphicsDevice graphics, Matrix view, Matrix projection)
        {
            SetVertices();

            Matrix world = Matrix.CreateTranslation(Vector3.Zero);

            graphics.VertexDeclaration = vertexDeclaration;
            graphics.Vertices[0].SetSource(vertexBuffer, 0, VertexPositionColor.SizeInBytes);

            basicEffect.Parameters["World"].SetValue(world);
            basicEffect.Parameters["View"].SetValue(view);
            basicEffect.Parameters["Projection"].SetValue(projection);
            basicEffect.Parameters["DiffuseColor"].SetValue(Vector3.One);
            basicEffect.Begin();

            foreach (EffectPass pass in basicEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                graphics.DrawPrimitives(PrimitiveType.LineList, 0, visibleConstraints.Count);
                pass.End();
            }

            basicEffect.End();
        }
    }
}
