﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework;

namespace GameEngine.ScreenSystem
{
    public static class AudioManager
    {
        public static Dictionary<string, SoundEffect> SoundEffects = new Dictionary<string,SoundEffect>();
        public static Dictionary<string, Song> Music = new Dictionary<string, Song>();
        static AudioListener al = new AudioListener();
        static AudioEmitter emitter = new AudioEmitter();


        static AudioManager()
        {
               
            SoundEffects = new Dictionary<string, SoundEffect>();
            al.Position = Vector3.Zero;
            al.Up = Vector3.Up;
            al.Velocity = Vector3.UnitX;
            emitter.Up = Vector3.Up;
            emitter.Velocity = Vector3.UnitX;
            emitter.Position = Vector3.Zero;
        }

        public static void StopMusic()
        {
            MediaPlayer.Stop();
        }

        public static void PlayMusic(string song)   
        {
            MediaPlayer.Volume = 0.5f;
            MediaPlayer.IsRepeating = true;
            try
            {
                MediaPlayer.Play(Music[song]);
            }
            catch (Exception) { }
        }

        public static void LoadSoundEffect(string path, ContentManager content)
        {
            if (!SoundEffects.ContainsKey(path))
            {
                try
                {
                    SoundEffects.Add(path, content.Load<SoundEffect>(path));
                }
                catch (InvalidOperationException)
                {
                }
            }            
        }

        public static void LoadSong(string path, ContentManager content)
        {
            if (!Music.ContainsKey(path))
            {
                try
                {
                    Music.Add(path, content.Load<Song>(path));
                }
                catch (InvalidOperationException)
                {
                }
            }
        }

        public static void LoadSoundEffect(ContentManager content, params string[] paths)
        {
            for (int i = 0; i < paths.Length; i++)
            {
                LoadSoundEffect(paths[i], content);    
            }            
        }

        public static void LoadSongs(ContentManager content, params string[] paths)
        {
            for (int i = 0; i < paths.Length; i++)
            {
                LoadSong(paths[i], content);
            }
        }

        public static void PlaySoundEffect(string path, float volume, float pitch, Vector3 pos)
        {
            try
            {
                emitter.Position = pos;
                emitter.Forward = -pos;
                if (SoundEffects[path].Play3D(al, emitter, volume, pitch, false).State == SoundState.Playing)
                {
                    return;
                }
            }
            catch (Exception)
            {
            }
        }

        public static void PlaySoundEffect(string path, float volume, float pitch)
        {
            try
            {
                if (SoundEffects[path].Play(volume, pitch, 0f, false).State == SoundState.Playing)
                {
                    return;
                }
            }
            catch (Exception)
            {
            }
        }

        public static void PlaySoundEffect(string path, float volume)
        {
            PlaySoundEffect(path, volume, 0, Vector3.Zero);
        }
    }
}
