﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.Helpers;
using Microsoft.Xna.Framework;

namespace GameEngine.Xml
{
    public class DynamicCircleNode : ObjectNode
    {
        public float radius;
        public Vector2 resolveOffset;
        public Vector2 LastMovement { 
            get 
            {
                Vector3 tmp = (Position + new Vector3(resolveOffset.X, 0, resolveOffset.Y)) - (LastPosition);
                return new Vector2(tmp.X, tmp.Z);
            } 
            private set 
            { 
            } 
        }
        public Vector3 LastPosition;

        public bool PassThrough { get; set; }

        public DynamicCircleNode(string name) : base(name)
        {
            radius = 10f;
            resolveOffset = Vector2.Zero;
            LastMovement = Vector2.Zero;
            LastPosition = Vector3.Zero;
            PassThrough = false;
        }

        public BoundingCircle GetCollisionCircle()
        {
            return new BoundingCircle(new Vector2(Position.X, Position.Z) + resolveOffset, radius);
        }

        public Vector2 ResolveOffset { get { return resolveOffset; } set { resolveOffset = value; } }

        public BoundingCircle ResolveCircleCollision(Vector2 resolve)
        {
            resolveOffset += resolve;
            return GetCollisionCircle();
        }

        public BoundingCircle ResolveWallCollision(Vector2 resolve)
        {
            /*
            if (Vector2.Dot(resolveOffset, resolve) < -0.001f)
            {
                Vector2 oldDirection = new Vector2(-resolveOffset.Y, resolveOffset.X);
                float d = Vector2.Dot(resolve, oldDirection);
                Vector2 newResolve= oldDirection * resolve.LengthSquared() / d;
                resolveOffset += newResolve;
            }
            else
            {
                resolveOffset += resolve;
            }
             */
            resolveOffset += resolve;
            return GetCollisionCircle();
        }

        public bool RecentCollision = false;
        public Vector2 oldResolve;
        public void CommitCollision()
        {
            
            if (resolveOffset != Vector2.Zero)
            {
                Position += new Vector3(resolveOffset.X, 0, resolveOffset.Y);
                oldResolve = resolveOffset;
                resolveOffset = Vector2.Zero;
                RecentCollision = true;
            } 
        }

        protected virtual void MoveToOtherNode(LevelTileNode node)
        {
            ((LevelTileNode)Parent).RemoveDynamicNode(this);
            node.AddDynamicNode(this);
        }

        public void CheckTileChange()
        {
            if (TilesetManager.CurrentTileset == null)
                return;

            float halfTile = TilesetManager.CurrentTileset.TileSize / 2f;

            // TODO: Fix så de ikke ryger ind i null-tiles!

            LevelTileNode myTile = Parent as LevelTileNode;
            if (Position.X < -halfTile)
            {
                if (myTile.Tile.TileLeft != null && !myTile.Tile.TileLeft.SceneNode.IsEmpty)
                {
                    MoveToOtherNode(myTile.Tile.TileLeft.SceneNode);
                    Position.X += TilesetManager.CurrentTileset.TileSize;
                    LastPosition.X += TilesetManager.CurrentTileset.TileSize;
                }
                else
                {
                    Position = Vector3.Zero;
                }
            }
            if (Position.X > halfTile)
            {
                if (myTile.Tile.TileRight != null && !myTile.Tile.TileRight.SceneNode.IsEmpty)
                {
                    MoveToOtherNode(myTile.Tile.TileRight.SceneNode);
                    Position.X -= TilesetManager.CurrentTileset.TileSize;
                    LastPosition.X -= TilesetManager.CurrentTileset.TileSize;
                }
                else
                {
                    Position = Vector3.Zero;
                }
            }
            if (Position.Z < -halfTile)
            {
                if (myTile.Tile.TileAbove != null && !myTile.Tile.TileAbove.SceneNode.IsEmpty)
                {
                    MoveToOtherNode(myTile.Tile.TileAbove.SceneNode);
                    Position.Z += TilesetManager.CurrentTileset.TileSize;
                    LastPosition.Z += TilesetManager.CurrentTileset.TileSize;
                }
                else
                {
                    Position = Vector3.Zero;
                }
            }
            if (Position.Z > halfTile)
            {
                if (myTile.Tile.TileBelow != null && !myTile.Tile.TileBelow.SceneNode.IsEmpty)
                {
                    MoveToOtherNode(myTile.Tile.TileBelow.SceneNode);
                    Position.Z -= TilesetManager.CurrentTileset.TileSize;
                    LastPosition.Z -= TilesetManager.CurrentTileset.TileSize;
                }
                else
                {
                    Position = Vector3.Zero;
                }
            }
        }

        public override void Update(GameTime dt, DrawInfo drawInfo)
        {
            LastPosition = Position;
            base.Update(dt, drawInfo);
            CheckTileChange();
            RecentCollision = false;
        }
    }
}
