﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using Microsoft.Xna.Framework;

namespace GameEngine.Xml
{
    public class SpawnPointNode : DynamicCircleNode
    {
        public enum SpawnType
        {
            Random,
            Player,
            Health,
            Ammunition,
            BigAmmo,
            Weapon,
            BigWeapon,
            Zombie,
            ZombieLauncher,
            ZombieLauncherInvisible,
            BigZombie
        }


        private SpawnType spawnPointType;
        [Category("SpawnPoint")]
        [DisplayName("Spawnpoint Type")]
        [TypeConverter(typeof(SpawnType))]
        public SpawnType SpawnPointType
        {
            get { return this.spawnPointType; }
            set
            {
                if (this.spawnPointType == value)
                    return;

                this.spawnPointType = value;
            }
        }

        private float spawnLikelyhood;
        [Category("SpawnPoint")]
        [DisplayName("Likelyhood of spawn (0 to 1)")]
        public float SpawnLikelyhood
        {
            get { return this.spawnLikelyhood; }
            set
            {
                if (this.spawnLikelyhood == value)
                    return;

                this.spawnLikelyhood = MathHelper.Clamp(value, 0f, 1f);
            }
        }

        [Browsable(false)]
        public ModelNode[] Models { get; set; }

        [Browsable(false)]
        public float RespawnTime { get; set; }

        [Browsable(false)]
        public bool HasObject = true;

        private Object oneItem = null;

        private bool expended = false;

        public int ItemIntValue { get; set; }

        public SpawnPointNode(string name) : base(name)
        {
            PassThrough = true;
            // Default respawn time of -1: Will provide one object then no more unless respawntime is changed
            RespawnTime = -1;
            ItemIntValue = 1;
            SpawnLikelyhood = 1;
        }

        public void SetModels(ModelNode modelNode, ModelNode ready, ModelNode waiting, ModelNode longtime)
        {
            if (Models == null)
                Models = new ModelNode[4];

            Models[0] = modelNode;
            Models[1] = ready;
            Models[2] = waiting;
            Models[3] = longtime;

            for (int i = 0; i < Models.Length; i++)
            {
                // Make sure we don't already have the same models as childnodes
                this.ChildNodes.Remove(Models[i]);
                // And add them again
                this.ChildNodes.Add(Models[i]);
            }
        }


        public override void PopulateXml(System.Xml.XmlElement element, System.Xml.XmlDocument document)
        {
            base.PopulateXml(element, document);

            element.SetAttribute("SpawnpointType", this.SpawnPointType.ToString());
            element.SetAttribute("SpawnLikelyhood", this.SpawnLikelyhood.ToString());
        }
        public override void SetFromXml(System.Xml.XmlElement element)
        {
            base.SetFromXml(element);

            if (element.HasAttribute("SpawnpointType"))
                this.SpawnPointType = (SpawnType)Enum.Parse(typeof(SpawnType), element.Attributes["SpawnpointType"].Value);
            if (element.HasAttribute("SpawnLikelyhood"))
                this.SpawnLikelyhood = (float)Convert.ToDouble(element.Attributes["SpawnLikelyhood"].Value);
        }

        public bool TryTakeObject(float timeToNextSpawn, out Object specificObject)
        {
            specificObject = null;

            if (oneItem == null)
            {
                if (HasObject)
                {
                    RespawnTime = timeToNextSpawn;
                    if (timeToNextSpawn != 0)
                        HasObject = false;
                    return true;
                }
            }
            else
            {
                specificObject = oneItem;
                
                // Just return this one object and no more
                timeToNextSpawn = -1;
                HasObject = false;
                oneItem = null;
                expended = true;

                return true;
            }

            return false;
        }

        public void GiveBackObject()
        {
            RespawnTime = 0;
            HasObject = true;
        }

        public void GiveObject(Object specificObject)
        {
            oneItem = specificObject;
            expended = false;
            GiveBackObject();
        }

        float turnSpeed = MathHelper.PiOver2;

        public override void Update(Microsoft.Xna.Framework.GameTime dt, DrawInfo drawInfo)
        {
            base.Update(dt, drawInfo);

            if (expended)
            {
                (Parent as LevelTileNode).RemoveSpawnPointNode(this);
            }

            if(Models != null) 
            {
                Rotation *= Quaternion.CreateFromAxisAngle(Vector3.UnitY, dt.ElapsedSeconds() * turnSpeed);
                if (HasObject)
                {
                    Models[0].IsVisible = true;
                    for (int i = 1; i < 4; i++)
			        {
                        Models[i].IsVisible = false;
			        }
                }
                else
                {
                    int activeModel = -1;
                    if (RespawnTime >= 40)
                        activeModel = 3;
                    else if (RespawnTime >= 10)
                        activeModel = 2;
                    else if (RespawnTime > 0)
                        activeModel = 1;

                    for (int i = 0; i < 4; i++)
                    {
                        if (i == activeModel)
                            Models[i].IsVisible = true;
                        else
                            Models[i].IsVisible = false;
                    }
                }
            }

            if (RespawnTime >= 0)
                RespawnTime -= dt.ElapsedSeconds();
            else
                return;

            if (RespawnTime < 0)
            {
                HasObject = true;
            }
        }
    }
}
