
float4x4 World;
float4x4 View;
float4x4 Projection;
float specularIntensity = 0;
float specularPower = 0.1; 
float4x4 Bones[59];

float4 tintColor = 1;

texture Texture;
sampler diffuseSampler = sampler_state
{
    Texture = (Texture);
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VertexShaderInputColor
{
    float4 Position : POSITION0;
    float3 Normal : NORMAL0;
    float4 Color : COLOR0;
};

struct VertexShaderOutputColor
{
    float4 Position : POSITION0;
	float4 Color : COLOR0;
};




struct VertexShaderInput
{
    float4 Position : POSITION0;
    float3 Normal : NORMAL0;
    float2 TexCoord : TEXCOORD0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
    float3 Normal : TEXCOORD1;
    float2 Depth : TEXCOORD2;
};


struct SkinnedVertexShaderInput
{
    float4 Position : POSITION0;
    float3 Normal : NORMAL0;
    float2 TexCoord : TEXCOORD0;
    float4 BoneIndices : BLENDINDICES;
    float4 BoneWeights : BLENDWEIGHT;
};

VertexShaderOutput SkinnedVS(SkinnedVertexShaderInput input)
{
	// Blend between the weighted bone matrices.
	float4x4 skinTransform = 0;

	skinTransform += Bones[input.BoneIndices.x] * input.BoneWeights.x;
	skinTransform += Bones[input.BoneIndices.y] * input.BoneWeights.y;
	skinTransform += Bones[input.BoneIndices.z] * input.BoneWeights.z;
	skinTransform += Bones[input.BoneIndices.w] * input.BoneWeights.w;

	// Skin the vertex position.
 	float4 weightedposition = mul(float4(input.Position.xyz, 1.0), skinTransform);
	float4 weightedNormal = mul(float4(input.Normal.xyz, 0.0), skinTransform);

	
	VertexShaderOutput output;

    float4 worldPosition = mul(weightedposition, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

    output.TexCoord = input.TexCoord; 
    
    //pass the texture coordinates further
    output.Normal =mul(weightedNormal,World); 
    
    //get normal into world space
    output.Depth.x = output.Position.z;
    output.Depth.y = output.Position.w;
    return output;
	
}

VertexShaderOutputColor ParticleVS(VertexShaderInputColor input)
{
    VertexShaderOutputColor output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
	output.Color = input.Color;
    return output;
}


VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

    output.TexCoord = input.TexCoord; 
    
    //pass the texture coordinates further
    output.Normal =mul(input.Normal,World); 
    
    //get normal into world space
    output.Depth.x = output.Position.z;
    output.Depth.y = output.Position.w;
    return output;
}

struct PixelShaderOutput
{
    float4 Color : COLOR0;
    float4 Normal : COLOR1;
    float4 Depth : COLOR2;
};



PixelShaderOutput PixelShaderFunction(VertexShaderOutput input)
{
    PixelShaderOutput output;
    output.Color = tex2D(diffuseSampler, input.TexCoord) * tintColor; 
    //output Color
    output.Color.a = specularIntensity; 
    //output SpecularIntensity
    output.Normal.rgb = 0.5f * (normalize(input.Normal) + 1.0f); 
    //transform normal domain
    output.Normal.a = specularPower; 
    //output SpecularPower
    output.Depth = input.Depth.x / input.Depth.y; 
    //output Depth
    return output;
}

PixelShaderOutput ParticlePS(VertexShaderOutputColor input)
{
    PixelShaderOutput output;
    output.Color = input.Color; 
    output.Normal = 0; 
    output.Depth = 0; 
    return output;
}

technique Technique1
{
    pass Pass1
    {
        VertexShader = compile vs_2_0 
        VertexShaderFunction();
        PixelShader = compile ps_2_0 
        PixelShaderFunction();
    }
}

technique Skinned
{
    pass Pass1
    {
        VertexShader = compile vs_2_0  SkinnedVS();
        PixelShader = compile ps_2_0  PixelShaderFunction();
    }
}

technique Particles
{
    pass Pass1
    {
        VertexShader = compile vs_2_0  ParticleVS();
        PixelShader = compile ps_2_0  ParticlePS();
    }
}
