﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;

namespace GamePrototype.GUI
{
    public enum ControllerType
    {
        Keyboard1,
        Keyboard2,
        GamePad1,
        GamePad2,
        GamePad3,
        GamePad4
    }

    [Serializable]
    public class PlayerControl
    {
        public const int NUMBER_OF_PLAYERS = 2;
        public const int NUMBER_OF_KEYS_PER_PLAYER = 8;
        public readonly string[] Documentation = new string[] 
        { "Move Forward", "Turn Right", "Move Backward", "Turn Left", "Fire", "Reload", "Next Weapon", "Pick Up Weapon" }; 
        
        /// <summary>
        /// See the property "Documentation" for details. 
        /// </summary>
        public Keys[,] Controls { get; set; }
        
        public void SetStandard()
        {
            Controls = new Keys[NUMBER_OF_PLAYERS, NUMBER_OF_KEYS_PER_PLAYER];

            // Player 1 standard controls
            Controls[0, 0] = Keys.Up;
            Controls[0, 1] = Keys.Right;
            Controls[0, 2] = Keys.Down;
            Controls[0, 3] = Keys.Left;
            Controls[0, 4] = Keys.LeftControl;
            Controls[0, 5] = Keys.R;
            Controls[0, 6] = Keys.LeftShift;
            Controls[0, 7] = Keys.Space;

            // Player 2 standard controls
            Controls[1, 0] = Keys.NumPad8;
            Controls[1, 1] = Keys.NumPad6;
            Controls[1, 2] = Keys.NumPad5;
            Controls[1, 3] = Keys.NumPad4;
            Controls[1, 4] = Keys.W;
            Controls[1, 5] = Keys.A;
            Controls[1, 6] = Keys.S;
            Controls[1, 7] = Keys.D;
        }
        
        public bool IsTheSame(Keys[,] otherControl)
        {
            for (int i = 0; i < Controls.GetLength(0); i++)
            {
                for (int j = 0; j < Controls.GetLength(1); j++)
                {
                    if (Controls[i,j] != otherControl[i,j] )
                    {
                        return false;
                    }
                }    
            }

            return true;
        }
    }
}
