﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System.IO;
using System.Reflection;

namespace GamePrototype.GUI.Screens
{
    class ConfigureGraphicsOptionsScreen : GameScreen
    {
        private const float headerScale = 2.0f;
        private const float spaceMenuPadding = 80f;
        private const float spaceBetweenMenuItems = 15f;
        private const float spaceFromControlsToButtons = 20f;
        private const float spaceBetweenButtons = 15f;
        private const float menuScale = 1.4f;
        private const float buttonScale = 1.7f;
        private Color backgroundColor = Color.Black;
        private Color textsColor = Color.Gray;
        private Color headerColor = Color.CornflowerBlue;
        private Color menuItemsColor = Color.CornflowerBlue;
        private Color menuItemsSelectedColor = Color.White;

        private int currentMenuItem = 0;

        private const string header = "Configure Graphics";
        
        private const string fullscreenLabel = "Screen mode:";
        private string[] fullscreenOptions = new string[] { "Full Screen", "Window" };
        private bool fullscreen;

        private const string screenresLabel = "Resolution:";
        private string[] screenresOptions = new string[] { "640x480", "800x600", "1024x768", "1280x720", "1280x1024", "1400x1050", "1650x1080", "1920x1080" };
        private int screenres;

        private const int buttonCountOffset = 2;
        private string[] buttonTexts = new string[] { "Apply settings", "Back to Main Menu" };
        private const string saveText = "Apply?";
        private string[] saveOptions = new string[] { "Yes", "No" }; 

        private Texture2D texture;

        public ConfigureGraphicsOptionsScreen(string id, ScreenController controller, Game1 game)
            : base(id, controller, game)
        {
        }

        public override void LoadContent(GraphicsDeviceManager graphics)
        {
            texture = new Texture2D(graphics.GraphicsDevice, 1, 1, 1, TextureUsage.None, SurfaceFormat.Vector4);
            texture.SetData(new Vector4[] { Vector4.One });
        }

        public override void Initialize(GraphicsDevice graphics)
        { }

        public void AboutToShow()
        {
            this.fullscreen = (this.Game as Game1).FullScreen;
            this.screenres = (this.Game as Game1).ScreenResolution;
        }

        public override void Update(GameTime gameTime)
        {
            DialogueScreen dialogue = (DialogueScreen)this.controller.GetScreen("DialogueScreen");

            if (dialogue.DialogueResult != "")
            { 
                string[] temp1 = dialogue.DialogueResult.Split('|');

                if (temp1[0] == "ConfigureGraphicsOptionsScreen")
                {
                    string[] temp2 = temp1[1].Split(':');

                    if (temp1[1] == screenresLabel)
                    {
                        this.screenres = dialogue.CurrentIndex;
                    }
                    else // Save Changes on exit?
                    {
                        if (temp1[1] == saveText)
                        {
                            if (temp1[2] == saveOptions[0]) // Yes
                            {
                                (this.Game as Game1).FullScreen = this.fullscreen;
                                (this.Game as Game1).ScreenResolution = this.screenres;
                                (this.Game as Game1).ReinitializeGraphics();
                                this.controller.SetHiddenScreen("ConfigureGraphicsOptionsScreen");
                                this.controller.SetActiveScreen("MainMenuScreen");
                            }
                            if (temp1[2] == saveOptions[1]) // No
                            {
                                this.controller.SetHiddenScreen("ConfigureGraphicsOptionsScreen");
                                this.controller.SetActiveScreen("MainMenuScreen");
                            }
                        }
                    }
                }

                dialogue.DialogueResult = "";
            }


            if (InputState.KeyPressed(Keys.Enter) || InputState.ButtonPressedOnAnyGamePad(Buttons.A))
            {
                if (currentMenuItem == 0) // Full Screen
                {
                    this.fullscreen = !this.fullscreen;
                }
                else if (currentMenuItem == 1) // Resolution
                {
                    this.controller.SetInactiveScreen("ConfigureGraphicsOptionsScreen");
                    dialogue.SetDialogueData(screenresLabel, this.screenresOptions, "ConfigureGraphicsOptionsScreen", this.screenres);
                    this.controller.SetActiveScreen("DialogueScreen");
                }
                else if (currentMenuItem == 2) // Save Changes
                {
                    (this.Game as Game1).FullScreen = this.fullscreen;
                    (this.Game as Game1).ScreenResolution = this.screenres;
                    (this.Game as Game1).ReinitializeGraphics();
                }
                else if (currentMenuItem == 3) // Back to main menu
                {
                    // Nothing is changed
                    if ((this.Game as Game1).FullScreen == this.fullscreen && (this.Game as Game1).ScreenResolution == this.screenres)
                    {
                        this.controller.SetHiddenScreen("ConfigureGraphicsOptionsScreen");
                        this.controller.SetActiveScreen("MainMenuScreen");
                    }
                    else
                    {
                        this.controller.SetInactiveScreen("ConfigureGraphicsOptionsScreen");
                        dialogue.SetDialogueData(saveText, saveOptions, "ConfigureGraphicsOptionsScreen", 0);
                        this.controller.SetActiveScreen("DialogueScreen");
                    }
                }
            }

            if (InputState.KeyPressed(Keys.Up) || InputState.ButtonPressedOnAnyGamePad(Buttons.DPadUp))
            {
                currentMenuItem--;
                if (currentMenuItem == -1)
                    currentMenuItem = buttonCountOffset + buttonTexts.Count() - 1;
            }

            if (InputState.KeyPressed(Keys.Down) || InputState.ButtonPressedOnAnyGamePad(Buttons.DPadDown))
            {
                currentMenuItem++;
                if (currentMenuItem == buttonCountOffset + buttonTexts.Count())
                    currentMenuItem = 0;
            }

            if (InputState.KeyPressed(Keys.Escape))
            {
                // Nothing is changed
                if ((this.Game as Game1).FullScreen == this.fullscreen && (this.Game as Game1).ScreenResolution == this.screenres)
                {
                    this.controller.SetHiddenScreen("ConfigureGraphicsOptionsScreen");
                    this.controller.SetActiveScreen("MainMenuScreen");
                }
                else
                {
                    this.controller.SetInactiveScreen("ConfigureGraphicsOptionsScreen");
                    dialogue.SetDialogueData(saveText, saveOptions, "ConfigureGraphicsOptionsScreen", 0);
                    this.controller.SetActiveScreen("DialogueScreen");
                }
            }

            AudioManager.PlaySoundEffect(Game1.MENU_CLICK, Game1.MENU_CLICK_VOLUME);
        }

        public override void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            float yPos = 0;

            graphics.Clear(backgroundColor);

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);

            // Draw teh menues
            DrawHeader(spriteBatch, spriteFont, ref yPos);

            DrawMenuFullscreen(spriteBatch, spriteFont, ref yPos);
            DrawMenuResolution(spriteBatch, spriteFont, ref yPos);

            DrawButtons(spriteBatch, spriteFont, ref yPos);

            spriteBatch.End();
        }

        private void DrawHeader(SpriteBatch spriteBatch, SpriteFont spriteFont, ref float yPos)
        {
            float x = GUIhelper.GetHorisontalCenter(header, spriteFont, Game, headerScale);
            Vector2 headLinePosition = new Vector2(x, yPos);

            spriteBatch.DrawString(spriteFont, header, headLinePosition, headerColor, 0, Vector2.Zero, headerScale, SpriteEffects.None, 0);

            yPos += spriteFont.MeasureString(header).Y * headerScale + 2;
        }

        private void DrawMenuFullscreen(SpriteBatch spriteBatch, SpriteFont spriteFont, ref float yPos)
        {
            Vector2 itemPosition = new Vector2(spaceMenuPadding, yPos);
            Vector2 valuePosition = new Vector2(Game.Window.ClientBounds.Width / 2, yPos);
            Color itemColor = currentMenuItem == 0 ? menuItemsSelectedColor : menuItemsColor;
            string currentChoice = this.fullscreen ? fullscreenOptions[0] : fullscreenOptions[1];

            spriteBatch.DrawString(spriteFont, fullscreenLabel, itemPosition, menuItemsColor, 0, Vector2.Zero, menuScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, currentChoice, valuePosition, itemColor, 0, Vector2.Zero, menuScale, SpriteEffects.None, 0);

            yPos += spriteFont.MeasureString(fullscreenLabel).Y * buttonScale + 2;
        }

        private void DrawMenuResolution(SpriteBatch spriteBatch, SpriteFont spriteFont, ref float yPos)
        {
            Vector2 itemPosition = new Vector2(spaceMenuPadding, yPos);
            Vector2 valuePosition = new Vector2(Game.Window.ClientBounds.Width / 2, yPos);
            Color itemColor = currentMenuItem == 1 ? menuItemsSelectedColor : menuItemsColor;
            string currentChoice = screenresOptions[screenres];

            spriteBatch.DrawString(spriteFont, screenresLabel, itemPosition, menuItemsColor, 0, Vector2.Zero, menuScale, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, currentChoice, valuePosition, itemColor, 0, Vector2.Zero, menuScale, SpriteEffects.None, 0);

            yPos += spriteFont.MeasureString(screenresLabel).Y * buttonScale + 2;
        }

        private void DrawButtons(SpriteBatch spriteBatch, SpriteFont spriteFont, ref float yPos)
        {
            Color itemColor;
            float xPos = GUIhelper.GetHorisontalCenter(buttonTexts[0], spriteFont, Game, buttonScale);

            yPos += spaceFromControlsToButtons;

            for (int i = 0; i < buttonTexts.Length; i++)
            {
                itemColor = currentMenuItem == (i + buttonCountOffset) ? menuItemsSelectedColor : menuItemsColor;

                spriteBatch.DrawString(spriteFont, buttonTexts[i], new Vector2(xPos, yPos), itemColor, 0, Vector2.Zero, buttonScale, SpriteEffects.None, 0);

                yPos += (spriteFont.MeasureString(buttonTexts[i]).Y * buttonScale) + spaceBetweenButtons;
            }
        }

        #region Settings load & save

        public static string GraphicsSettingsFilename = "Graphicssettings.dat";
        public void LoadGraphicsSettings()
        {
        }

        public void SaveGraphicsSettings()
        {
        }

        private string GetDataFile()
        {
            return Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) + Path.DirectorySeparatorChar + Path.DirectorySeparatorChar + GraphicsSettingsFilename;
        }

        #endregion
    }
}
