﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework;
using GameEngine.Xml;
using Microsoft.Xna.Framework.Graphics;
using GameEngine.Helpers;
using GameEngine.LevelGeneration;
using GameEngine.Cameras;
using System.Xml;
using DeferredShadingTutorial;
using GamePrototype.GameLogic;

namespace GamePrototype.GUI.Screens
{
    public class DebuggingScreen : GameScreen
    {
        private QuadRenderer quadRenderer;
        private GraphicsDevice GraphicsDevice;
        private SpriteBatch spriteBatch;

        //this Render Target will hold color and Specular Intensity
        private RenderTarget2D colorRT;

        //this Render Target will hold normals and Specular Power
        private RenderTarget2D normalRT;

        //finally, this one will hold the depth
        private RenderTarget2D depthRT;

        private RenderTarget2D lightRT;

        private Effect finalCombineEffect;
        private Effect gbufferEffect;
        private Effect directionalLightEffect;
        private Effect clearBufferEffect;
        private Effect pointLightEffect;


        Model model, sphere;
        Texture2D white;
        DrawInfo di = new DrawInfo();
        List<PolygonWallNormals> walls = new List<PolygonWallNormals>(); 

        public DebuggingScreen(string id, ScreenController controller, Game game, params string[] args)
            : base(id, controller, game)
        {
            //GameWrapper.LoadTileset(game.Content);
        }
        
        public override void LoadContent(GraphicsDeviceManager graphics)
        {

            foreach (var item in TilesetManager.LoadRandomTile(TileType.Hallway))
            {
                PolygonWallNode pwn = item as PolygonWallNode;
                if (pwn == null)
                    continue;

                pwn.Dirty();
                pwn.ResolveDirty();
                walls.Add(PolygonWallNormals.Generate(graphics.GraphicsDevice, pwn.Points, pwn.Height, Vector2.Zero, Vector2.Zero, Vector4.One));

            }

            this.GraphicsDevice = graphics.GraphicsDevice;
            int backBufferWidth = GraphicsDevice.PresentationParameters.BackBufferWidth;

            int backBufferHeight = GraphicsDevice.PresentationParameters.BackBufferHeight;

            colorRT = new RenderTarget2D(GraphicsDevice,
                                         backBufferWidth,
                                         backBufferHeight,
                                         1,
                                         SurfaceFormat.Color);

            normalRT = new RenderTarget2D(GraphicsDevice,
                                          backBufferWidth,
                                          backBufferHeight,
                                          1,
                                          SurfaceFormat.Color);

            depthRT = new RenderTarget2D(GraphicsDevice,
                                         backBufferWidth,
                                         backBufferHeight,
                                         1,
                                         SurfaceFormat.Single);
            clearBufferEffect = content.Load<Effect>("EngineFx/ClearGBuffer");
            quadRenderer = new QuadRenderer();
            quadRenderer.LoadContent(GraphicsDevice);
            
            pointLightEffect = content.Load<Effect>("EngineFx/PointLight");

            gbufferEffect = content.Load<Effect>("EngineFx/RenderGBuffer");
            model = content.Load<Model>("models\\player\\player");
            spriteBatch = new SpriteBatch(GraphicsDevice);
            //white = new Texture2D(GraphicsDevice, 1, 1);
            //white.SetData(new Color[] { Color.White });

            white = content.Load<Texture2D>("models/elevator/rust_texture_jpg");

            di.Camera = new OrthoOrbitCamera(MathHelper.PiOver2, new Vector3(0,100,100), Vector3.Zero, Vector3.Up, GraphicsDevice.Viewport, 1, 10000, CameraMode.Perspective);
            di.GraphicsDevice = GraphicsDevice;
            directionalLightEffect = content.Load<Effect>("EngineFx/DirectionalLight");
            
            lightRT = new RenderTarget2D(GraphicsDevice, backBufferWidth, backBufferHeight, 1, SurfaceFormat.Color);
            finalCombineEffect = content.Load<Effect>("EngineFx/CombineFinal");
            sphere = content.Load<Model>("EngineFx\\sphere");

        }

        public override void Update(GameTime gameTime)
        {
            
            if (InputState.KeyboardState.IsKeyDown(Microsoft.Xna.Framework.Input.Keys.Left))
                di.Camera.Target.X -= 500 * gameTime.ElapsedSeconds();
            if (InputState.KeyboardState.IsKeyDown(Microsoft.Xna.Framework.Input.Keys.Right))
                di.Camera.Target.X += 500 * gameTime.ElapsedSeconds();
            if (InputState.KeyboardState.IsKeyDown(Microsoft.Xna.Framework.Input.Keys.Up))
                di.Camera.Target.Z -= 500 * gameTime.ElapsedSeconds();
            if (InputState.KeyboardState.IsKeyDown(Microsoft.Xna.Framework.Input.Keys.Down))
                di.Camera.Target.Z += 500 * gameTime.ElapsedSeconds();

            if (InputState.KeyboardState.IsKeyDown(Microsoft.Xna.Framework.Input.Keys.A))
                di.Camera.Position.X -= 500 * gameTime.ElapsedSeconds();
            if (InputState.KeyboardState.IsKeyDown(Microsoft.Xna.Framework.Input.Keys.D))
                di.Camera.Position.X += 500 * gameTime.ElapsedSeconds();
            if (InputState.KeyboardState.IsKeyDown(Microsoft.Xna.Framework.Input.Keys.W))
                di.Camera.Position.Z -= 500 * gameTime.ElapsedSeconds();
            if (InputState.KeyboardState.IsKeyDown(Microsoft.Xna.Framework.Input.Keys.S))
                di.Camera.Position.Z += 500 * gameTime.ElapsedSeconds();

            di.Camera.UpdateCamera(gameTime);


        }
        Vector2 halfPixel;

        public override void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            halfPixel.X = 0.5f / (float)GraphicsDevice.PresentationParameters.BackBufferWidth;
            halfPixel.Y = 0.5f / (float)GraphicsDevice.PresentationParameters.BackBufferHeight;
            
            SetGBuffer();
            GraphicsDevice.Clear(Color.Gray);
            ClearGBuffer();
            //scene.DrawScene(camera, gameTime);

            GraphicsDevice.RenderState.DepthBufferEnable = true;
            GraphicsDevice.RenderState.CullMode = CullMode.CullCounterClockwiseFace;
            GraphicsDevice.RenderState.AlphaBlendEnable = false;

            gbufferEffect.Parameters["View"].SetValue(di.Camera.View);
            gbufferEffect.Parameters["Projection"].SetValue(di.Camera.Projection);
            gbufferEffect.Parameters["Texture"].SetValue(white);
            gbufferEffect.Parameters["World"].SetValue(Matrix.Identity);

            gbufferEffect.Begin();
            gbufferEffect.CurrentTechnique.Passes[0].Begin();
            
            foreach (var wall in walls)
            {
                wall.SetSource(GraphicsDevice);
                wall.Draw(GraphicsDevice);
            }
            

            foreach (ModelMesh mesh in model.Meshes)
            {

                foreach (ModelMeshPart meshPart in mesh.MeshParts)
                {
                    GraphicsDevice.VertexDeclaration = meshPart.VertexDeclaration;
                    GraphicsDevice.Vertices[0].SetSource(mesh.VertexBuffer, meshPart.StreamOffset, meshPart.VertexStride);
                    GraphicsDevice.Indices = mesh.IndexBuffer;
                    GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.TriangleList, meshPart.BaseVertex, 0, meshPart.NumVertices, meshPart.StartIndex, meshPart.PrimitiveCount);
                }
            }

            gbufferEffect.CurrentTechnique.Passes[0].End();
            gbufferEffect.End();

            ResolveGBuffer();

            GraphicsDevice.Clear(Color.Black);

            DrawLights(gameTime, di);

            //spriteBatch.Begin();
            //spriteBatch.Draw(colorRT.GetTexture(), new Rectangle(0, 0, GraphicsDevice.Viewport.Width, GraphicsDevice.Viewport.Height), Color.White);
            //spriteBatch.End();
            /*
            int halfWidth = GraphicsDevice.Viewport.Width / 2;
            int halfHeight = GraphicsDevice.Viewport.Height / 2;
            spriteBatch.Begin();
            spriteBatch.Draw(colorRT.GetTexture(),
            new Rectangle(0, 0, halfWidth, halfHeight), Color.White);
            spriteBatch.Draw(normalRT.GetTexture(),
            new Rectangle(0, halfHeight, halfWidth, halfHeight), Color.White);
            spriteBatch.Draw(depthRT.GetTexture(),
            new Rectangle(halfWidth, 0, halfWidth, halfHeight), Color.White);
            spriteBatch.End();*/
            
        }

        private void DrawPointLight(DrawInfo di, Vector3 lightPosition, Color color, float lightRadius,float lightIntensity)
        {
            //set the G-Buffer parameters

            pointLightEffect.Parameters["colorMap"].SetValue(colorRT.GetTexture());

            pointLightEffect.Parameters["normalMap"].SetValue(normalRT.GetTexture());

            pointLightEffect.Parameters["depthMap"].SetValue(depthRT.GetTexture());

            //compute the light world matrix
            //scale according to light radius, and translate it to light position
            Matrix sphereWorldMatrix = Matrix.CreateScale(lightRadius) *
            Matrix.CreateTranslation(lightPosition);

            pointLightEffect.Parameters["World"].SetValue(sphereWorldMatrix);
            pointLightEffect.Parameters["View"].SetValue(di.Camera.View);

            pointLightEffect.Parameters["Projection"].SetValue(di.Camera.Projection);
            //light position

            pointLightEffect.Parameters["lightPosition"].SetValue(lightPosition);

            //set the color, radius and Intensity
            pointLightEffect.Parameters["Color"].SetValue(color.ToVector3());

            pointLightEffect.Parameters["lightRadius"].SetValue(lightRadius);

            pointLightEffect.Parameters["lightIntensity"].SetValue(lightIntensity);

            //parameters for specular computations

            pointLightEffect.Parameters["cameraPosition"].SetValue(di.Camera.Position);

            pointLightEffect.Parameters["InvertViewProjection"].SetValue(Matrix.Invert(di.Camera.View
                                                                * di.Camera.Projection));
            //size of a halfpixel, for texture coordinates alignment
            pointLightEffect.Parameters["halfPixel"].SetValue(halfPixel);

            //calculate the distance between the camera and light center
            float cameraToCenter = Vector3.Distance(di.Camera.Position, lightPosition);
            //if we are inside the light volume, draw the sphere's inside face
            if (cameraToCenter < lightRadius)
                GraphicsDevice.RenderState.CullMode = CullMode.CullClockwiseFace;
            else
                GraphicsDevice.RenderState.CullMode = CullMode.CullCounterClockwiseFace;
            pointLightEffect.Begin();
            pointLightEffect.Techniques[0].Passes[0].Begin();

            foreach (ModelMesh mesh in sphere.Meshes)
            {
                foreach (ModelMeshPart meshPart in
                mesh.MeshParts)
                {

                    GraphicsDevice.VertexDeclaration = meshPart.VertexDeclaration;

                    GraphicsDevice.Vertices[0].SetSource(mesh.VertexBuffer,
                    meshPart.StreamOffset, meshPart.VertexStride);

                    GraphicsDevice.Indices = mesh.IndexBuffer;

                    GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.TriangleList,
                    meshPart.BaseVertex, 0, meshPart.NumVertices, meshPart.StartIndex,
                    meshPart.PrimitiveCount);
                }
            }

            pointLightEffect.Techniques[0].Passes[0].End();
            pointLightEffect.End();

            GraphicsDevice.RenderState.CullMode =
            CullMode.CullCounterClockwiseFace;
        }


        private void DrawLights(GameTime gameTime, DrawInfo di)
        {
            GraphicsDevice.RenderState.DepthBufferEnable = false;

            GraphicsDevice.SetRenderTarget(0, lightRT);
            //clear all components to 0
            GraphicsDevice.Clear(Color.TransparentBlack);
            GraphicsDevice.RenderState.AlphaBlendEnable = true;
            
            //use additive blending, and make sure the blending factors are as we need them
            GraphicsDevice.RenderState.AlphaBlendOperation = BlendFunction.Add;
            GraphicsDevice.RenderState.SourceBlend = Blend.One;
            GraphicsDevice.RenderState.DestinationBlend = Blend.One;
            
            //use the same operation on the alpha channel
            GraphicsDevice.RenderState.SeparateAlphaBlendEnabled = false;

            //draw some lights
            //DrawDirectionalLight(new Vector3(0, -1, 1), Color.White,di );
            DrawPointLight(di, new Vector3(50 * (float)Math.Sin(gameTime.TotalGameTime.TotalSeconds), 10, 50 * (float)Math.Cos(gameTime.TotalGameTime.TotalSeconds)), Color.White, 100, 4);

            GraphicsDevice.RenderState.AlphaBlendEnable = false;
            GraphicsDevice.SetRenderTarget(0, null);

            //set the effect parameters
            finalCombineEffect.Parameters["colorMap"].SetValue(colorRT.GetTexture());
            finalCombineEffect.Parameters["lightMap"].SetValue(lightRT.GetTexture());
            finalCombineEffect.Parameters["halfPixel"].SetValue(halfPixel);

            finalCombineEffect.Begin();
            finalCombineEffect.Techniques[0].Passes[0].Begin();

            //render a full-screen quad
            quadRenderer.Render(GraphicsDevice, Vector2.One * -1, Vector2.One);

            finalCombineEffect.Techniques[0].Passes[0].End();
            finalCombineEffect.End();


        }


        private void DrawDirectionalLight(Vector3 lightDirection, Color color, DrawInfo di)
        {

            //set all parameters
            directionalLightEffect.Parameters["colorMap"].SetValue(colorRT.GetTexture());
            directionalLightEffect.Parameters["normalMap"].SetValue(normalRT.GetTexture());
            directionalLightEffect.Parameters["depthMap"].SetValue(depthRT.GetTexture());
            directionalLightEffect.Parameters["lightDirection"].SetValue(lightDirection);
            directionalLightEffect.Parameters["Color"].SetValue(color.ToVector3());
            directionalLightEffect.Parameters["cameraPosition"].SetValue(di.Camera.Position);
            directionalLightEffect.Parameters["InvertViewProjection"].SetValue(Matrix.Invert(di.Camera.View * di.Camera.Projection));
            directionalLightEffect.Parameters["halfPixel"].SetValue(halfPixel);

            directionalLightEffect.Begin();
            directionalLightEffect.Techniques[0].Passes[0].Begin();
            //draw a full-screen quad
            quadRenderer.Render(di.GraphicsDevice, Vector2.One * -1, Vector2.One);
            directionalLightEffect.Techniques[0].Passes[0].End();
            directionalLightEffect.End();
        }


        public override void Initialize(GraphicsDevice graphics)
        {

        }

        private void SetGBuffer()
        {
            GraphicsDevice.SetRenderTarget(0, colorRT);
            GraphicsDevice.SetRenderTarget(1, normalRT);
            GraphicsDevice.SetRenderTarget(2, depthRT);
        }
        private void ResolveGBuffer()
        {
            // set all rendertargets to null. In XNA 2.0, switching a 
            // rendertarget causes the resolving of the previous rendertarget.

            // In XNA 1.1, we needed to call GraphicsDevice.ResolveRenderTarget(i);

            GraphicsDevice.SetRenderTarget(0, null);
            GraphicsDevice.SetRenderTarget(1, null);
            GraphicsDevice.SetRenderTarget(2, null);
        }

        private void ClearGBuffer()
        {
            clearBufferEffect.Begin();
            clearBufferEffect.Techniques[0].Passes[0].Begin();
            quadRenderer.Render(GraphicsDevice, Vector2.One * -1, Vector2.One);
            clearBufferEffect.Techniques[0].Passes[0].End();
            clearBufferEffect.End();
        }

    }
}
