using System;
using System.Collections.Generic;
using System.Text;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using GameEngine.Level;
using GamePrototype.GameLogic;
using GameEngine.LevelGeneration;

namespace GamePrototype.GUI.Screens
{
    public class InGameScreen : GameScreen
    {
        private Color backgroundColor = Color.White;
        private const string menuDialogueText = "Game Options";
        private string[] menuItems = new string[] { "Resume Game", "Exit to Main Menu" };
                
        public int Difficulty { get; set; } // 1 = Easy, 2 = Normal, 3 = Hard
        public int PlayerCount { get; set; } // 1 = One player, 2 = Two players
        public int CurrentLevel { get; set; }

        public ControllerType[] JoinedPlayers { get; set; }

        private GameWrapper gameWrapper;
        public PlayerControl Controls { get; set; }
        
        public InGameScreen(string id, ScreenController controller, Game game) 
            : base(id, controller, game)
	    {
	    }

        public void PrepareLevel(LevelGenerator levelGenerator)
        {
            gameWrapper = new GameWrapper(this.content, JoinedPlayers, Difficulty, Controls.Controls, 
                levelGenerator, CurrentLevel, this.controller, this.Game);
            gameWrapper.Initialize(this.Game.GraphicsDevice);
        }

        public override void Initialize(GraphicsDevice graphics)
        {

        }

        public override void LoadContent(GraphicsDeviceManager graphics)
        {

        }

        public override void Update(GameTime gameTime)
        {
            DialogueScreen dialogue = (DialogueScreen)this.controller.GetScreen("DialogueScreen");

            if (dialogue.DialogueResult != "")
            {
                if (dialogue.DialogueResult == "InGameScreen" + "|" + menuDialogueText + "|" + menuItems[1]) // Exit to main menu
                {
                    this.controller.SetHiddenScreen("InGameScreen");
                    this.controller.SetActiveScreen("MainMenuScreen");
                    gameWrapper.Dispose();
                    AudioManager.PlayMusic(Game1.MUSIC_MENU);
                }

                dialogue.DialogueResult = "";
            }
            
            if (InputState.KeyPressed(Keys.Escape) || InputState.ButtonPressedOnAnyGamePad(Buttons.Back))
            {
                this.controller.SetInactiveScreen("InGameScreen");
                dialogue.SetDialogueData(menuDialogueText, menuItems, "InGameScreen", 0);
                this.controller.SetActiveScreen("DialogueScreen");
            }

            gameWrapper.Update(gameTime);
        }

        public override void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            graphics.Clear(backgroundColor);

            gameWrapper.Draw(gameTime, graphics, spriteBatch, spriteFont);
        }        
    }
}
