﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework;
using GameEngine.LevelGeneration;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace GamePrototype.GUI.Screens
{
    public class SetupGameScreen : GameScreen
    {
        private int STARTING_LEVEL = 0;

        private bool showDiffPopup = true;

        private List<ControllerType> joinedPlayers = new List<ControllerType>();
        
        private const string headLine = "Game Setup";
        //private string[] helpTexts = new string[] { "Use Arrow Keys to Navigate.", "Use Enter Key to Change a Value." };
        private const float helpTextScale = 1.2f;
        private const float headLineScale = 2.0f;
        private Color headLineColor = Color.CornflowerBlue;
        private Color helpTextColor = Color.Gray;
        private const float spaceBetweenHeaders = 5f;
        
        private const float spaceMenuPadding = 80f; 
        private const float spaceBetweenMenuItems = 15f;
        private const float spaceBetweenPlayers = 5f;
        private const float menuScale = 1.4f;
        private const float buttonScale = 1.7f;
        private Color backgroundColor = Color.Black;
        private Color textsColor = Color.Gray;
        private Color menuItemsColor = Color.CornflowerBlue;
        private Color menuItemsSelectedColor = Color.White;

        private const string playerDialogue = "Select Players:";
        private const string difficultyDialogue = "Select Difficulty:";
        private const string mapsizeDialogue = "Select Map Size:";

        private string[] playerOptions = new string[] { "One Player", "Two Players" };
        private int playerSelection = 0;

        private string[] difficultyOptions = new string[] { "Normal", "Hard", "Nightmare" };
        private int difficultySelection = 0;
                
        private string[] menuTexts = new string[] { "Game Difficulty:" };

        private string[] menuItems;
        private int currentMenuIndex = 2;

        private int numberOfPlayers = 0;

        private PlayerControl controls;
        
        private Texture2D texture;

        public SetupGameScreen(string id, ScreenController controller, Game game)
            : base(id, controller, game)
        {
            SetCurrentItems();

            controls = (this.controller.GetScreen("ConfigureControlsScreen") as ConfigureControlsScreen).GetControls();
        }

        public override void LoadContent(GraphicsDeviceManager graphics)
        {
            texture = new Texture2D(graphics.GraphicsDevice, 1, 1, 1, TextureUsage.None, SurfaceFormat.Vector4);
            texture.SetData(new Vector4[] { Vector4.One });
        }

        public override void Update(GameTime gameTime)
        {
            int menuIndex = currentMenuIndex;

            DialogueScreen dialogue = (DialogueScreen)this.controller.GetScreen("DialogueScreen");

            if (dialogue.DialogueResult != "")
            {
                string[] temp = dialogue.DialogueResult.Split('|');

                if (temp[0] == "SetupGameScreen")
                {
                    if (temp[1] == playerDialogue)
                    {
                        playerSelection = Array.IndexOf(playerOptions, temp[2]);    
                    }
                    if (temp[1] == difficultyDialogue)
                    {
                        difficultySelection = Array.IndexOf(difficultyOptions, temp[2]);
                        showDiffPopup = false;
                    }                    
                }

                dialogue.DialogueResult = "";
            }

            if (showDiffPopup)
            {
                this.controller.SetInactiveScreen("SetupGameScreen");
                dialogue.SetDialogueData(difficultyDialogue, difficultyOptions, "SetupGameScreen", difficultySelection);
                this.controller.SetActiveScreen("DialogueScreen");
            }

            SetCurrentItems();

            //if (InputState.KeyPressed(Keys.Up))
            //{
            //    currentMenuIndex = (currentMenuIndex == 0) ? menuItems.Length - 1 : currentMenuIndex - 1;
            //}

            //if (InputState.KeyPressed(Keys.Down))
            //{
            //    currentMenuIndex = (currentMenuIndex = currentMenuIndex + 1) % menuItems.Length;
            //}

            if (InputState.KeyPressed(Keys.Escape) || InputState.ButtonPressedOnAnyGamePad(Buttons.B))
            {
                this.controller.SetHiddenScreen("SetupGameScreen");
                this.controller.SetActiveScreen("MainMenuScreen");
                ResetScreen();
            }
            
            //if (InputState.KeyPressed(Keys.Enter))
            //{
            //    switch (currentMenuIndex)
            //    {
            //        case 0: // Players
            //            this.controller.SetInactiveScreen("SetupGameScreen");
            //            dialogue.SetDialogueData(playerDialogue, playerOptions, "SetupGameScreen", playerSelection);
            //            this.controller.SetActiveScreen("DialogueScreen");
            //            break;
            //        case 1: // Difficulty
            //            this.controller.SetInactiveScreen("SetupGameScreen");
            //            dialogue.SetDialogueData(difficultyDialogue, difficultyOptions, "SetupGameScreen", difficultySelection);
            //            this.controller.SetActiveScreen("DialogueScreen");
            //            break;
            //        case 2: // Start game
            //            StatusScreen statusScreen = (StatusScreen)this.controller.GetScreen("StatusScreen");
            //            statusScreen.PlayerCount = playerSelection + 1;
            //            statusScreen.Difficulty = difficultySelection + 1;
            //            statusScreen.CurrentLevel = STARTING_LEVEL;
            //            statusScreen.Prepare();
            //            this.controller.SetHiddenScreen("SetupGameScreen");
            //            this.controller.SetActiveScreen("StatusScreen");
            //            showDiffPopup = true;
            //            break;
            //        case 3: // Back to menu
            //            this.controller.SetHiddenScreen("SetupGameScreen");
            //            this.controller.SetActiveScreen("MainMenuScreen");
            //            break;
            //        default:
            //            break;
            //    }
            //}

            if (InputState.KeyPressed(Keys.Enter) && (joinedPlayers.Contains(ControllerType.Keyboard1) || joinedPlayers.Contains(ControllerType.Keyboard2)))
                StartGame();

            if (InputState.KeyPressed(controls.Controls[0, 4]) || InputState.KeyPressed(Keys.Enter)) 
            {
                if (!joinedPlayers.Contains(ControllerType.Keyboard1))
                    joinedPlayers.Add(ControllerType.Keyboard1);
            }

            if (InputState.KeyPressed(controls.Controls[1, 4]))
            {
                if (!joinedPlayers.Contains(ControllerType.Keyboard2))
                    joinedPlayers.Add(ControllerType.Keyboard2);
            }

            if (InputState.ButtonPressed(PlayerIndex.One, Buttons.A))
            {
                if (joinedPlayers.Contains(ControllerType.GamePad1))
                    StartGame();
                else
                    joinedPlayers.Add(ControllerType.GamePad1);
            }
            if (InputState.ButtonPressed(PlayerIndex.Two, Buttons.A))
            {
                if (joinedPlayers.Contains(ControllerType.GamePad2))
                    StartGame();
                else
                    joinedPlayers.Add(ControllerType.GamePad2);
            }
            if (InputState.ButtonPressed(PlayerIndex.Three, Buttons.A))
            {
                if (joinedPlayers.Contains(ControllerType.GamePad3))
                    StartGame();
                else
                    joinedPlayers.Add(ControllerType.GamePad3);
            }
            if (InputState.ButtonPressed(PlayerIndex.Four, Buttons.A))
            {
                if (joinedPlayers.Contains(ControllerType.GamePad4))
                    StartGame();
                else
                    joinedPlayers.Add(ControllerType.GamePad4);
            }


            if (currentMenuIndex != menuIndex )
            {
                AudioManager.PlaySoundEffect(Game1.MENU_CLICK, Game1.MENU_CLICK_VOLUME);
            }
        }

        private void StartGame()
        {
            StatusScreen statusScreen = (StatusScreen)this.controller.GetScreen("StatusScreen");
            statusScreen.PlayerCount = joinedPlayers.Count;
            statusScreen.JoinedPlayers = joinedPlayers.ToArray();
            statusScreen.Difficulty = difficultySelection + 1;
            statusScreen.CurrentLevel = STARTING_LEVEL;
            statusScreen.Prepare();
            this.controller.SetHiddenScreen("SetupGameScreen");
            this.controller.SetActiveScreen("StatusScreen");
            ResetScreen();
        }

        private void ResetScreen()
        {
            joinedPlayers.Clear();
            showDiffPopup = true;
        }

        public override void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            graphics.Clear(backgroundColor);

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);

            DrawHeader(spriteBatch, spriteFont);

            DrawMenu(spriteBatch, spriteFont);

            spriteBatch.End();
        }

        public override void Initialize(GraphicsDevice graphics)
        {
            
        }

        private void SetCurrentItems()
        {
            menuItems = new string[] { difficultyOptions[difficultySelection] };
        }

        private void DrawHeader(SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            float x = GUIhelper.GetHorisontalCenter(headLine, spriteFont, Game, headLineScale);
            float y = GUIhelper.GetVerticalCenter(headLine, spriteFont, Game, headLineScale) * 0.25f;
            Vector2 headLinePosition = new Vector2(x, y);

            spriteBatch.DrawString(spriteFont, headLine, headLinePosition, headLineColor, 0, Vector2.Zero,
                headLineScale, SpriteEffects.None, 0);

            //float headlineHeight = spriteFont.MeasureString(headLine).Y * headLineScale;
            //float helpTextHeight = spriteFont.MeasureString(helpTexts[0]).Y * helpTextScale;

            //y += headlineHeight + 2 * spaceBetweenHeaders;

            //for (int i = 0; i < helpTexts.Length; i++)
            //{
            //    spriteBatch.DrawString(spriteFont, helpTexts[i],
            //    new Vector2(GUIhelper.GetHorisontalCenter(helpTexts[i], spriteFont, game, helpTextScale), y),
            //        helpTextColor, 0, Vector2.Zero, helpTextScale, SpriteEffects.None, 0);

            //    y += helpTextHeight + spaceBetweenHeaders;
            //}
        }

        private void DrawMenu(SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            float menuTextsWidth = 0;

            for (int i = 0; i < menuTexts.Length; i++)
            {
                float j = spriteFont.MeasureString(menuTexts[i]).X * menuScale;
                menuTextsWidth = j > menuTextsWidth ? j : menuTextsWidth; 
            }

            float menuItemsWidth = spriteFont.MeasureString(difficultyOptions[2]).X * menuScale;

            //for (int i = 0; i < menuItems.Length; i++)
            //{
            //    float j = spriteFont.MeasureString(menuItems[i]).X * menuScale;
            //    menuItemsWidth = j > menuItemsWidth ? j : menuItemsWidth;
            //}

            float menuWidth = menuTextsWidth + spaceMenuPadding + menuItemsWidth;

            float xPos = (Game.Window.ClientBounds.Width - menuWidth) / 2;
            float yPos = Game.Window.ClientBounds.Height * 0.2f;

            float menuItemsHeight = spriteFont.MeasureString(menuTexts[0]).Y * menuScale;
            float textsYpos = yPos;

            for (int i = 0; i < menuTexts.Length; i++)
            {
                spriteBatch.DrawString(spriteFont, menuTexts[i], new Vector2(xPos, textsYpos), textsColor, 0, Vector2.Zero, menuScale, SpriteEffects.None, 0);
                textsYpos += spaceBetweenMenuItems + menuItemsHeight;
            }

            float menuItemsXpos = xPos + menuTextsWidth + spaceMenuPadding;
            float menuItemsYpos = yPos;

            //spriteBatch.DrawString(spriteFont, numberOfPlayers.ToString(), new Vector2(menuItemsXpos, menuItemsYpos), textsColor, 0, Vector2.Zero, menuScale, SpriteEffects.None, 0);

            //menuItemsYpos += spaceBetweenMenuItems + menuItemsHeight;

            for (int i = 0; i < menuItems.Length; i++)
            {
                //Color itemColor = i == currentMenuIndex ? menuItemsSelectedColor : menuItemsColor;
                float scale = (i >= menuTexts.Length) ? buttonScale : menuScale;

                spriteBatch.DrawString(spriteFont, menuItems[i], new Vector2(menuItemsXpos, menuItemsYpos), menuItemsColor, 0,
                    Vector2.Zero, scale, SpriteEffects.None, 0);

                if (i == menuTexts.Length - 1)
                {
                    menuItemsYpos += spaceBetweenMenuItems * 2 + menuItemsHeight;
                }
                else
                {
                    menuItemsYpos += spaceBetweenMenuItems + menuItemsHeight;
                }
            }

            for (int i = 0; i < 4; i++)
            {
                string playerString = "Player " + (i + 1).ToString() + ": ";

                if (i < joinedPlayers.Count)
                {
                    playerString += "Joined, press ";
                    int type = (int)joinedPlayers[i];
                    if (type < 2)
                        playerString += Keys.Enter;
                    else
                        playerString += "GamePad " + Buttons.A;
                    playerString += " to start the game.";
                }
                else if (i == joinedPlayers.Count)
                {
                    playerString += "Press ";
                    bool othersleft = false;
                    if (!joinedPlayers.Contains(ControllerType.Keyboard1))
                    {
                        othersleft = true;
                        playerString += controls.Controls[0, 4];
                        if (!joinedPlayers.Contains(ControllerType.Keyboard2))
                            playerString += ", " + controls.Controls[1, 4];
                    } else if(!joinedPlayers.Contains(ControllerType.Keyboard2)) {
                        playerString += controls.Controls[1, 4];
                        othersleft = true;
                    }
                    if (othersleft)
                        playerString += " or ";
                    playerString += "GamePad A";
                    playerString += " to join.";
                }
                else
                {
                    playerString += " - ";
                }
                spriteBatch.DrawString(spriteFont, playerString, new Vector2(xPos, menuItemsYpos), textsColor, 0, Vector2.Zero, menuScale, SpriteEffects.None, 0);
                menuItemsYpos += spaceBetweenPlayers + menuItemsHeight;
            }

        }
    }
}
