﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using GameEngine.LevelGeneration;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;
using GameEngine.Helpers;
using GameEngine;
using GameEngine.Xml;

namespace GamePrototype.GUI.Screens
{
    public class StatusScreen : GameScreen
    {
        private const int NUMBER_OF_LEVELS = 3;
        private readonly string[] LEVEL_TITLES = { "Level 1: The Beginning", "Level 2: The Center of Action", "Level 3: The Final Battle" };

        private const string END_TEXT = "The End...";
        private const string END_END_TEXT = "Thank you for playing. Press Escape to return to main menu.";
        private readonly string[] GAME_COMPLETED_TEXT = { "You completed the game. Congratulations!",
                                                          "This game was made as a university semester",
                                                          "project by group sp1e082, Department of ",
                                                          "Computer Science, Aalborg University, fall 2008.",
                                                          "",
                                                          "- Jacob, Joergen, Jan, Bo and Dan."};

        private readonly string[,] LEVEL_TEXTS = {  // Level 1
                                                    {"Something went wrong. Very wrong. You must get out of here. Quick.",                                          
                                                     "Use the experimental radar to guide you towards the exit.", 
                                                     "Harmless looking things never are... You have been partly warned."},
                                                    // Level 2
                                                    {"To prevent the zombies from reaching the surface, the", 
                                                     "underground facility has been put under lock-down. You need to",
                                                     "hurry, as there is more ground to cover on this level."},
                                                    // Level 3
                                                    {"The exit is near! You only need to get to the elevator at this level", 
                                                     "to escape this place for good.", 
                                                     "Hopefully the zombies have not made it to the surface yet"}
                                                 };

        //

                                                     // Level 1
                                                    

                                                     
                                                     // Level 2
                                                    

                                                    // Level 3
                                                    


        //






        private const string CONTINUE_TEXT = "Press Enter or GamePad StartButton to Continue...";
        private const string MAP_HELP_PLAYER = "You are here";
        private const string MAP_HELP_EXITS = "Possible exits";
        private const float MAP_HELP_SCALE = 1.0f;
        private const float MAP_HELP_PADDING = 50f;
        private const int MAP_ARROWS_WIDTH = 1;
        private readonly Color MAP_HELP_COLOR = Color.White;

        private const float CONTINUE_TEXT_SCALE = 1.5f;
        private const float LEVEL_TITLE_SCALE = 1.8f;
        private const float LEVEL_TEXT_SCALE = 1.1f;
        private const float LEVEL_TEXT_TOP_PADDING = 60f;
        private const float SPACE_FROM_TITLE_TO_MAP = 40f;
        private const float SPACE_FROM_MAP_TO_TEXTS = 40f;
        private const float SPACE_BETWEEN_TEXTS = 10f;
        private const float SPACE_FROM_TEXTS_TO_CONTINUE = 45f;
                
        private readonly Color TEXTS_COLOR = Color.White;
        private readonly Color LEVEL_CONTINUE_COLOR = Color.CornflowerBlue;


        
        private readonly Color BACKGROUND_COLOR = Color.Black; 
        private const string TILESET_FILE = @"Tilesets\Tilesets";
        private const int MAP_SIZE_FACTOR = 15; // Level 1 = 25, level 2 = 40, level 3 = 55
        private const float MAP_DEVIATION_FACTOR = 0.10f;
        
        public int Difficulty { get; set; } // 1 = Easy, 2 = Normal, 3 = Hard
        public int PlayerCount { get; set; } // 1 = One player, 2 = Two players
        public int CurrentLevel { get; set; }

        public ControllerType[] JoinedPlayers { get; set; }

        private Texture2D texture;
        private LevelGenerator levelGenerator;
        private bool showLoading = false;
        private bool hasShowedLoading = false;
                
        public StatusScreen(string id, ScreenController controller, Game game)
            : base(id, controller, game)
        {
            
        }

        public void Prepare()
        {
            showLoading = false;
            hasShowedLoading = false;

            LoadTileset(content);

            int averageTiles = (CurrentLevel + 1) * MAP_SIZE_FACTOR + 10;
            int allowedDeviation = (int)(averageTiles * MAP_DEVIATION_FACTOR);

            this.levelGenerator = new LevelGenerator(averageTiles, allowedDeviation);

            int n = 12;
            int brokenLevelGenerator = 0;

            while (!levelGenerator.PositionPoints(n))
            {
                levelGenerator.ConstructLevel(averageTiles, allowedDeviation);
                brokenLevelGenerator++;

                if (brokenLevelGenerator > 1000)
                {
                    throw new Exception("LevelGenerator has reconstructed " + brokenLevelGenerator + " times now. It must be broken.");
                }
            }
            
            // Remove all but first and the 2 last elements in points of interest.
            for (int i = levelGenerator.PointsOfInterest.Count - 3; i >= 1; i--)
            {
                levelGenerator.PointsOfInterest[i].IsPointOfInterest = false;
                levelGenerator.PointsOfInterest.RemoveAt(i);
            }

            levelGenerator.PointsOfInterest[0].SpecialType = SpecialTileNodeType.Start;
            levelGenerator.PointsOfInterest[2].SpecialType = SpecialTileNodeType.End;

        }

        public static void LoadTileset(ContentManager content)
        {
            TilesetWrapper wrapper = content.Load<TilesetWrapper>(TILESET_FILE);
            TilesetManager.TileSets.AddRange(XmlTest.LoadTest(wrapper.Tilesets).Where(x => x is TilesetNode).Cast<TilesetNode>());
            TilesetManager.Tiles = wrapper.Tiles;
            TilesetManager.SpecialTiles = wrapper.SpecialTiles;


            
            foreach (var item in TilesetManager.TileSets)
            {
                if (item.Name == "Laboratory")
                    TilesetManager.CurrentTileset = item;
            }                       
        }

        public override void LoadContent(GraphicsDeviceManager graphics)
        {
            texture = new Texture2D(graphics.GraphicsDevice, 1, 1, 1, TextureUsage.None, SurfaceFormat.Vector4);
            texture.SetData(new Vector4[] { Vector4.One });
        }

        public override void Initialize(GraphicsDevice graphics)
        {
            

        }

        public override void Update(GameTime gameTime)
        {
            if (InputState.KeyPressed(Keys.Escape) && CurrentLevel >= NUMBER_OF_LEVELS)
            {
                this.controller.SetHiddenScreen("StatusScreen");
                this.controller.SetActiveScreen("MainMenuScreen"); 
            }

            if (InputState.KeyPressed(Keys.Enter) && InputState.IsKeyUp(Keys.LeftAlt) || InputState.ButtonPressedOnAnyGamePad(Buttons.Start))
            {
                showLoading = true;           
            }

            if (hasShowedLoading)
            {
                ConfigureControlsScreen configureControlsScreen =
                    (ConfigureControlsScreen)this.controller.GetScreen("ConfigureControlsScreen");
                configureControlsScreen.LoadPlayerControls();

                InGameScreen inGameScreen =
                    (InGameScreen)this.controller.GetScreen("InGameScreen");

                inGameScreen.Controls = configureControlsScreen.GetControls();
                inGameScreen.PlayerCount = this.PlayerCount;
                inGameScreen.JoinedPlayers = this.JoinedPlayers;
                inGameScreen.Difficulty = this.Difficulty;
                inGameScreen.CurrentLevel = this.CurrentLevel;
                inGameScreen.PrepareLevel(this.levelGenerator);

                this.controller.SetHiddenScreen("StatusScreen");
                this.controller.SetActiveScreen("InGameScreen");
                showLoading = false;
                hasShowedLoading = false;
                AudioManager.StopMusic();
            }
        }

        public override void Draw(GameTime gameTime, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            graphics.Clear(BACKGROUND_COLOR);

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);

            if (!showLoading)
            {
                if (CurrentLevel < NUMBER_OF_LEVELS)
                {
                    float a = DrawLevelTitle(graphics, spriteBatch, spriteFont);
                    float b = DrawLevelMap(a, graphics, spriteBatch, spriteFont);
                    float c = DrawLevelTexts(b, graphics, spriteBatch, spriteFont);
                    DrawContinueText(c, graphics, spriteBatch, spriteFont);
                }
                else
                {
                    DrawCompletedText(graphics, spriteBatch, spriteFont);
                }    
            }
            else
            {
                float scale = 2.0f;
                string loadingText = "Loading Game Data...";

                float loadingX = GUIhelper.GetHorisontalCenter(loadingText, spriteFont, this.Game, scale);
                float loadingY = GUIhelper.GetVerticalCenter(loadingText, spriteFont, this.Game, scale);
                                
                spriteBatch.DrawString(spriteFont, loadingText, new Vector2(loadingX, loadingY), Color.YellowGreen, 0,
                Vector2.Zero, scale, SpriteEffects.None, 0);

                hasShowedLoading = true;
            }            
            
            spriteBatch.End();
        }

        private float DrawLevelTitle(GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            string levelTitle = LEVEL_TITLES[CurrentLevel];
            float xPos = GUIhelper.GetHorisontalCenter(levelTitle, spriteFont, this.Game, LEVEL_TITLE_SCALE);

            spriteBatch.DrawString(spriteFont, levelTitle, new Vector2(xPos, LEVEL_TEXT_TOP_PADDING), LEVEL_CONTINUE_COLOR, 0, 
                Vector2.Zero, LEVEL_TITLE_SCALE, SpriteEffects.None, 0);

            float height = spriteFont.MeasureString(levelTitle).Y * LEVEL_TITLE_SCALE;

            return LEVEL_TEXT_TOP_PADDING + height;
        }

        private int DrawLevelMap(float yPos, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            int crap1, crap2;
            LevelTile[,] level = levelGenerator.GetLevelTileArray(out crap1, out crap2);
            
            yPos += SPACE_FROM_TITLE_TO_MAP;
            
            int tileSize = 10;

            if (levelGenerator.CreatedTiles.Count < 60)
            {
                tileSize = 16;
            }
            if (levelGenerator.CreatedTiles.Count < 30)
            {
                tileSize = 22;
            }


            int mapWidth = level.GetLength(0) * tileSize;
            int mapHeight = level.GetLength(1) * tileSize;

            int mapXpos = (Game.Window.ClientBounds.Width / 2) - (mapWidth / 2);
            int mapYpos = (int)yPos;



            Point player = Point.Zero;
            List<Point> exits = new List<Point>();
           
            for (int i = 0; i < level.GetLength(0); i++)
            {
                for (int j = 0; j < level.GetLength(1); j++)
                {
                    Color c = Color.Gray; // empty tiles

                    if (level[i, j] != null)
                    {
                        if (level[i, j] == levelGenerator.PointsOfInterest[0])
                        {
                            c = Color.Red; // Start
                            player = new Point(i, j);
                        }
                        else if (level[i, j] == levelGenerator.PointsOfInterest[1] || level[i, j] == levelGenerator.PointsOfInterest[2])
                        {
                            c = Color.Green; // Possible ends
                            exits.Add(new Point(i,j));
                        }
                        else
                        {
                            c = Color.White;
                        }                       
                    }

                    spriteBatch.Draw(texture, new Rectangle
                       (i * tileSize + mapXpos, 
                        j * tileSize + mapYpos, 
                        tileSize, 
                        tileSize), c);
                }
            }

            bool drawPlayerOnLeftSide = (player.X < level.GetLength(0) / 2) ? true : false;
            float playerWidth = spriteFont.MeasureString(MAP_HELP_PLAYER).X * MAP_HELP_SCALE;
            float exitsWidth = spriteFont.MeasureString(MAP_HELP_EXITS).X * MAP_HELP_SCALE;

            float helpYpos = yPos + (mapHeight / 2);

            float playerXpos = (drawPlayerOnLeftSide) ? mapXpos - MAP_HELP_PADDING - playerWidth : mapXpos + mapWidth + MAP_HELP_PADDING;
            float exitsXpos = (!drawPlayerOnLeftSide) ? mapXpos - MAP_HELP_PADDING - exitsWidth : mapXpos + mapWidth + MAP_HELP_PADDING;

            spriteBatch.DrawString(spriteFont, MAP_HELP_PLAYER, new Vector2(playerXpos, helpYpos), MAP_HELP_COLOR, 0, Vector2.Zero, MAP_HELP_SCALE, SpriteEffects.None, 0);
            spriteBatch.DrawString(spriteFont, MAP_HELP_EXITS, new Vector2(exitsXpos, helpYpos), MAP_HELP_COLOR, 0, Vector2.Zero, MAP_HELP_SCALE, SpriteEffects.None, 0);

            // Player
            Vector2 playerArrowStart = (drawPlayerOnLeftSide) ? new Vector2(mapXpos - (MAP_HELP_PADDING * 0.75f) - 2, helpYpos + 10) : new Vector2(mapXpos + mapWidth + (MAP_HELP_PADDING * 0.75f), helpYpos + 10);
            Vector2 playerField = 
                new Vector2(mapXpos + (player.X * tileSize) + (tileSize / 2),
                            mapYpos + (player.Y * tileSize) + (tileSize / 2));

            Vector2 playerDirection = Vector2.Subtract(playerField, playerArrowStart);
            Rectangle playerArrow = new Rectangle((int)playerArrowStart.X, (int)playerArrowStart.Y, (int)playerDirection.Length(), MAP_ARROWS_WIDTH); 
            
            // Exits
            Vector2 exitsArrowStart = (!drawPlayerOnLeftSide) ? new Vector2(mapXpos - (MAP_HELP_PADDING * 0.75f) - 2, helpYpos + 10) : new Vector2(mapXpos + mapWidth + (MAP_HELP_PADDING * 0.75f), helpYpos + 10);
            Vector2 exit1 = new Vector2(mapXpos + (exits[0].X * tileSize) + (tileSize / 2), mapYpos + (exits[0].Y * tileSize) + (tileSize / 2));
            Vector2 exit2 = new Vector2(mapXpos + (exits[1].X * tileSize) + (tileSize / 2), mapYpos + (exits[1].Y * tileSize) + (tileSize / 2));

            Vector2 exit1Dir = Vector2.Subtract(exit1, exitsArrowStart);
            Vector2 exit2Dir = Vector2.Subtract(exit2, exitsArrowStart);

            Rectangle exit1Arrow = new Rectangle((int)exitsArrowStart.X, (int)exitsArrowStart.Y, (int)exit1Dir.Length(), MAP_ARROWS_WIDTH);
            Rectangle exit2Arrow = new Rectangle((int)exitsArrowStart.X, (int)exitsArrowStart.Y, (int)exit2Dir.Length(), MAP_ARROWS_WIDTH);
            
            spriteBatch.Draw(texture, playerArrow, null, MAP_HELP_COLOR, (float)(Math.Atan2(playerDirection.Y, playerDirection.X)), Vector2.Zero, SpriteEffects.None, 0);
            spriteBatch.Draw(texture, exit1Arrow, null, MAP_HELP_COLOR, (float)(Math.Atan2(exit1Dir.Y, exit1Dir.X)), Vector2.Zero, SpriteEffects.None, 0);
            spriteBatch.Draw(texture, exit2Arrow, null, MAP_HELP_COLOR, (float)(Math.Atan2(exit2Dir.Y, exit2Dir.X)), Vector2.Zero, SpriteEffects.None, 0);    
                        
            return mapYpos + mapHeight;
        }

        private float DrawLevelTexts(float yPos, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            yPos += SPACE_FROM_MAP_TO_TEXTS;
            float textWidth = spriteFont.MeasureString(LEVEL_TEXTS[CurrentLevel, 0]).X * LEVEL_TEXT_SCALE; 
            float textHeight = spriteFont.MeasureString(LEVEL_TEXTS[CurrentLevel, 0]).Y * LEVEL_TEXT_SCALE;
            float xPos = (Game.Window.ClientBounds.Width - textWidth) / 2;

            for (int i = 0; i < LEVEL_TEXTS.GetLength(1); i++)
            {
                spriteBatch.DrawString(spriteFont, LEVEL_TEXTS[CurrentLevel, i], new Vector2(xPos, yPos), TEXTS_COLOR, 0,
                    Vector2.Zero, LEVEL_TEXT_SCALE, SpriteEffects.None, 0);

                yPos += textHeight + SPACE_BETWEEN_TEXTS; 
            }
            
            return yPos;
        }

        private void DrawContinueText(float yPos, GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            yPos += SPACE_FROM_TEXTS_TO_CONTINUE;

            spriteBatch.DrawString(spriteFont, CONTINUE_TEXT, 
                new Vector2(GUIhelper.GetHorisontalCenter(CONTINUE_TEXT, spriteFont, this.Game, CONTINUE_TEXT_SCALE), yPos), LEVEL_CONTINUE_COLOR, 0, 
                Vector2.Zero, CONTINUE_TEXT_SCALE, SpriteEffects.None, 0);
        }

        private void DrawCompletedText(GraphicsDevice graphics, SpriteBatch spriteBatch, SpriteFont spriteFont)
        {
            float titleScale = 1.8f;
            float textScale = 1.4f;
            float endScale = 1.6f;

            float yPos = Game.Window.ClientBounds.Height * 0.2f;

            spriteBatch.DrawString(spriteFont, END_TEXT,
                new Vector2(GUIhelper.GetHorisontalCenter(END_TEXT, spriteFont, this.Game, titleScale), yPos), Color.YellowGreen, 0,
                Vector2.Zero, titleScale, SpriteEffects.None, 0);

            yPos += 130;

            float xPos = GUIhelper.GetHorisontalCenter(GAME_COMPLETED_TEXT[0], spriteFont, Game, textScale);
            float textHeight = spriteFont.MeasureString(GAME_COMPLETED_TEXT[0]).Y * textScale;

            for (int i = 0; i < GAME_COMPLETED_TEXT.Length; i++)
            {
                spriteBatch.DrawString(spriteFont, GAME_COMPLETED_TEXT[i],
                new Vector2(xPos, yPos), Color.YellowGreen, 0, Vector2.Zero, textScale, SpriteEffects.None, 0);

                yPos += textHeight + 6f;
            }

            yPos += 40;

            spriteBatch.DrawString(spriteFont, END_END_TEXT,
                new Vector2(GUIhelper.GetHorisontalCenter(END_END_TEXT, spriteFont, this.Game, endScale), yPos), Color.YellowGreen, 0,
                Vector2.Zero, endScale, SpriteEffects.None, 0);
        }
    }
}