﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameEngine.ScreenSystem;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using GameEngine;

namespace GamePrototype.GameLogic
{
    public enum AmmoType
    {
        Heavy,
        Light
    }

    public enum WeaponType
    {
        M4,
        Beretta,
        MiniShotgun,
        Garand,
        Uzi,
        Minigun
    }

    public static class WeaponFactory
    {

        public static Dictionary<WeaponType, Texture2D> WhiteTextures { get; set; }
        public static Dictionary<WeaponType, Texture2D> BlackTextures { get; set; }


        public static void LoadAllWeapons(ContentManager content)
        {
            WhiteTextures = new Dictionary<WeaponType, Texture2D>();
            BlackTextures = new Dictionary<WeaponType, Texture2D>();

            BlackTextures.Add(WeaponType.M4, content.Load<Texture2D>("HUD/Weapons/m4black"));
            WhiteTextures.Add(WeaponType.M4, content.Load<Texture2D>("HUD/Weapons/m4white"));

            BlackTextures.Add(WeaponType.Beretta, content.Load<Texture2D>("HUD/Weapons/berettablack"));
            WhiteTextures.Add(WeaponType.Beretta, content.Load<Texture2D>("HUD/Weapons/berettawhite"));

            BlackTextures.Add(WeaponType.MiniShotgun, content.Load<Texture2D>("HUD/Weapons/shotgunblack"));
            WhiteTextures.Add(WeaponType.MiniShotgun, content.Load<Texture2D>("HUD/Weapons/shotgunwhite"));

            BlackTextures.Add(WeaponType.Minigun, content.Load<Texture2D>("HUD/Weapons/minigunblack"));
            WhiteTextures.Add(WeaponType.Minigun, content.Load<Texture2D>("HUD/Weapons/minigunwhite"));

            BlackTextures.Add(WeaponType.Garand, content.Load<Texture2D>("HUD/Weapons/garandblack"));
            WhiteTextures.Add(WeaponType.Garand, content.Load<Texture2D>("HUD/Weapons/garandwhite"));
         
            BlackTextures.Add(WeaponType.Uzi, content.Load<Texture2D>("HUD/Weapons/uziblack"));
            WhiteTextures.Add(WeaponType.Uzi, content.Load<Texture2D>("HUD/Weapons/uziwhite"));

            string FireSound, FireDrySound, ReloadSound;

            FireSound = "Audio/Weapons/mgfire"; // Firing sound
            FireDrySound = "Audio/Weapons/mgempty"; // "Clicking sound" for when the weapon is empty
            ReloadSound = "Audio/Weapons/mgreload"; // Reloading sound
            AudioManager.LoadSoundEffect(content, FireSound, FireDrySound, ReloadSound);

            FireSound = "Audio/Weapons/deserteaglefire";
            FireDrySound = "Audio/Weapons/deserteagleempty";
            ReloadSound = "Audio/Weapons/deserteaglereload";
            AudioManager.LoadSoundEffect(content, FireSound, FireDrySound, ReloadSound);

            FireSound = "Audio/Weapons/minishotgunfire";
            FireDrySound = "Audio/Weapons/minishotgunempty";
            ReloadSound = "Audio/Weapons/minishotgunreload";
            AudioManager.LoadSoundEffect(content, FireSound, FireDrySound, ReloadSound);

            FireSound = "Audio/Weapons/garandfire";
            FireDrySound = "Audio/Weapons/garandempty";
            ReloadSound = "Audio/Weapons/mgreload";
            AudioManager.LoadSoundEffect(content, FireSound, FireDrySound, ReloadSound);

            FireSound = "Audio/Weapons/mgfire";
            FireDrySound = "Audio/Weapons/mgempty";
            ReloadSound = "Audio/Weapons/mgreload";
            AudioManager.LoadSoundEffect(content, FireSound, FireDrySound, ReloadSound);

            FireSound = "Audio/Weapons/minigunfire";
            FireDrySound = "Audio/Weapons/mgempty";
            ReloadSound = "Audio/Weapons/mgreload";
            AudioManager.LoadSoundEffect(content, FireSound, FireDrySound, ReloadSound);
        }



        public static bool IsSmallWeapon(WeaponType type)
        {
            return (type == WeaponType.Beretta || type == WeaponType.MiniShotgun || type == WeaponType.Uzi);
        }


        public static Weapon GetWeapon(WeaponType type)
        {
            switch (type)
            {
                case WeaponType.M4:

                    Weapon m4 = new Weapon(WeaponType.M4);

                    m4.AmmoType = AmmoType.Heavy;
                    m4.SemiAutomatic = false; // Semi or full automatic
                    m4.Damage = 25; // Damage to deal to target.
                    m4.RateOfFire = 75; // Milliseconds between shots when holding fire key down (if not semi automatic) 
                    m4.ReloadTime = 1400; // Reloading time in milliseconds 
                    m4.ClipSize = 30; // Number of bullets in a full clip 
                    m4.Penetration = 2; // Number of possible targets to hit with one shot
                    m4.AllowedDeviation = 0.05f; // Radians. The shooting ray is allowed this deviation.
                    m4.ProjectileCount = 1; // Number of shooting rays for a shot (Remember to set allowedDeviation to a higher value, if this value is changed).
                    m4.FireSound = "Audio/Weapons/mgfire"; // Firing sound
                    m4.FireDrySound = "Audio/Weapons/mgempty"; // "Clicking sound" for when the weapon is empty
                    m4.ReloadSound = "Audio/Weapons/mgreload"; // Reloading sound

                    m4.PrepareWeapon(); // Set up starting ammo and load sounds effect, etc

                    return m4;
 
                case WeaponType.Beretta:

                    Weapon beretta = new Weapon(WeaponType.Beretta);

                    beretta.AmmoType = AmmoType.Light;
                    beretta.SemiAutomatic = true; 
                    beretta.Damage = 50; 
                    beretta.RateOfFire = 50;
                    beretta.ReloadTime = 2000; 
                    beretta.ClipSize = 12; 
                    beretta.Penetration = 1; 
                    beretta.AllowedDeviation = 0.05f; 
                    beretta.ProjectileCount = 1;
                    beretta.FireSound = "Audio/Weapons/deserteaglefire"; 
                    beretta.FireDrySound = "Audio/Weapons/deserteagleempty"; 
                    beretta.ReloadSound = "Audio/Weapons/deserteaglereload"; 

                    beretta.PrepareWeapon(); 

                    return beretta;                                        

                case WeaponType.MiniShotgun:


                    Weapon minishotgun = new Weapon(WeaponType.MiniShotgun);

                    minishotgun.AmmoType = AmmoType.Light;
                    minishotgun.SemiAutomatic = true;
                    minishotgun.Damage = 185;
                    minishotgun.RateOfFire = 1000;
                    minishotgun.ReloadTime = 1600;
                    minishotgun.ClipSize = 7;
                    minishotgun.Penetration = 3;
                    minishotgun.AllowedDeviation = 0.05f;
                    minishotgun.ProjectileCount = 1;
                    minishotgun.FireSound = "Audio/Weapons/minishotgunfire";
                    minishotgun.FireDrySound = "Audio/Weapons/minishotgunempty";
                    minishotgun.ReloadSound = "Audio/Weapons/minishotgunreload";

                    minishotgun.PrepareWeapon();

                    return minishotgun;

                case WeaponType.Garand:


                    Weapon garand = new Weapon(WeaponType.Garand);

                    garand.AmmoType = AmmoType.Heavy;
                    garand.SemiAutomatic = true;
                    garand.Damage = 60;
                    garand.RateOfFire = 50;
                    garand.ReloadTime = 1400;
                    garand.ClipSize = 8;
                    garand.Penetration = 2;
                    garand.AllowedDeviation = 0.05f;
                    garand.ProjectileCount = 1;
                    garand.FireSound = "Audio/Weapons/garandfire";
                    garand.FireDrySound = "Audio/Weapons/garandempty";
                    garand.ReloadSound = "Audio/Weapons/mgreload";

                    garand.PrepareWeapon();

                    return garand;



                case WeaponType.Uzi:


                    Weapon uzi = new Weapon(WeaponType.Uzi);

                    uzi.AmmoType = AmmoType.Light;
                    uzi.SemiAutomatic = false;
                    uzi.Damage = 12;
                    uzi.RateOfFire = 40;
                    uzi.ReloadTime = 1500;
                    uzi.ClipSize = 40;
                    uzi.Penetration = 1;
                    uzi.AllowedDeviation = 0.05f;
                    uzi.ProjectileCount = 1;
                    uzi.FireSound = "Audio/Weapons/mgfire";
                    uzi.FireDrySound = "Audio/Weapons/mgempty";
                    uzi.ReloadSound = "Audio/Weapons/mgreload";

                    uzi.PrepareWeapon();

                    return uzi;


                case WeaponType.Minigun:


                    Weapon minigun = new Weapon(WeaponType.Minigun);

                    minigun.AmmoType = AmmoType.Heavy; 
                    minigun.SemiAutomatic = false;
                    minigun.Damage = 16;
                    minigun.RateOfFire = 15;
                    minigun.ReloadTime = 3000;
                    minigun.ClipSize = 200;
                    minigun.Penetration = 1;
                    minigun.AllowedDeviation = 0.05f;
                    minigun.ProjectileCount = 1;
                    minigun.FireSound = "Audio/Weapons/minigunfire";
                    minigun.FireDrySound = "Audio/Weapons/mgempty";
                    minigun.ReloadSound = "Audio/Weapons/mgreload";

                    minigun.PrepareWeapon();

                    return minigun;


                default:
                    break;
            }
            
            return null;
        }

        public static Weapon[] GetRandomDifferentWeaponsForAGroupOfPlayersBasedOnDifficultySettings(int numberOfPlayers)
        {
            Weapon[] result = new Weapon[numberOfPlayers];

            WeaponType[] possibleTypes = (WeaponType[])Enum.GetValues(typeof(WeaponType));
            int[] possibleWeapons = new int[possibleTypes.Length];

            for (int i = 0; i < possibleWeapons.Length; i++)
            {
                possibleWeapons[i] = i;
            }

            int n = possibleWeapons.Length;

            while (n > 1)
            {
                int k = Util.Random.Next(n);
                n--;
                int temp = possibleWeapons[n];
                possibleWeapons[n] = possibleWeapons[k];
                possibleWeapons[k] = temp;
            }

            for (int i = 0; i < numberOfPlayers; i++)
            {
                result[i] = WeaponFactory.GetWeapon(possibleTypes[possibleWeapons[i]]);
            }

            return result;
        }
    }
}
